define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/store/Memory",
    "dojo/Stateful",
    "dojo/Evented",
    "thornberry/serverRequest"
],function(declare,lang,arrayUtils,Memory,Stateful,Evented,serverRequest) {
    var getFirst = function(arrayVal) {
        var first = null;
        if (arrayVal && arrayVal.length > 0) {
            first = arrayVal[0];
        }
        return first;
    };

    var PagePreset = declare([Stateful],{
        id: "",
        name: "System Default",
        description: "This is the default system configuration",
        system: true,
        def: true,
        filters: null, // object
        columns: null, // array
        sort: null, // object
        summarizeBy: "None",
        aggregations: null, // array
        options: null, // object,

        modified: false,

        postscript: function() {
            var t = this;
            t.inherited(arguments);

            if (t.filters === null) {
                t.filters = {};
            }
            if (t.columns === null) {
                t.columns = [];
            }
            if (t.aggregations === null || t.aggregations.length === 0) {
                t.aggregations = ["_COUNT"];
            }
            if (t.options === null) {
                t.options = {};
            }
        },

        save: function(userId,providerClass) {
            var t = this;

            var presetData = {
                id: t.id,
                name: t.name,
                description: t.description,
                system: t.system,
                def: t.def,
                filters: t.filters,
                columns: t.columns,
                sort: t.sort,
                summarizeBy: t.summarizeBy,
                aggregations: t.aggregations,
                options: t.options,
                userId: userId
            }

            return serverRequest(providerClass,"SavePreset",presetData).then(function(id) {
                t.id = id;
                t.modified = false;
            });
        },

        getName: function() {
            var t = this;
            return t.name+(t.modified ? "*" : "");
        }
    });

    var PresetManager = declare([Stateful,Evented], {
        userId: "",
        presetData: null,
        providerClass: "",

        store: null,
        currentPreset: null,

        constructor: function(args) {
            var t = this;
            t.store = new Memory();
            lang.mixin(t,args);
        },
        
        getDefault: function() {
            return getFirst(this.store.query({system:false,def:true})) || getFirst(this.store.query({system:true,def:true}));
        },

        getPresetsSorted: function() {
            return this.store.query({},{sort:[{attribute:"def",descending:true},{attribute:"name"}]});
        },

        saveMyDefault: function() {
            var t = this;
            t.currentPreset.name = "My Default";
            t.currentPreset.description = "This is your default configuation";
            t.currentPreset.system = false;
            t.currentPreset.def = true;
            t.currentPreset.save(t.userId,t.providerClass).then(function() {
                var existingMyDefault = getFirst(t.store.query({system:false,def:true}));
                if (existingMyDefault) {
                    t.store.remove(existingMyDefault.id);
                }
                t.store.put(lang.clone(t.currentPreset));
                t.currentPreset.id = "";
                t.emit("changePresets");
            },actionShowMessage);
        },

        copyToMyDefault: function() {
            var t = this;
            t.currentPreset.name = "My Default";
            t.currentPreset.description = "This is your default configuation";
            t.currentPreset.system = false;
            t.currentPreset.def = true;
            t.currentPreset.modified = true;
            t.emit("setPreset",t.currentPreset);
        },

        _presetDataSetter: function(presetData) {
            var t = this;
            t.presetData = presetData;

            var presets = arrayUtils.map(presetData,function(preset) {
                return new PagePreset(preset);
            });
            t.store.setData(presets);
            t.set("currentPreset",t.getDefault());
        },

        _currentPresetSetter: function(preset) {
            var t = this;
            if (preset === null) {
                preset = new PagePreset();
                t.store.put(preset);
            }
            t.currentPreset = lang.clone(preset);
            t.currentPreset.id = "";
            t.emit("setPreset",t.currentPreset);
        }
    });
	PresetManager.PagePreset = PagePreset;
	return PresetManager;
});