define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/json",
	"dojo/dom-construct",
	"thornberry/app/Component/_Component",
	"thornberry/app/Component/_ViewMixin",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./Templates/Dashboard.html",
	"thornberry/app/service/AccountsReceivable",
	"thornberry/app/utils",
	"thornberry/app/Component/ScoreCard",
	"dojo/currency"
],function(
	declare,
	lang,
	arrayUtils,
	json,
	domConstruct,
	_Component,
	_ViewMixin,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	template,
	service,
	utils,
	ScoreCard,
	currency
) {
	return declare([_Component,_ViewMixin,_TemplatedMixin,_WidgetsInTemplateMixin],{
		title: "Dashboard",

		templateString: template,

		data: null,

		getPageProps: function() {
			var t = this;
			return lang.mixin(t.inherited(arguments),{
				content: t.domNode,
				actions: [
					{ label: "Refresh", onClick: lang.hitch(t,"refreshData") }
				],
				helpButtonShown: true
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			var getNetDisplay = function(key,value) {
				if (key == "net") {
					value = (value === "" ? "$ - " : "$"+currency.format(value));
				}
				return value;
			};

			var countNetTemplate = 
				"<div class='flex-field-container'>"+
					"<div class='field-name field-name-grand'>${props.item.name}</div>"+
					"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.count}</div>"+
					"<div class='field-value field-value-medium' style='text-align:right'>${props.item.net}</div>"+
				"</div>";

			t.accountsCard = new ScoreCard({
				title: "Accounts",
				props: {
					rowTemplate: countNetTemplate,
					getDisplayValue: getNetDisplay
				}
			});
			t.cardContainerLeft.appendChild(t.accountsCard.domNode);
			t.cardContainerLeft.appendChild(domConstruct.toDom("<br>"));

			t.eraCard = new ScoreCard({
				title: "ERA",
				props: {
					rowTemplate: 
						"<div class='flex-field-container'>"+
						"<div class='field-name field-name-grand'>${props.item.name}</div>"+
						"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.count}</div>"+
					"</div>"
				}
			});
			t.cardContainerLeft.appendChild(t.eraCard.domNode);
			t.cardContainerLeft.appendChild(domConstruct.toDom("<br>"));

			t.financialPeriodCard = new ScoreCard({
				title: "Financial Period",
				props: {
					rowTemplate: 
						"<div class='flex-field-container'>"+
						"<div class='field-name field-name-grand'>${props.item.name}</div>"+
						"<div class='field-value field-value-medium' style='text-align:right'>${props.item.value}</div>"+
					"</div>"
				}
			});
			t.cardContainerLeft.appendChild(t.financialPeriodCard.domNode);

			t.arAgingCard = new ScoreCard({
				title: "A/R Aging",
				props: {
					rowTemplate:
					"<div class='flex-field-container'>"+
						"<div class='field-name field-name-grand'>${props.item.name}</div>"+
						"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.total}</div>"+
						"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.bucket1}</div>"+
						"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.bucket2}</div>"+
						"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.bucket3}</div>"+
						"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.bucket4}</div>"+
						"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.bucket5}</div>"+
					"</div>",
					getDisplayValue: function(key, value) {
						if ((key != "name") && (parseFloat(value) == value)) {
							value = (value === "" ? "$ - " : "$" + currency.format(value));
						}
						return value;
					}
				}
			});
			t.cardContainerRight.appendChild(t.arAgingCard.domNode);
			t.cardContainerRight.appendChild(domConstruct.toDom("<br>"));

			t.productivityCard = new ScoreCard({
				title: "Productivity",
				props: {
					//rowTemplate: countNetTemplate,
					//getDisplayValue: getNetDisplay
				}
			});
			t.cardContainerRight.appendChild(t.productivityCard.domNode);
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			t.accountsCard.set("props", {
				onClickTitle: function() { t.handleRowClick(1,"accounts"); },
				onClickRow: function(rowId) { t.handleRowClick(1,"accounts",rowId); }
			});
			t.eraCard.set("props", {
				onClickTitle: function() { t.handleRowClick(3,"era"); },
				onClickRow: function(rowId) { t.handleRowClick(3,"era",rowId); }
			});
			t.productivityCard.set("props", {
				//onClickTitle: function() { t.handleRowClick(1,"productivity"); },
				//onClickRow: function(rowId) { t.handleRowClick(1,"productivity",rowId); }
			});
		},

		refreshRendering: function() {
			var t = this;
			if (!t.data) {
				var promise = service.getDashboardData().then(function(data) {
					t.set("data",data);
				},utils.showMessage);
				t.addReadyPromise(promise);
				t.props.onSetLoadingPromise(promise);
			}
			t.inherited(arguments);
		},

		_setDataAttr: function(data) {
			var t = this;
			t.data = data;

			t.accountsCard.set({ props: { data: data.accounts } });
			t.eraCard.set({ props: { data: data.era } });
			t.financialPeriodCard.set({ props: { data: data.financialPeriod } });
			t.arAgingCard.set({ props: { data: data.arAging } });
			t.productivityCard.set({ props: { data: data.productivity } });
		},

		handleRowClick: function(viewIndex,key,id) {
			var t = this;
			var filter = null;
			if (id) {
				filter = arrayUtils.filter(t.data[key],function(item) {
					return item.id === id;
				})[0].filter;
			}

			var viewMap = {
				"accounts":			{ name: "Accounts", filter: "{}"},
				"era":				{ name: "Era", filter: "{}"},
				"financialPeriod":	{ name: "FinancialPeriod", filter: "{}"},
				"arAging":			{ name: "ArAging", filter: "{}"},
				"productivity":		{ name: "Productivity", filter: "{}"}
			};
			if (filter === null) {
				filter = viewMap[key].filter;
			}

			t.props.onSetViewFilters(viewMap[key].name,json.parse(filter));
			t.props.onSwitchView(viewIndex);
		},

		refreshData: function() {
			var t = this;
			t.data = null;
			if (t.isShown) { 
				t.refreshRendering();
			}
		}
	});
});