define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "./EraEditor",
    "thornberry/app/service/AccountsReceivableEra",
    "thornberry/app/utils"
],function(
	declare,
	lang,
	arrayUtils,
	_Component,
    _EditorViewMixin,
    Editor,
    service,
    utils
) {
	var canPostTransactions = false;
	utils.checkPermission("ARPostTransactions",false).then(function(allowed){canPostTransactions=!!allowed});
	
	return declare([_Component,_EditorViewMixin], {
		title: "ERA Details",

		EditorClass: Editor,

		lockCheckEnabled: true,
		objectType: "Billing.Era.Summary",

        objectId: null,
        summary: null,

        getTitle: function() {
            var t = this;
            return "ERA Details" + (t.summary && t.summary.fileName != "" ? " - " + t.summary.fileName : "");
        },

		getInitialProps: function() {
            var t = this;
			return lang.mixin(this.inherited(arguments), {
				mode: "edit", // view or edit
                onSave: null
			});
		},

		getPageProps: function() {
            var t = this;
            var canEdit = (t.summary && t.summary.isDeleted != 1 && t.summary.isProcessed != 1);

			var itemActions = [];
			if (canPostTransactions) {
				itemActions.push({ label: "Process", onClick: function() {t.startSave().then(t.props.onCloseView)}, hidden: !canEdit });
			}
			itemActions.push({ label: "Delete", onClick: lang.hitch(t,"deleteItem"), hidden: !canEdit });
			
			return lang.mixin(t.inherited(arguments), {
                actions: [
                    { label: "Back", onClick: t.props.onCloseView },
                    itemActions
                ],
				accountSummaryShown: false
			});
        },
        
        refreshRendering: function() {
            var t = this;
            t.set("objectId",t.props.objectId);
            t.inherited(arguments);
        },

		_setObjectIdAttr: function(value) {
			var t = this;
			if (value && t.objectId !== value) {
				t.addReadyPromise(service.get(value).then(function(summary) {
					if (t.objectId === value) {
						t.set("summary", summary);
					}
				}));
			} else if (!value) {
				t.set("summary", null);
			}
			t.objectId = value;
        },
        
        _setSummaryAttr: function(value) {
            var t = this;
            t.summary = value;
            t.editor.set({
                props: {
                    value: value
                }
            });
            t.setPageProps();
        },

        save: function() {
            var t = this;
            var data = lang.mixin(t.editor.get("value"),{
                id: t.props.objectId
            });
            return service.process(data).then(function() {
                t.objectId = null;
                if (t.props.onSave) {
                    t.props.onSave();
                }
            });
        },

        deleteItem: function() {
            var t = this;
            utils.showMessage("Are you sure you want to delete this ERA?",[
                { label: "Yes", callback: function() {
                    service.deleteItem(t.objectId).then(function() {
                        t.editor.modified = false;
                        t.props.onCloseView();
                        t.objectId = null;
                        if (t.props.onSave) {
                            t.props.onSave();
                        }
                    },utils.showMessage);
                } },{
                    label: "No"
                }
            ]);
        },

        onBeforeClose: function() {
            return true;
        }
	});
});