define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "thornberry/app/Component/Report/View",
    "thornberry/app/utils",
    "thornberry/app/Service/EnsembleFactory"
],function(
    declare,
    lang,
    ReportView,
    utils,
    ensembleFactory
) {
    return declare(ReportView,{
        title: "ERA",

        defaultToFilterView: false,
        showPresetControl: false,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.Era",
                filters: null
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            return lang.mixin(t.inherited(arguments),{
                viewTitle: "Electronic Remittance Advice",
                actions: [
                    { label: "Refresh", onClick: function() {t.controller.gridView.updateGridData(true);} },
                    { label: "Upload",     onClick: function() { t.loadSubView("uploadFileView"); } },
                    [
                        { label: "View", onClick: function() { t.loadSubView("eraEditor") }, hidden: selectedIds.length !== 1 }
                    ]
                ]
            });
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            // Force the report initialization to occur when the module preloads the view
            t.invalidateRendering();
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);
            if (t.firstShow) {
                t.addSubView("eraEditor","thornberry/app/Component/AccountsReceivable/EraEditorView",function() {
                    return {
                        objectId: t.props.selectedIds[0],
                        onSave: lang.hitch(t,"refreshData")
                    };
                });
                t.addSubView("uploadFileView","thornberry/app/Component/FileUploadView",function() {
                    return { 
                        title: "Upload ERA",
                        serverClass: "Billing.Era.Service",
                        serverMethod: "UploadFile",
                        onUpload: function(promise) {
                            t.getSubView("uploadFileView").props.onCloseView();
                            var service = ensembleFactory("Billing.Era.EnsembleResponse");
                            var promise = service(promise,120)
                            promise.then(function(eraSummaryId) {
                                t.loadSubView("eraEditor",{
                                    objectId: eraSummaryId
                                });
                                t.refreshData();
                            },utils.showMessage);
                            t.props.onSetLoadingPromise(promise,"Processing...");
                        }
                    };
                });
            }
        },

        onRowDoubleClick: function(id) {
            var t = this;
            t.loadSubView("eraEditor",{ objectId: id } );
        }
    });
});