define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/app/Component/_Component",
	"thornberry/app/Component/_EditorViewMixin",
	"./TransactionEditor",
	"thornberry/app/service/AccountsReceivable",
	"dojo/Deferred",
	"thornberry/app/utils",
    "dojo/when"
],function(
	declare,
	lang,
	arrayUtils,
	_Component,
	_EditorViewMixin,
	TransactionEditor,
	service,
	Deferred,
	utils,
	when
) {
	return declare([_Component,_EditorViewMixin], {
		title: "Transaction",

		EditorClass: TransactionEditor,

		lockCheckEnabled: true,
		objectType: "Billing.AccountsReceivable.Transaction",

		objectId: null,
		transaction: null,

		getTitle: function() {
			if (this.editor.props.value) {
				var patient = this.editor.props.value.patient;
				var claim = this.editor.props.value.claim;
				var description = patient.Name + " (" + patient.AccountId + ")";
				if (claim) {
					description += " - " + claim.typeName + " - " + claim.billPeriodDisplay;
				}
				var mode = (this.props.mode === "add" ? "Add" : "Edit");
				var transactionType = (this.props.transactionType == 2 ? "Payment" : "Adjustment");
				
				return mode + " " + transactionType + " - " + description;
			} else {
				return this.title;
			}
		},

		getInitialProps: function() {
			return lang.mixin(this.inherited(arguments), {
				mode: "edit", // view or edit
				onSave: null
			});
		},

		getPageProps: function() {
			var t = this;
			var props = lang.mixin(t.inherited(arguments), {
				accountSummaryShown: false
			});
			return props;
		},

		refreshRendering: function() {
			var t = this;
			if (t.isShown) {
				t.set("objectId", t.props.objectId);

				t.editor.set("props", {
					transactionType: t.props.transactionType,
				});
			}
			t.inherited(arguments);
		},

		_setObjectIdAttr: function(value) {
			var t = this;
			if ((value && t.objectId !== value) || this.props.accountId) {
				t.addReadyPromise(service.getTransaction({
					"id": value,
					"noChargeTotals": true,
					"accountId": this.props.accountId
				}).then(function(transaction) {
					if (t.objectId === value) {
						t.set("transaction", transaction);
					}
				}));
			} else if (!value) {
				t.set("transaction", null);
			}
			t.objectId = value;
		},

		_setTransactionAttr: function(value) {
			var t = this;
			t.transaction = value;
			t.editor.set("props", {
				value: value
			});
		},

		startSave: function() {
			var t = this;
			var deferred = new Deferred();
			if (arrayUtils.some(t.editor.chargeGrid.rows(), function(row) {
				return !row.grid.cellWidget.getCellWidget(row.id, "amount").amountField.isValid();
			})) {
				utils.showMessage("Please correct any invalid fields.");
				deferred.reject();
			} else {
				t.inherited(arguments).then(deferred.resolve, deferred.reject);
			}
			return deferred.promise;
		},

		save: function() {
			var t = this;
			var data = t.editor.getSaveData();
			return service.postTransaction(data).then(
				function() {
					t.objectId = null;
					if (t.props.onSave) {
						t.props.onSave();
					}
				});
		},

		resize: function() {
			var t = this;
			t.editor.resize();
		},

		onBeforeClose: function() {
			var t = this;
			var closeResult = t.inherited(arguments);
			when (closeResult, function(value) {
				if (value) {
					t.objectId = null;
				}
			});
			return closeResult;
		}
	});
});