define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/date",
    "dojo/promise/all",
    "dojo/store/Memory",
    "./LocationAdjustmentEditor",
    "thornberry/app/Service/LocationAdjustment",
    "thornberry/app/utils",
    "dijit/form/Button"
],function(
    declare,
    _Component,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    lang,
    arrayUtils,
    date,
    all,
    Memory,
    LocationAdjustmentEditor,
    service,
    utils
) {
    return declare([ _Component,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: "<div><div data-dojo-attach-point='containerNode'></div><input data-dojo-type='dijit/form/Button' data-dojo-props='label:\"Add\"' data-dojo-attach-event='onClick:_add' data-dojo-attach-point='addButton'></input></div>",

        editorStore: null,
        locationId: "",
        disabled: false,

        _lastEditorId: 0,
    
        constructor: function() {
            var t = this;
            t.editorStore = new Memory();
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                locationId: "",
                type: "" // QRP or HHVBP
            });
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.set("disabled",t.props.disabled);
            t.set("locationId",t.props.locationId);
        },

        _setLocationIdAttr: function(value) {
            var t = this;
            if (t.locationId != value) {
                t.locationId = value;

                // Use a service to get bill rates for this charge code. If none, then put one charge code editor in place
                t.removeAll();
                if (!!value && !t.disabled) {
                    t.addReadyPromise(service.get(t.props.type,t.props.locationId).then(function(locationAdjustments) {
                        if (locationAdjustments.length > 0) {
                            t.removeAll();
                            arrayUtils.forEach(locationAdjustments,t.add,t);
                        } else {
                            t._onUserUpdate();
                        }
                    }));
                } else {
                    t.removeAll();
                }
            }
        },

        _add: function() {
            this.add();
            this._onUserUpdate();
        },

        add: function(value) {
            var t = this;
            value = value || null;
            var id = ++t._lastEditorId;
            var editor = new LocationAdjustmentEditor({
                layout: t.props.type,
                props: {
                    value: value,
                    disabled: t.disabled,
                    onRemove: function() {
                        t.remove(id);
                        t.defer("_onUserUpdate");
                    },
                    onChange: function(value) {
                        t.onEditorChanged(id);
                    }
                }
            });
            t.editorStore.put({ id: id, editor: editor });

            editor.readyPromise.then(function() {
                t._updateMinDate(t.editorStore.data.length - 1);
                t.containerNode.appendChild(editor.domNode);
            });
        },

        removeAll: function(full) {
            var t = this;
            var ids = arrayUtils.map(t.editorStore.data,function(item) {return item.id;});
            arrayUtils.forEach(ids,function(id) {t.remove(id)});
        },

        remove: function(id) {
            var t = this;
            var item = t.editorStore.get(id);
            if (item) {
                var position = t.editorStore.index[id];
                item.editor.destroy();
                t.editorStore.remove(id);
                t._updateMinDate(position);
            }
        },

        onEditorChanged: function(id) {
            var t = this;
            var position = t.editorStore.index[id] + 1;
            t._updateMinDate(position);
            t._onUserUpdate();
        },

        _updateMinDate: function(position) {
            var t = this;
            if (position < t.editorStore.data.length) {
                var editor = t.editorStore.data[position].editor;
                var previousEditor = (position > 0 ? t.editorStore.data[position - 1].editor : null);
                var field = editor.getField("effectiveDate");
                var minDate = previousEditor && previousEditor.getField("effectiveDate").get("value");
                field.constraints.min = (minDate ? date.add(minDate,"day",1) : utils.validators.minDate);
                field.validate();
            }
        },

        _getValueAttr: function() {
            var t = this;
            return arrayUtils.map(t.editorStore.data,function(item) {
                return item.editor.get("value");
            });
        },

        _setDisabledAttr: function(value) {
            var t = this;
            t.disabled = value;
            t.addButton.set("disabled",value);
        },

        _onUserUpdate: function() {
            var t = this;
            var readyArray = arrayUtils.map(t.editorStore.data,function(item) { return item.editor.readyPromise; });
            all(readyArray).then(function() {
                service.setInSession({
                    locationAdjustments: t.get("value"),
                    status: t.get("status"),
                    type: t.props.type
                });
            });
        },

        _getStatusAttr: function() {
            var t = this;
            var status = "";
            if (arrayUtils.some(t.editorStore.data,function(item) { return !item.editor.checkValid(); })) {
                status = "invalid";
            } else if (arrayUtils.some(t.editorStore.data,function(item) { 
                var blank = true;
                if (!item.editor.checkRequired()) {
                    var value = item.editor.get("value");
                    for (var name in value) {
                        if (value[name] != "") {
                            blank = false;
                            break;
                        }
                    }
                }
                return !blank;
            })) {
                status = "incomplete";
            } else if (arrayUtils.some(t.editorStore.data,function(item) { return !item.editor.checkRequired(); })) {
                status = "required";
            }
            return status;
        }
    });
});