define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/Report/View",
    "dijit/Menu", 
    "dijit/MenuItem", 
    "dijit/form/DropDownButton",
    "thornberry/app/service/BillingManagement",
    "thornberry/app/utils"
],function(
    declare,
    lang,
    arrayUtils,
    ReportView,
    Menu,
    MenuItem,
    DropDownButton,
    service,
    utils
) {
	var canPostToAR = false;
	utils.checkPermission("BillingPostClaimsToAR",false).then(function(allowed){canPostToAR=!!allowed});
	
    return declare(ReportView,{
        title: "Billed Claims",

        defaultToFilterView: false,
        showPresetControl: false,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.BilledClaims",
                filters: null
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            var canMark = selectedIds.length > 0 && arrayUtils.every(selectedIds,function(id) { return t.getGridData(id).Status !== "Voided"; });
            var canVoid = canMark && arrayUtils.every(selectedIds,function(id) { return t.getGridData(id).VoidClaimType === "No"; });
			var itemActions = [
				{ label: "View", onClick: function() { t.loadSubView("claim"); }, hidden: selectedIds.length !== 1 }
			];
			if (canPostToAR) {
				itemActions.push({ label: "Regenerate File", onClick: function() { t.loadSubView("postingSummary"); }, hidden: selectedIds.length === 0 });
                itemActions.push( canMark ? t.markAsButton : null );
                itemActions.push({ label: "Void", onClick: function() { t.loadSubView("void"); }, hidden: !canVoid });
			}
            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh", onClick: function() {t.controller.gridView.updateGridData(true);} },
                    itemActions
                ]
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var claimStatuses = [
                { id: 4, name: "Submitted" },
                { id: 5, name: "Accepted" },
                { id: 6, name: "Rejected" },
                { id: 7, name: "Adjudicated" }
            ];

            var menu = new Menu({ style: "display: none;"});
            arrayUtils.forEach(claimStatuses,function(item) {
                var menuItem = new MenuItem({
                    label: item.name,
                    onClick: function() { t.updateBilledStatus(item.id); }
                });
                menu.addChild(menuItem);
            });

            t.markAsButton = new DropDownButton({
                label: "Mark as...",
                dropDown: menu
            });
            t.markAsButton.startup();
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            // Force the report initialization to occur when the module preloads the view
            t.invalidateRendering();
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);
            if (t.firstShow) {
                t.addSubView("claim","thornberry/app/Component/BillingManagement/ClaimView",function() {
                    return {
                        viewOnly: true,
                        objectId: t.props.selectedIds[0]
                    };
                });
                t.addSubView("postingSummary","thornberry/app/Component/BillingManagement/PostingSummaryView",function() {
                    return {
                        claimIds: t.props.selectedIds,
                        agingDate: null,
                        onComplete: t.props.onUpdateData
                    }
                });
                t.addSubView("void","thornberry/app/Component/BillingManagement/VoidClaimView",function() {
                    return {
                        claimIds: t.props.selectedIds,
                        onComplete: t.props.onUpdateData
                    };
                });
            }
        },

        onRowDoubleClick: function(id) {
            var t = this;
            t.loadSubView("claim",{ objectId: id } );
        },

        updateBilledStatus: function(status) {
            var t = this;
            var promise = service.updateBilledStatus(t.props.selectedIds,status);
            t.props.onSetLoadingPromise(promise);
            promise.then(function() {
                t.props.onUpdateData();
            },utils.showMessage);
        }
    });
});