define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/Report/View",
    "thornberry/app/service/BillingManagement",
    "thornberry/app/utils",
    "thornberry/app/Service/EnsembleFactory"
],function(
    declare,
    lang,
    arrayUtils,
    ReportView,
    service,
    utils,
    ensembleFactory
) {
    return declare(ReportView,{
        title: "Claim Response Files",

        defaultToFilterView: false,
        showPresetControl: false,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.ClaimResponseFiles",
                filters: null
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh", onClick: function() {t.controller.gridView.updateGridData(true);} },
                    { label: "Upload",  onClick: function() { t.loadSubView("uploadFileView"); } },
                    [
                        { label: "View", onClick: function() { t.showDetails(t.props.selectedIds[0]); }, hidden: selectedIds.length !== 1 }
                    ]
                ]
            });
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            // Force the report initialization to occur when the module preloads the view
            t.invalidateRendering();
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);
            if (t.firstShow) {
                t.addSubView("claimResponseFileDetails","thornberry/app/Component/BillingManagement/ClaimResponseFileDetailsView",function() {
                    return {
                        onComplete: t.props.onUpdateData
                    };
                });
                t.addSubView("uploadFileView","thornberry/app/Component/FileUploadView",function() {
                    return { 
                        title: "Upload Response File",
                        serverClass: "Billing.Claim.Service",
                        serverMethod: "UploadFile",
                        onUpload: function(promise) {
                            t.getSubView("uploadFileView").props.onCloseView();
                            var service = ensembleFactory("Billing.Claim.EnsembleResponse");
                            var promise = service(promise)
                            promise.then(function(claimResponseFileId) {
                                t.loadSubView("claimResponseFileDetails",{
                                    objectId: claimResponseFileId
                                });
                                t.refreshData();
                            },utils.showMessage);
                            t.props.onSetLoadingPromise(promise,"Processing...");
                        }
                    };
                });
            }
        },

        onRowDoubleClick: function(id) {
            this.showDetails(id);
        },

        download: function() {
            var t = this;
            service.getClaimFileDownloadLink(t.props.selectedIds[0]).then(function(fileLinkId) {
                utils.promptDownload(fileLinkId);
            },utils.showMessage);
        },

        showDetails: function(claimFileId) {
            var t = this;
            t.loadSubView("claimResponseFileDetails",{
                objectId: claimFileId
            });
        }
    });
});