define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "./ClaimEditor",
    "thornberry/app/service/BillingManagement",
    "dojo/Deferred",
    "thornberry/app/utils",
    "dojo/when"
],function(
    declare,
    lang,
    arrayUtils,
    _Component,
    _EditorViewMixin,
    ClaimEditor,
    service,
    Deferred,
    utils,
    when
) {
    return declare([_Component,_EditorViewMixin], {
        title: "Claim",

        EditorClass: ClaimEditor,

        lockCheckEnabled: true,
        objectType: "Billing.Claim.Claim",

        objectId: null,
        claim: null,

        getTitle: function() {
            return (this.props.mode === "view" ? "View " : "Code ") + this.title + (this.claim ? " ("+this.claim.claim.claimId+")" : "");
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                mode: "edit", // view or edit
                onSave: null
            });
        },

        getPageProps: function() {
            var t = this;
            var props = lang.mixin(t.inherited(arguments),{
                accountSummaryShown: false
            });
            if (t.props.mode === "view") {
                props.actions[1].label = "Code";
            }
            return props;
        },

        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                t.set("objectId",t.props.objectId);
            }
            t.inherited(arguments);
        },

        _setObjectIdAttr: function(value) {
            var t = this;
            if (value && t.objectId !== value) {
                t.addReadyPromise(service.getClaim(value).then(function(claim) {
                    if (t.objectId === value) {
                        t.set("claim",claim);
                    }
                },utils.showMessage));
            } else if (!value) {
                t.set("claim",null);
            }
            t.objectId = value;
        },

        _setClaimAttr: function(value) {
            var t = this;
            t.claim = value;
            t.editor.set("props",{
                value: value
            });
        },

        onBeforeClose: function() {
            var t = this;
            var closeResult = t.inherited(arguments);
            when (closeResult,function(value) {
                if (value) {
                    t.objectId = null;
                }
            });
            return closeResult;
        },

        startSave: function() {
            var t = this;
            var deferred = new Deferred();
            if (arrayUtils.some(t.editor.chargeGrid.rows(),function(row) {
                return !row.grid.cellWidget.getCellWidget(row.id,"modifiers").modifiersField.isValid();
            })) {
                utils.showMessage("Please correct any invalid fields.");
                deferred.reject();
            } else {
                t.inherited(arguments).then(deferred.resolve,deferred.reject);
            }
            return deferred.promise;
        },

        save: function() {
            var t = this;
            var data = t.editor.getSaveData();
            return service.codeClaim(data).then(function() {
                t.objectId = null;
                if (t.props.onSave) {
                    t.props.onSave();
                }
            });
        },

        resize: function() {
            var t = this;
            t.editor.resize();
        }
    })
});