define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/FinancialPeriodSummaryView.html",
    "thornberry/app/service/BillingManagement",
    "thornberry/app/Service/Settings",
    "dojo/dom-style",
    "thornberry/app/utils",
    "dojo/store/Memory",
    "thornberry/ReportManagerII"
],function(
    declare,
    lang,
    arrayUtils,
    _Component,
    _ViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    service,
    settingsService,
    domStyle,
    utils,
    Memory,
    ReportManager
) {
    var GeneratedItem = declare(ReportManager.GeneratedItem,{
        generateReport: function() {}
    });

    var ReportControl = declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: "<div><span data-dojo-attach-point='reportNameNode' style='display:inline-block;width:275px'></span><input data-dojo-type='dijit/form/Button' data-dojo-attach-event='onClick:_clickReprint' label='Reprint'><input data-dojo-type='dijit/form/Button' data-dojo-attach-event='onClick:_clickReexport' label='Re-export CSV'></div>",

        getInitialProps: function() {
            var t = this;
            return lang.hitch(t.inherited(arguments),{
                report: null,
                reportManager: null
            });
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);

            if (t.props.report) {
                t.reportNameNode.innerText = t.props.report.name;
            }
        },

        _clickReprint: function() {
            this.generate("PDF");
        },

        _clickReexport: function() {
            this.generate("CSV");
        },

        generate: function(presentationType) {
            var t = this;
            var generatedItem = new GeneratedItem({
                report: { displayName: t.props.report.name },
                presentationType: presentationType,
                manager: t.props.reportManager
            });
            t.props.reportManager.show();
            service.generateReportFromConfig(t.props.report.id,presentationType).then(
                lang.hitch(generatedItem,"onComplete"),lang.hitch(generatedItem,"onError")
            );
        }
    });

    return declare([_Component,_ViewMixin,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: template,
        
        title: "Financial Period Summary",

        settings: null,

        reportControllers: null,

        constructor: function() {
            var t = this;
            t.reportControllers = [];
        },

        getPageProps: function() {
            var t = this;
            var props = lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Done", onClick: function() { t.set("objectId",null); t.props.onCloseView(); } }
                ],
                content: t.domNode,
                printButtonShown: true,
                onClickPrint: lang.hitch(t.reportManager,"show")
            });
            return props;
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            t.reportManager = new ReportManager();
            t.reportManager.startup();
            t.reportManager.set("report",null);
        },

        refreshRendering: function() {
            var t = this;
            if (!t.settings) {
                t.addReadyPromise(settingsService.get("billingComponent").then(function(settings) { 
                    t.settings = settings; 
                    if (t.financialPeriod) {
                        t.set("financialPeriod",t.financialPeriod);
                    }
                }));
            }
            if (t.isShown) {
                t.set("objectId",t.props.objectId);
            }
            t.inherited(arguments);
        },

        _setObjectIdAttr: function(value) {
            var t = this;
            if (value && t.objectId !== value) {
                t.addReadyPromise(service.getFinancialPeriod(value).then(function(financialPeriod) {
                    if (t.objectId === value) {
                        t.set("financialPeriod",financialPeriod);
                    }
                },utils.showMessage));
            } else if (!value) {
                t.set("financialPeriod",null);
            }
            t.objectId = value;
        },

        _setFinancialPeriodAttr: function(value) {
            var t = this;
            t.financialPeriod = value;

            arrayUtils.forEach(t.reportControllers,function(item) {
                item.destroy();
            });

            if (t.settings) {
                var basis = t.settings.financialOrganizationalUnitBasis;
                domStyle.set(t.locationTypeContainer,   "display", basis == 1 ? "" : "none");
                domStyle.set(t.locationContainer,       "display", basis == 2 ? "" : "none");
            }

            if (value) {
                if (value.type) {
                    t.typeField.setStore(new Memory({ data: [ { id: value.type, name: value.typeDisplay } ], getLabel: function(item) { return item.name; } }));
                }
                if (value.location) {
                    t.locationField.setStore(new Memory({ data: [ { id: value.location, name: value.locationDisplay } ], getLabel: function(item) { return item.name; } }));
                }
                t.fromDateField.set("value",utils.conversions.date(null,value.startDate,"set"));
                t.throughDateField.set("value",utils.conversions.date(null,value.date,"set"));

                arrayUtils.forEach(value.reports,function(item) {
                    var reportControl = new ReportControl({
                        props: {
                            report: item,
                            reportManager: t.reportManager
                        }
                    });
                    reportControl.startup();
                    t.reportItemContainer.appendChild(reportControl.domNode);
                    t.reportControllers.push(reportControl);
                });
            }
        },

        _onClickDownloadZip: function() {
            service.getFinancialPeriodDownloadLink(this.objectId).then(utils.promptDownload,utils.showMessage);
        }
    });
});