define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/ReportItem.html",
    "thornberry/app/Component/Report/View",
    "dojo/promise/all",
    "dojo/when",
    "dojo/aspect",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/dom-construct",
    "thornberry/ReportManagerII",
    "thornberry/report/Report",
    "thornberry/report/filter/FinancialDateRange",
    "thornberry/app/utils",
    "dijit/ProgressBar",
    "dijit/form/ValidationTextBox"
],function(
    declare,
    lang,
    arrayUtils,
    _Component,
    _ViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    View,
    all,
    when,
    aspect,
    domStyle,
    domClass,
    domConstruct,
    ReportManager,
    Report,
    FinancialDateRange,
    utils
) {
    var initializingFinancialReportCount = 0;
    var financialPeriod = null;

    Report.extend({
        getColumn: function(column) {
            var t = this;
            var columnDefinition = t.columnStore.get(column);
            if (initializingFinancialReportCount > 0 && columnDefinition && columnDefinition.filterWidget && columnDefinition.filterWidget == "thornberry/report/filter/DateRange") {
                columnDefinition.filterWidget = "thornberry/report/filter/FinancialDateRange";
            }
            return columnDefinition;
        }
    });

    FinancialDateRange.extend({
        getDisplayValue: function() {
            var t = this;
            if (t.preset.get("value") == "financial") {
                return utils.formatters.date("startDate")(financialPeriod)+" through "+utils.formatters.date("date")(financialPeriod);
            } else {
                return this.inherited(arguments);
            }
        },
        getFilterValue: function() {
            var t = this;
            if (t.preset.get("value") == "financial" && financialPeriod) {
                return { start: utils.formatters.date("startDate")(financialPeriod), end: utils.formatters.date("date")(financialPeriod) };
            } else {
                return this.inherited(arguments);
            }
        }
    });

    var PrintOptionsView = declare([_Component,_ViewMixin],{
        title: "Print Options",
        content: null,
        controller: null,
        onApply: null,

        getPageProps: function() {
            var t = this;
            return {
                content: t.domNode,
                actions : [
                    { label: "Back", onClick: function() { t.props.onCloseView();} },
                ],
                helpButtonShown:    true,

                onClickHelp: function() { t.controller.showHelp(); },
            };
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            
            var reportManager = t.controller.reportManager;
            reportManager.show = function() {};

            // Prevent these report items from setting the user defaults (the current defaults have already been applied)
            reportManager.userSettings = {
                set: function() {},
                get: function() {},
                has: function() {}
            }

            t.domNode = domConstruct.toDom("<div class='report-display report-manager' style='display:block;background:none;border:none;z-index:auto;'></div>");

            t.reportControl = new ReportManager.ReportControl({
                report: t.controller.report,
                manager: reportManager,
                displayName: "" 
            }); 
            t.domNode.appendChild(t.reportControl.optionDiv);
            domConstruct.destroy(t.reportControl.exportOptions);
        },

        refreshRendering: function() {
            var t = this;
            t.addReadyPromise(all(t.controller.loadDeferredArray));
            t.inherited(arguments);
        }
    });

    var ReportItem = declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: template,

        _loaded: false,

        getInitialProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                label: "Report",
                financialPeriod: null
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            initializingFinancialReportCount++;

            t.reportView = new View({
                defaultToFilterView: false,
                showPresetControl: false,
                props: {
                    onPushView: null,
                    reportClass: t.props.reportClass
                }
            });
            aspect.before(t.reportView,"onLoadController",function(controller) {
                domStyle.set(t.filterButtonNode,"visibility",controller.report.filterColumns != "" ? "visible" : "hidden");
                domStyle.set(t.sumButtonNode,"visibility",controller.report.summarizableColumns != "" ? "visible" : "hidden");
            },true);
            aspect.after(t.reportView,"onLoadController",function(controller) {
                domStyle.set(t.progressBar.domNode,"display","none");
                domStyle.set(t.containerNode,"display","");

                t.printOptionsView = new PrintOptionsView({
                    controller: controller
                });

                initializingFinancialReportCount--;
                aspect.before(controller,"getContext",function() {
                    financialPeriod = t.props.financialPeriod;
                });
                aspect.after(controller,"getContext",function(context) {
                    context.options.presetName = t.reportNameField.get("value");
                    return context;
                });

                t._loaded = true;
            },true);

            if (t.props.onValidateName) {
                t.reportNameField.validator = function() {
                    return t.props.onValidateName(t,t.reportNameField.get("value"));
                }
            }
        },

        refreshRendering: function() {
            var t = this;
            t.reportNameField.set("value",t.props.label);
        },

        copy: function() {
            var t = this;
            var copy = new ReportItem({
                props: t.props
            });
            copy.props.label = t.reportNameField.get("value")+" copy";
            copy.setConfiguration(t.getConfiguration());
            return copy;
        },

        setConfiguration: function(configuration) {
            var t = this;
            var _setConfig = function(controller) {
                t.defer(function() {
                    controller.presetManager.set("currentPreset",configuration);
                    arrayUtils.forEach(controller.filterEditor.fields,function(field) {
                        controller.filterEditor.disable(field.name,false);
                    });

                    arrayUtils.forEach(t.printOptionsView.reportControl.optionWidgets,function(widget) {
                        if (typeof configuration.options[widget.optionDefinition.name] !== "undefined") {
                            widget.deserialize(configuration.options[widget.optionDefinition.name]);
                        }
                    });
                });
            };
            t._loaded ? _setConfig(t.reportView.controller) : aspect.after(t.reportView,"onLoadController",_setConfig,true);
        },

        getConfiguration: function(configuration) {
            var t = this;
            var configuration = t.reportView.controller.presetManager.currentPreset;
            arrayUtils.forEach(t.printOptionsView.reportControl.optionWidgets,function(widget) {
                configuration.options[widget.optionDefinition.name] = widget.serialize();
            });
            return configuration;
        },

        _onClickRemove: function() {
            var t = this;
            if (t.props.onRemove) {
                t.props.onRemove(t);
            }
        },

        _onClickCopy: function() {
            var t = this;
            if (t.props.onCopy) {
                t.props.onCopy(t.copy());
            }
        },

        _onClickPrint: function() {
            var t = this;
            var generatedItem = t.printOptionsView.reportControl.createGeneratedItem("PDF");
            if (t.props.onGenerateReport) {
                t.props.onGenerateReport(generatedItem);
            }
        },

        _onClickExport: function() {
            var t = this;
            var generatedItem = t.printOptionsView.reportControl.createGeneratedItem("CSV");
            if (t.props.onGenerateReport) {
                t.props.onGenerateReport(generatedItem);
            }
        },

        _onClickFilter: function() {
            var t = this;
            if (t.props.onPushView) { 
                t.reportView.filterView.getTitle = function() { return t.reportNameField.get("value") + " Filters"; };
                t.props.onPushView(t.reportView.filterView); 
            }
        },

        _onClickSum: function() {
            var t = this;
            if (t.props.onPushView) { 
                t.reportView.sumView.getTitle = function() { return t.reportNameField.get("value") + " Summarization"; };
                t.props.onPushView(t.reportView.sumView); 
            }
        },

        _onClickOptions: function() {
            var t = this;
            if (t.props.onPushView) { 
                t.reportView.optionView.getTitle = function() { return t.reportNameField.get("value") + " Options"; };
                t.props.onPushView(t.reportView.optionView); 
            }
        },

        _onClickPrintOptions: function() {
            var t = this;
            if (t.props.onPushView) { 
                t.printOptionsView.getTitle = function() { return t.reportNameField.get("value") + " Print Options"; };
                t.props.onPushView(t.printOptionsView); 
            }
        },

        _onClickHelp: function() {
            this.reportView.controller.showHelp();
        },

        _onChangeName: function() {
            var t = this;
            domClass.toggle(t.reportNameField.domNode,"pinkBackground",t.reportNameField.get("value") == "");
            if (t.props.onChangeName) {
                t.props.onChangeName();
            }
        }
    });
    return ReportItem;
});