define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "dijit/_TemplatedMixin",
    "thornberry/app/utils",
    "thornberry/report/formatter",
    "thornberry/serverRequest",
    "thornberry/app/service/BillingManagement"
],function(
    declare,
    lang,
    arrayUtils,
    _Component,
    _ViewMixin,
    _TemplatedMixin,
    utils,
    formatter,
    serverRequest,
    service
) {
    var structure = [
        { id: "patient", field: "PatientNameID", name: "Patient" },
        { id: "billPeriod", field: "Bill Period", name: "Bill Period" },
        { id: "insurance", field: "Insurance", name: "Insurance" },
        { id: "type", field: "Type", name: "Type" },
        { id: "agingDate", field: "Aging Date", name: "Aging Date" },
        { id: "gross", field: "GrossRevenue", name: "Gross", formatter: formatter.getForColumn({field:"GrossRevenue"},{formatter:"dollars"}) },
        { id: "net", field: "NetRevenue", name: "Net", formatter: formatter.getForColumn({field:"NetRevenue"},{formatter:"dollars"}) },
        { id: "claimStatus", field: "Status", name: "Claim Status" },
        { id: "claimId", field: "Claim ID", name: "Claim ID" },
        { id: "submissionCount", field: "Submission Count", name: "Submission Count" }
    ];

    return declare([_Component,_ViewMixin,_TemplatedMixin],{
        templateString: "<div class='content-area'><div class='content-area-title'><div class='content-area-title-text'>Claims to be voided</div></div><div data-dojo-attach-point='gridContainer'></div></div>",

        title: "Void Claim(s)",

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Back", onClick: t.props.onCloseView },
                    { label: "Void", onClick: lang.hitch(t,"voidClaims") },
                ],
                content: t.domNode
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.grid = utils.createGrid(structure,t.gridContainer);
        },
 
        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                var context = {
                    userId: utils.environment.getUserId(),
                    deviceAppId: utils.environment.getDeviceAppId(),
                    presentation: "Interactive",
                    reportClass: "Billing.Report.BilledClaims",
                    filters: [
                        { column: "id", value: t.props.claimIds }
                    ],
                    columns: arrayUtils.map(structure,function(column) { return column.field; }),
                    pageStart: 0,
                    pageSize: 100
                };

                var promise = serverRequest(context.reportClass,"GenerateData",context).then(function(results) {
                    t.grid.model.store.setData(results.data);
                    t.grid.model.clearCache();
                    t.grid.body.refresh();
                },utils.showMessage);
                t.addReadyPromise(promise);
            }
            t.inherited(arguments);
        },

        voidClaims: function() {
            var t = this;
            service.voidClaims(t.props.claimIds).then(function(errors) {
                if (errors.length) {
                    var message = "";
                    arrayUtils.forEach(errors,function(error) {
                        message +="\n\n"+error.claimId+" - "+error.error;
                    });
                    utils.showMessage("There was a problem voiding "+errors.length+" claim(s). These will not be included in a claim file."+message,null,true,500);
                }

                t.props.onCloseView();
                if (t.props.onComplete) {
                    t.props.onComplete();
                }
            },utils.showMessage)
        },

        onShow: function() {
            var t = this;
            t.resize();
        },

        resize: function() {
            this.grid.resize();
        }
    });
});