define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_GridViewMixin",
    "dojo/_base/lang",
    "dojo/topic",
    "thornberry/app/Service/PatientInsurance",
    "thornberry/app/utils",
    "thornberry/object/TableManager",
    "thornberry/app/Service/Settings",
    "thornberry/app/Service/EnsembleFactory",
    "dijit/Menu", 
    "dijit/MenuItem", 
    "dijit/form/ComboButton"
],function(
    declare,
    _Component,
    _GridViewMixin,
    lang,
    topic,
    patientInsuranceService,
    utils,
    tableManager,
    settingsService,
    ensembleFactory,
    Menu,
    MenuItem,
    ComboButton
) {
    return declare([_Component,_GridViewMixin], {
        // properties
        title: "Policies",
        helpPath: "/help/PatientInsurance.html#policies",

        accountNum: "",

        billingComponentSettings: null,
        eligibilitySettings: null,

        _requestComboButton: null,

        getColumns: function() {
            return [
                { id: "seq", name:"Sequence", formatter: function(item) { return item.seq ? item.seq.toString() : "N/A"; }, sortFormatted: true },
                { id: "insurance", name:"Insurance" },
                { id: "group", name:"Group" },
                { id: "plan", name:"Plan" },
                { id: "policyNum", name:"Policy Number" },
                { id: "claim", name:"Claim" },
                { id: "effectiveDate", name:"Effective Date", formatter: utils.formatters.date("effectiveDate") },
                { id: "stopDate", name:"Stop Date", formatter: utils.formatters.date("stopDate") },
                { id: "verifyDate", name:"Verify Date", formatter: utils.formatters.date("verifyDate") }
            ];
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                ndocNum: ""
            });
        },

        constructor: function() {
            var t = this;
            t.serializedFilters = t.filters = { "status" : 1 };
        },

        getGridProps: function() {
            return {
                sortInitialOrder:   { colId: "seq", descending: false }
            };
        },

        onRowDoubleClick: function(id) {
            var t = this;
            t.loadSubView("policyView",{ 
                mode: "view",
                objectId: id
            });
        },

        getFilterDefinition: function() {
            return [
                { id: "effectiveDate", name: "Effective Date", filterWidget: "thornberry/report/filter/DateRange" },
                { id: "stopDate", name: "Stop Date", filterWidget: "thornberry/report/filter/DateRange" },
                { id: "insurances", name: "Insurance", filterWidget: "thornberry/report/filter/TableMultiSelect", options: tableManager.get("Tables.Insurance").data },
                { id: "status", name: "Status", filterWidget: "thornberry/report/filter/Select", options: [ { id: 3, name: "All" }, { id: 1, name: "Active" }, { id: 4, name: "Active + Future" }, { id: 2, name: "Inactive" } ]}
            ];
        },

        filter: function(item,filters) {
            var show = !item.isErrored;
            if (show && filters) {
                show = 
                    utils.filters.inList(filters.insurances,item.insTabId) &&
                    utils.filters.dateRange(filters.effectiveDate,item.effectiveDate) &&
                    utils.filters.dateRange(filters.stopDate,item.stopDate);
                if (show && filters.status) {
                    var today = utils.conversions.date("",new Date(),"get");
                    var active = item.effectiveDate <= today && (!item.stopDate || item.stopDate >= today);
                    var future = item.effectiveDate > today;
                    show = ((filters.status == 3) || (filters.status == 4 ? active || future : filters.status == 1 ? active : !active));
                }
            }
            return show;
        },

        getPageProps: function() {
            var t = this;

            var showEligibilityCheck = utils.environment.getIsServer() && t.billingComponentSettings.enabledNow && t.props.selectedIds.length && t.eligibilitySettings.userHasAccess;

            return lang.mixin(t.inherited(arguments),{
                actions : [
                    [
                        { label: "Add",     onClick: function() { t.loadSubView("policyView",{ objectId: "" }); } },
                        { label: "Edit",    onClick: function() { t.loadSubView("policyView"); },                   hidden: t.props.selectedIds.length !== 1 },
                        { label: "View",    onClick: function() { t.loadSubView("policyView",{ mode: "view" }); },  hidden: t.props.selectedIds.length !== 1 },
                        { label: "Delete",  onClick: function() {t.deletePolicies(t.props.selectedIds)},            hidden: t.props.selectedIds.length === 0 }
                    ],
                    [
                        (showEligibilityCheck ? t.eligibilitySettings.interfaceEnabled ? t._requestComboButton : { label: "Check Eligibility",         onClick: function() { t.createEligibilityRequest() } } : null),
                        { label: "Show Authorizations", onClick: lang.hitch(t,"showAuthorizations"),                hidden: t.props.selectedIds.length !== 1 },
                        { label: "Change Sequence",     onClick: function() { t.loadSubView("sequenceView"); } }
                    ]
                ],
                accountSummaryShown: true,
                printButtonShown: true,
                report: {
                    reportClass: "Patient.Insurance.Reports",
                    getFilters: lang.hitch(t,"getReportFilters")
                }
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var menu = new Menu({ style: "display: none;"});
            var menuItem = new MenuItem({
                label: "Create File",
                onClick: function() { t.createEligibilityRequest() }
            });
            menu.addChild(menuItem);

            t._requestComboButton = new ComboButton({
                label: "Check Eligibility",
                dropDown: menu,
                onClick: function() { t.createEligibilityRequest(true) }
            });
            t._requestComboButton.startup();
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);
            tableManager.get("Tables.Insurance"); // Preload table for filters
        },

        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                t.set("ndocNum",t.props.ndocNum);
            }
            if (!t.billingComponentSettings) {
                // Get billing component settings
                t.addReadyPromise(settingsService.get("billingComponent").then(function(result) {
                    t.billingComponentSettings = result;
                }));
            }
            if (!t.eligibilitySettings) {
                t.addReadyPromise(settingsService.get("insuranceEligibility").then(function(settings) {
                    t.eligibilitySettings = settings;
                }));
            }
            t.inherited(arguments);
        },

        _setNdocNumAttr: function(value) {
            var t = this;
            if (t.ndocNum !== value && value) {
                t.ndocNum = value;
                t.addReadyPromise(patientInsuranceService.getPolicies(value));
                t.setHandle("watchPolicies",patientInsuranceService.watchPolicies(value,lang.hitch(t,"setGridData")));

                patientInsuranceService.getGeneral(value).then(function(result) {
                    t.accountNum = result.accountNum; // get the account num for report filter display
                });
            }
        },

        deletePolicies: function(ids) {
            var t = this;
            utils.showMessage("Deleting a policy also deletes all associated authorizations and cannot be undone. Are you sure you want to delete all selected policies?",[
                { label: "Yes", callback: function() {
                    patientInsuranceService.deletePolicies(ids).then(function() {
                        patientInsuranceService.getPolicies(t.props.ndocNum,true);
                        patientInsuranceService.getPolicySequences(t.props.ndocNum,true);
                    },utils.showMessage);
                } },
                { label: "No"}
            ]);
        },

        /*createEligibilityRequest: function(useInterface) {
            var t = this;
            useInterface = !!useInterface;
            var promise = eligibilityService.submitRequest(t.props.selectedIds,useInterface).then(function(result) {
                t.loadSubView("eligibilityRequestSummary",result);
                patientInsuranceService.getPolicies(t.props.ndocNum,true);
                t.getSubView("policyView").objectId = "";
            },function(error) {
                utils.showMessage(error);
            });
            t.props.onSetLoadingPromise(promise);
        },*/

        createEligibilityRequest: function(useInterface) {
            var t = this;
            var service = ensembleFactory("Billing.Eligibility.EnsembleRequest");
            var promise = service({policyIds:t.props.selectedIds,useInterface:!!useInterface});
            promise.then(function(result) {
                t.loadSubView("eligibilityRequestSummary",result);
                patientInsuranceService.getPolicies(t.props.ndocNum,true);
                t.getSubView("policyView").objectId = "";
            },utils.showMessage);
            t.props.onSetLoadingPromise(promise);
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);

            if (t.firstShow) {
                t.addSubView("policyView","thornberry/app/Component/PatientInsurance/AddEditPolicyView",function() {
                    return { 
                        ndocNum: t.props.ndocNum,
                        objectId: (t.props.selectedIds && t.props.selectedIds.length === 1 ? t.props.selectedIds[0] : ""),
                        mode: "edit"
                    };
                });
                t.addSubView("sequenceView","thornberry/app/Component/PatientInsurance/AddEditSequenceView",function() {
                    return { 
                        ndocNum: t.props.ndocNum,
                        mode: "edit" 
                    };
                });
                t.addSubView("eligibilityRequestSummary","thornberry/app/Component/InsuranceEligibility/RequestSummaryView",function() {
                    return {};
                });
            }
        },

        showAuthorizations: function() {
            var t = this;
            topic.publish("AuthorizationsView",{
                filters: {
                    policy: t.props.selectedIds[0]
                }
            });
            t.props.onSwitchView(3);
        },

        getReportFilters: function(presentation,report) {
            var t = this;
            var selectedIds = t.grid.getSelected(); // Get current value since t.props.selectedIds doesn't update on sort
            var filters = [
                {
                    column: "NDocNumber",
                    displayName: "Patient",
                    value: [ t.props.ndocNum ],
                    displayValue: t.accountNum
                }
            ];
            if (report.className === "Report.Definition.PatientInsuranceWithHistory") {
                filters.push({
                    column: "id",
                    value: (selectedIds.length === 0 ? t.grid.getAll() : selectedIds),
                    hidden: true
                });
            }
            return filters;
        }
    });
});