// thornberry/med/AdministrationEvent.js
define([
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/focus",
	"dojo/text!./templates/administrationEvent.html",
	"dojo/date/locale",
	"dojo/dom",
	"dojo/dom-style",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/store/Memory",
	"dijit/form/FilteringSelect",
	"dijit/form/DateTextBox",
	"thornberry/form/TimeTextBox",
	"thornberry/form/TimeSpinner",
	"thornberry/med/employeeMultiSelect",
	"dijit/form/ValidationTextBox",
	"dijit/form/NumberTextBox",
	"dijit/form/ComboBox",
	"dijit/form/Textarea",
	"dijit/form/CheckBox",
	"dijit/form/Form",
	"dijit/form/DropDownButton",
	"dijit/TooltipDialog",
	"dijit/form/Button",
	"dijit/popup",
	"dojox/form/CheckedMultiSelect",
	"dojo/on",
	"dojo/topic",
	"dojo/date",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/med/quantityTimingSet",
	"thornberry/med/medicationDetail"
], function(declare,_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin,focusUtil,template,locale,dom,domStyle,domConstruct,domGeom,Memory,FilteringSelect,DateTextBox,TimeTextBox,TimeSpinner,EmployeeMultiSelect,ValidationTextBox,NumberTextBox,ComboBox,Textarea,CheckBox,Form,DropDownButton,TooltipDialog,Button,popup,CheckedMultiSelect,on,topic,date,lang,arrayUtils,QuantityTimingSet,medicationDetail){
        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
			templateString: template,
			baseClass: "administrationEvent",
			
			administration: "",
			
			modified: false,
			userId: "",
			patientStore: {},
			medStore: {},
			quantityTimingStore: {},
			cacheManager: {},
			notGivenReasonStore: {},
			patientReactionStore: {},
			prnReasonStore: {},
			unitStore: {},
			userStore: {},
			medSearch: function() {},
			actionShowMessage: function() {},
			settings:settings,
			
			postMixInProperties: function() {
				var t = this;
				t.inherited(arguments);

				t._formatTime = medicationDetail.prototype._formatTime;
			},
			
			buildRendering: function() {
				var t = this;
				t.inherited(arguments);

				lang.hitch(t,medicationDetail.prototype.buildGrids)();
				
				t.adminDateBox = new DateTextBox({
					required: true,
					style: "width:110px",
					validator: function(value,constraints) {
						var valid = DateTextBox.prototype.validator.call(this, value, constraints);
						var adminTime = t._getAdminTime(true);
						if (valid) {
							if (adminTime) {
								var qt = t.quantityTimingStore.get(t.administration.qtId)
								if (!qt) {
									qt = QuantityTimingSet.getDummyForMed(t.administration.medId);
								}
								valid = qt.isActiveOnDate(adminTime,true);
								this.invalidMessage = "The Administration Date/Time is outside this med or quantity/timing set's active range.";
							}
						} else {
							this.invalidMessage = "The value entered is not valid.";
						}
						return valid;
					}
				},t.adminDateNode);
				t.adminDateBox.startup();
				t.own(on(t.adminDateBox,"change",lang.hitch(t,t._modifyDate)));
				t.own(on(t.adminDateBox,"blur",function(){checkForT(t.adminDateBox.id)}));
				
				t.adminTimeBox = new TimeTextBox({
					required: true,
					constraints: {timePattern: "HHmm", clickableIncrement: "T00:15:00", visibleIncrement: "T01:00:00", visibleRange: "T01:00:00"},
					style: "width:90px",
					maxHeight: -1/*,
					disabled: true*/
				},t.adminTimeNode);
				t.adminTimeBox.startup();
				t.own(on(t.adminTimeBox,"change",lang.hitch(t,t._modifyTime)));
				t.own(on(t.adminTimeBox,"blur",function(){checkForN(t.adminTimeBox.id)}));
				
				t.totalTimeBox = new ValidationTextBox({
					style: "width:8em",
					validator: function(value) {
						var valid = true;
						if (this.required || value !== "") {
							var regExp = /^\d{1,3}:\d{2}$/;
							valid = regExp.test(value);
							if (valid) {
								var pieces = value.toString().split(":");
								if (pieces.length > 1 && parseInt(pieces[1]) >= 60) {
									valid = false;
								}
							}
						}
						return valid;
					},
					invalidMessage: "Total time must be entered as H:MM"
				},t.totalTimeNode);
				t.totalTimeBox.startup();
				t.own(on(t.totalTimeBox,"change",lang.hitch(t,t._modifyTotalTime)));
				t.tTField = t.totalTimeBox;
				
				t.adminByBox = new FilteringSelect({
					required: true,
					store: t.userStore,
					pageSize: 100
				},t.adminByNode);
				t.adminByBox.startup();
				t.own(on(t.adminByBox,"change",lang.hitch(t,t._modifyAdminBy)));
				
				t.witnessesMultiSelect = new EmployeeMultiSelect({
					required: false,
					employeeStore: t.employeeStore,
					pageSize: 100
				},t.witnessesNode);
				t.witnessesMultiSelect.startup();
				t.own(on(t.witnessesMultiSelect,"change",lang.hitch(t,t._modifyWitnesses)));
				
				t.notGivenReasonBox = new ComboBox({
					store: t.notGivenReasonStore,
					searchAttr: "name",
					maxLength: 100
				},t.notGivenReasonNode);
				t.notGivenReasonBox.startup();
				t.own(on(t.notGivenReasonBox,"change",lang.hitch(t,t._modifyNotGivenReason)));
				
				t.reactionBox = new ComboBox({
					store: t.patientReactionStore,
					searchAttr: "name",
					maxLength: 100
				},t.reactionNode);
				t.reactionBox.startup();
				t.own(on(t.reactionBox,"change",lang.hitch(t,t._modifyReaction)));
				
				t.prnReasonBox = new FilteringSelect({
					store: new Memory(),
					searchAttr: "name",
					maxLength: 100
				},t.prnReasonNode);
				t.prnReasonBox.startup();
				t.own(on(t.prnReasonBox,"change",lang.hitch(t,t._modifyPRNReason)));
				
				t.startDateBox = new DateTextBox({
					style: "width:110px",
					validator: function(value,constraints) {
						var valid = DateTextBox.prototype.validator.call(this, value, constraints);
						var startTime = t._getStartTime(true);
						if (valid) {
							if (startTime) {
								var qt = t.quantityTimingStore.get(t.administration.qtId)
								if (!qt) {
									qt = QuantityTimingSet.getDummyForMed(t.administration.medId);
								}
								valid = qt.isActiveOnDate(startTime,true);
								this.invalidMessage = "The Start Date/Time is outside this med or quantity/timing set's active range.";
							}
						} else {
							this.invalidMessage = "The value entered is not valid.";
						}
						return valid;
					}
				},t.startDateNode);
				t.startDateBox.startup();
				t.own(on(t.startDateBox,"change",lang.hitch(t,t._modifyStartDate)));
				t.own(on(t.startDateBox,"blur",function(){checkForT(t.startDateBox.id)}));
				t.sDField = t.startDateBox;
				
				t.startTimeBox = new TimeTextBox({
					constraints: {timePattern: "HHmm", clickableIncrement: "T00:15:00", visibleIncrement: "T01:00:00", visibleRange: "T01:00:00"},
					style: "width:90px",
					maxHeight: -1
				},t.startTimeNode);
				t.startTimeBox.startup();
				t.own(on(t.startTimeBox,"change",lang.hitch(t,t._modifyStartTime)));
				t.own(on(t.startTimeBox,"blur",function(){checkForN(t.startTimeBox.id)}));
				t.sTField = t.startTimeBox;
				
				t.endDateBox = new DateTextBox({
					style: "width:110px",
					validator: function(value,constraints) {
						var valid = DateTextBox.prototype.validator.call(this, value, constraints);
						var startTime = t._getStartTime();
						var endTime = t._getEndTime(true);
						if (valid) {
							if (startTime && endTime && startTime > endTime) {
								valid = false;
								this.invalidMessage = "Stop Date/Time cannot occur before the Start Date/Time.";
							} else if (endTime) {
								var qt = t.quantityTimingStore.get(t.administration.qtId)
								if (!qt) {
									qt = QuantityTimingSet.getDummyForMed(t.administration.medId);
								}
								valid = qt.isActiveOnDate(endTime,true);
								this.invalidMessage = "The Stop Date/Time is outside this med or quantity/timing set's active range.";
							}
						} else {
							this.invalidMessage = "The value entered is not valid.";
						}
						return valid;
					}
				},t.endDateNode);
				t.endDateBox.startup();
				t.own(on(t.endDateBox,"change",lang.hitch(t,t._modifyEndDate)));
				t.own(on(t.endDateBox,"blur",function(){checkForT(t.endDateBox.id)}));
				t.eDField = t.endDateBox;
				
				t.endTimeBox = new TimeTextBox({
					constraints: {timePattern: "HHmm", clickableIncrement: "T00:15:00", visibleIncrement: "T01:00:00", visibleRange: "T01:00:00"},
					style: "width:90px",
					maxHeight: -1
				},t.endTimeNode);
				t.endTimeBox.startup();
				t.own(on(t.endTimeBox,"change",lang.hitch(t,t._modifyEndTime)));
				t.own(on(t.endTimeBox,"blur",function(){checkForN(t.endTimeBox.id)}));
				t.eTField = t.endTimeBox;
				
				t.commentBox = new Textarea({
					maxLength: 250
				},t.commentNode);
				t.commentBox.startup();
				t.own(on(t.commentBox,"change",lang.hitch(t,t._modifyComment)));
				
				t.errorReasonBox = new ValidationTextBox({
					maxLength: 250
				},t.errorReasonNode);
				t.errorReasonBox.startup();
				t.own(on(t.errorReasonBox,"change",lang.hitch(t,t._modifyErrorReason)));
				
				t.administrationInstructionsBox = new Textarea({
					maxLength: 250
				},t.administrationInstructionsNode);
				t.administrationInstructionsBox.startup();
				t.own(on(t.administrationInstructionsBox,"change",lang.hitch(t,t._modifyAdministrationInstructions)));
				
				t.ivCheckBox = new CheckBox({
					value: true
				},t.ivNode);
				t.ivCheckBox.startup();
				t.own(on(t.ivCheckBox,"change",lang.hitch(t,t._modifyIVCheckBox)));
				
				t.ivDropFactorBox = new ValidationTextBox({
					style:"width:80px",
					pattern: "([1-9]\\d{0,5}|0)(\\.\\d{1,3})?"
				},t.ivDropFactorNode);
				t.ivDropFactorBox.startup();
				t.own(on(t.ivDropFactorBox,"change",lang.hitch(t,t._modifyIVDropFactor)));
				
				t.ivDripRateBox = new ValidationTextBox({
					style:"width:80px",
					pattern: "([1-9]\\d{0,5}|0)(\\.\\d{1,3})?"
				},t.ivDripRateNode);
				t.ivDripRateBox.startup();
				t.own(on(t.ivDripRateBox,"change",lang.hitch(t,t._modifyIVDripRate)));
				
				t.ivFlowRateBox = new ValidationTextBox({
					style:"width:80px",
					pattern: "([1-9]\\d{0,5}|0)(\\.\\d{1,3})?"
				},t.ivFlowRateNode);
				t.ivFlowRateBox.startup();
				t.own(on(t.ivFlowRateBox,"change",lang.hitch(t,t._modifyIVFlowRate)));
				
				t.ivTypeBox = new ValidationTextBox({
					style:"width:120px",
					maxLength: 50
				},t.ivTypeNode);
				t.ivTypeBox.startup();
				t.own(on(t.ivTypeBox,"change",lang.hitch(t,t._modifyIVType)));
				
				/*t.ivVolumeBox = new ValidationTextBox({style:"width:80px"},t.ivVolumeNode);
				t.ivVolumeBox.startup();
				t.own(on(t.ivVolumeBox,"change",lang.hitch(t,t._modifyIVVolume)));*/
				
				t.ivSizeBox = new ValidationTextBox({
					style:"width:80px",
					maxLength: 50
				},t.ivSizeNode);
				t.ivSizeBox.startup();
				t.own(on(t.ivSizeBox,"change",lang.hitch(t,t._modifyIVSize)));
				
				t.ivSiteGivenBox = new ValidationTextBox({
					style:"width:160px",
					maxLength: 50
				},t.ivSiteGivenNode);
				t.ivSiteGivenBox.startup();
				t.own(on(t.ivSiteGivenBox,"change",lang.hitch(t,t._modifyIVSiteGiven)));
				
				t.own(on(t.ivSiteAppearanceMultiSelect,"click",lang.hitch(t,t._modifyIVSiteAppearance)));
				
				t.ivSiteAppearanceOtherBox = new ValidationTextBox({
					maxLength: 50
				},t.ivSiteAppearanceOtherNode);
				t.ivSiteAppearanceOtherBox.startup();
				t.own(on(t.ivSiteAppearanceOtherBox,"change",lang.hitch(t,t._modifyIVSiteAppearanceOther)));
				
				t.medGivenBox = new ValidationTextBox({},t.medGivenNode);
				t.medGivenBox.startup();
				t.own(on(t.medGivenBox,"change",lang.hitch(t,t._modifyMedGiven)));
				
				var mainNode = domConstruct.create("div");
				t.medResultsNode = domConstruct.place("<div></div>",mainNode);
				t.medSearchSelect = domConstruct.place("<select size='7'></select>",t.medResultsNode);
				domConstruct.place("<br>",t.medResultsNode);
				t.okMedSearchBtn = new Button({
					style:"display:inline-block",
					label:"Select"
				});
				domConstruct.place(t.okMedSearchBtn.domNode,t.medResultsNode);
				t.medInfoBtn = new Button({
					style:"display:inline-block",
					label:"Info"
				});
				domConstruct.place(t.medInfoBtn.domNode,t.medResultsNode);
				t.medSearchNoResults = domConstruct.create("div");
				t.medNoResultsNode = domConstruct.place("<div></div>",mainNode);
				domConstruct.place(t.medSearchNoResults,t.medNoResultsNode);
				
				t.medSearchDialog = new TooltipDialog({
					content: mainNode
				});
				
				t.findButton = new DropDownButton({
					label:"Find",
					dropDown: t.medSearchDialog
				});
				t.medGivenDiv.appendChild(t.findButton.domNode);
				
				t.quantityGivenBox = new NumberTextBox({
					required: true,
					style: "width:80px",
					maxLength: 250,
					placeHolder: "Quantity",
					constraints: {min:0,max:999999999}
				},t.quantityGivenNode);
				t.quantityGivenBox.startup();
				t.own(on(t.quantityGivenBox,"change",lang.hitch(t,t._modifyQuantityGiven)));

				t.unitGivenBox = new ValidationTextBox({
					required: true,
					style: "width:150px",
					maxLength: 200,
					placeHolder: "Unit"
				},t.unitGivenNode);
				t.unitGivenBox.startup();
				t.own(on(t.unitGivenBox,"change",lang.hitch(t,t._modifyUnitGiven)));
				
				t.expirationDateBox = new DateTextBox({
					style: "width:110px"
				},t.expirationDateNode);
				t.expirationDateBox.startup();
				t.own(on(t.expirationDateBox,"change",lang.hitch(t,t._modifyExpirationDate)));
				
				t.lotNumberBox = new ValidationTextBox({
					style: "width:8em"
				},t.lotNumberNode);
				t.lotNumberBox.startup();
				t.own(on(t.lotNumberBox,"change",lang.hitch(t,t._modifyLotNumber)));
			},
			
			postCreate: function() {
				var t = this;
				t.inherited(arguments);
				
				t.own(on(t.findButton,"focus",lang.hitch(t,"searchForMed")));
				t.own(on(t.okMedSearchBtn.domNode,"click",lang.hitch(t,"selectMed")));
				
				var seconds = (new Date()).getSeconds();
				setTimeout(function() {
					var func = lang.hitch(t,"refreshDisplay");
					func();
					setInterval(func,60000);
				},(60-seconds)*1000);
				
				t.own(on(t.schedulingLink,"click",function() {
					var expand = (domStyle.get(t.qtGridNode,"display") === "none");
					dom.byId('schedulingLinkIndicator').innerHTML = (expand ? '-' : '+');
					domStyle.set(t.qtGridNode,"display",(expand ? "" : "none"));
					t.grid.resize();
				}));
				t.own(on(t.administrationsLink,"click",function() {
					var expand = (domStyle.get(t.adminGridNode,"display") === "none");
					dom.byId('administrationsLinkIndicator').innerHTML = (expand ? '-' : '+');
					domStyle.set(t.adminGridNode,"display",(expand ? "" : "none"));
					t.adminGrid.resize();
				}));
			},
			
			refreshDisplay: function() {
				var t = this;
				if (t.administration && t.administration.action === "view") {
					var administration = t.cacheManager.eventCache.get(t.administration.id);
					administration = t.copyAdministration(administration);
					administration.action = "view";
					t.set("administration",administration);
				}
			},
			
			toggleField: function(widget,enable,require,preventOnChange) {
				if (enable) {
					widget.set("disabled",false);
					if (require) {
						widget.set("required",true);
					}
				} else {
					widget.set("disabled",true);
					widget.set("required",false);
					widget.set("value","",!preventOnChange);
				}
			},
			
			notify: function(name,value) {
				var t = this;
				if (t[name+"Field"]) {
					if (t.administration[name+"GetDisplay"]) {
						value = t.administration[name+"GetDisplay"]();
					}
					t[name+"Field"].set("value",value,false);
				}
				if (name === "tT" && t.administration.action !== "error") {
					var totalTime = t._getTotalTime();
					var duration = t._getDurationAsDate();
					if (duration && totalTime) {
						if (totalTime > duration) {
							t.actionShowMessage("Warning: the total time is longer than the scheduled duration. Please double check the start and end times.");
						}
					} else if (totalTime) {
						var pieces = t.totalTimeBox.get("value").toString().split(":");
						if (parseInt(pieces[0]) >= 1) {
							t.actionShowMessage("Warning: the total time is an hour or longer. Please double check the start and end times.");
						}
					}
				}
			},
			
			getAdministration: function(status,preventOnChange) {
				var preventOnChange = preventOnChange || false;
				var t = this;
				var ret = "";
				if (t.administration.action !== "error") {
					// Do validation logic...
					if (status === "" || status === 2 || status === 5) {
						t.toggleField(t.quantityGivenBox,true,true,preventOnChange);
						t.toggleField(t.unitGivenBox,true,true,preventOnChange);

						if (t.administration.version >= 2 && t.administration.doseUnit === "") {
							var qt = t.quantityTimingStore.get(t.administration.qtId);
							t.unitGivenBox.set("value",(qt ? qt.unit : ""),false);
							t.administration.doseUnit = (qt ? qt.unit : "");
						}
						t.defer(function() {t.administrationInstructionsBox.focus()});

						t.ivCheckBox.set("disabled",(t.administration.med.isIv !== ""));
						t.toggleField(t.notGivenReasonBox,false,false,preventOnChange);
					} else if (status === 3) {
						t.toggleField(t.notGivenReasonBox,true,true,preventOnChange);
						
						t.toggleField(t.quantityGivenBox,false,false,preventOnChange);
						t.toggleField(t.unitGivenBox,false,false,preventOnChange);
						t.administration.amountGiven = "";
						
						if (status !== t.administration.status) {
							t.ivCheckBox.set("checked",false);
							t.ivCheckBox.set("disabled",true);
							t.administration.isIv = 0;
						}
					}
					
					t.prnReasonBox.set("required",false);
					var med = t.medStore.get(t.administration.medId);

					if (med.isPRN(t.administration.qtId)) {
						t.toggleField(t.prnReasonBox,true,true,preventOnChange);
					} else {
						t.toggleField(t.prnReasonBox,false,false,preventOnChange);
					}
					
					t.administration.set("status",status);
					t.updateTags();
					t._modifyPost();
					if (t.formNode.validate() && t.witnessesMultiSelect.isValid()) {
						if (t.addedThisSession) {
							t.administration.action = "add";
						}
						ret = t.administration;
						t.modified = false;
						 
						if (t.administration.isIv && !preventOnChange) {
							var pat = t.patientStore.get(t.administration.patId);
							pat.ivLocation = t.administration.siteGiven;
							pat.ivType = t.administration.ivType;
							pat.ivSize = t.administration.ivSize;
							pat.ivFlowRate = t.administration.ivfr;
							pat.ivSiteAppearance = t.administration.ivSiteAppearance;
							pat.ivSiteAppearanceOther = t.administration.ivSiteAppearanceOther;
							t.patientStore.put(pat);
						}
						if (!preventOnChange) {
							var med = t.medStore.get(t.administration.medId);
							med.admInst = t.administration.admInst;
						}
					} else {
						if (t.administration.action === "add") {
							t.administration.action = "edit";
						}
					}
				} else {
					t.errorReasonBox.set("required",true);
					if (t.errorReasonBox.validate()) {
						t.administration.err = 1;
						ret = t.administration;
						t.modified = false;
					} else {
						t.formNode.validate();
					}
				}
				return ret;
			},
			
			validate: function() {
				var t = this;
				return t.formNode.validate();
			},
			
			requireEmpty: function(widget,type,requireEmpty) {
				if (typeof requireEmpty === "undefined") {
					requireEmpty = true;
				}
				if (requireEmpty) {
					widget.set("required",false);
				}
				if (requireEmpty) {
					widget.validator = function(value,constraints) {
						var valid = true;
						if (value !== "" && value !== null) {
							valid = false;
							this.set('invalidMessage', 'This value must be empty.');
						}
						return valid;
					};
				} else {
					widget.validator = type.prototype.validator;
					widget.set('invalidMessage', 'The value entered is not valid.');
				}
			},
			
			copyAdministration: function(a) {
				var t = this;
				var copy = {};
				for (var prop in a) {
					copy[prop] = a[prop];
				}
				return copy;
			},
			
			_setDisabledAttr: function(disabled) {
				var t = this;
				t.disabled = disabled;
				t._set("disabled",disabled);
				
				t.administrationInstructionsBox.set("disabled",disabled);
				t.adminDateBox.set("disabled",disabled);
				t.adminTimeBox.set("disabled",disabled);
				t.adminByBox.set("disabled",disabled);
				t.witnessesMultiSelect.set("disabled",disabled);
				t.reactionBox.set("disabled",disabled);
				t.prnReasonBox.set("disabled",disabled);
				t.notGivenReasonBox.set("disabled",disabled);
				t.startDateBox.set("disabled",disabled);
				t.startTimeBox.set("disabled",disabled);
				t.endDateBox.set("disabled",disabled);
				t.endTimeBox.set("disabled",disabled);
				t.totalTimeBox.set("disabled",disabled);
				t.commentBox.set("disabled",disabled);
				//t.errorReasonBox.set("disabled",disabled);
				t.ivCheckBox.set("disabled",disabled);
				t.ivTypeBox.set("disabled",disabled);
				//t.ivVolumeBox.set("disabled",disabled);
				t.ivSizeBox.set("disabled",disabled);
				t.ivDropFactorBox.set("disabled",disabled);
				t.ivDripRateBox.set("disabled",disabled);
				t.ivFlowRateBox.set("disabled",disabled);
				t.ivSiteGivenBox.set("disabled",disabled);
				t.ivSiteAppearanceMultiSelect.set("disabled",disabled);
				t.ivSiteAppearanceOtherBox.set("disabled",disabled);
				t.medGivenBox.set("disabled",disabled);
				t.quantityGivenBox.set("disabled",disabled);
				t.unitGivenBox.set("disabled",disabled);
				t.expirationDateBox.set("disabled",disabled);
				t.lotNumberBox.set("disabled",disabled);
				t.findButton.set("disabled",disabled);
			},
			
			showIv: function(show,getFromEvent,preventUpdate) {
				var t = this;
				
				if (t.administration.status === 3) {
					show = false;
				}
				
				var currentValue = domStyle.get(t.ivDiv,"display");
				if (currentValue !== (show ? "" : "none")) {
					domStyle.set(t.ivDiv,"display",(show ? "" : "none"));
					
					t.ivSiteAppearanceMultiSelect.set("value",[1,2,3,4,6],false);
					var a = t.administration;
					
					if (show) {
						if (!preventUpdate) {a.isIv = 1};
						if (getFromEvent) {
							t.ivTypeBox.set("value",a.ivType,false);
							t.ivSizeBox.set("value",a.ivSize,false);
							t.ivSiteGivenBox.set("value",a.siteGiven,false);
							t.ivSiteAppearanceMultiSelect.set("value",a.ivSiteAppearance.toString().split(","),false);
							t.ivSiteAppearanceOtherBox.set("value",a.ivSiteAppearanceOther,false);
							t.ivDripRateBox.set("value",a.ivdr,false);
							t.ivFlowRateBox.set("value",a.ivfr,false);
							t.ivDropFactorBox.set("value",a.ivdf,false);
							if (a.ivdr !== "" || a.ivdf !== "") {
								t.ivFlowRateBox.set("disabled",true);
							} else {
								t.ivFlowRateBox.set("disabled",false);
							}
							if (a.ivfr !== "") {
								t.ivDripRateBox.set("disabled",true);
							} else {
								t.ivDripRateBox.set("disabled",false);
							}
						} else {
							var pat = t.patientStore.get(a.patId);
							t.ivTypeBox.set("value",pat.ivType,false);
							t.ivSizeBox.set("value",pat.ivSize,false);
							t.ivSiteGivenBox.set("value",pat.ivLocation,false);
							t.ivFlowRateBox.set("value",pat.ivFlowRate,false);
							t.ivSiteAppearanceMultiSelect.set("value",pat.ivSiteAppearance.toString().split(","),false);
							t.ivSiteAppearanceOtherBox.set("value",pat.ivSiteAppearanceOther,false);
							if (!preventUpdate) {
								a.ivType = pat.ivType;
								a.ivSize = pat.ivSize;
								a.siteGiven = pat.ivLocation;
								a.ivSiteAppearance = pat.ivSiteAppearance;
								a.ivSiteAppearanceOther = pat.ivSiteAppearanceOther;
							}
						}
						
						t.ivSiteAppearanceOtherBox.set("required",false);
						if (a.ivSiteAppearance.toString().indexOf(6) !== -1 || a.ivSiteAppearanceOther !== "") {
							t.ivSiteAppearanceOtherBox.set("disabled",false);
							if (a.ivSiteAppearance.toString().indexOf(6) !== -1) {
								t.ivSiteAppearanceOtherBox.set("required",true);
							}
						}
						
						if (!preventUpdate && !t.startDateBox.get("value") && !t.startTimeBox.get("value")) {
							a.sD = a.date;
							a.sT = a.time;
							t.startDateBox.set("value",a.sD,false);
							t.startTimeBox.set("value",locale.parse(a.sT.toString(),{selector:"time", timePattern:"HHmm"}),false);
						}
						
						if (!preventUpdate) {
							t.calculateIVRate("totalTime");
						}
					} else {
						if (!preventUpdate) {a.isIv = 0};
						t.ivTypeBox.set("value","",false);
						t.ivSizeBox.set("value","",false);
						t.ivSiteGivenBox.set("value","",false);
						t.ivSiteAppearanceMultiSelect.set("value","",false);
						t.ivSiteAppearanceOtherBox.set("value","",false);
						t.ivDripRateBox.set("value","",false);
						t.ivFlowRateBox.set("value","",false);
						t.ivDropFactorBox.set("value","",false);
						if (!preventUpdate) {
							a.ivType = "";
							a.ivSize = "";
							a.siteGiven = "";
							a.ivSiteAppearance = "";
							a.ivSiteAppearanceOther = "";
							a.ivdr = "";
							a.ivfr = "";
							a.ivdf = "";
						}
						t.ivDripRateBox.set("required",false);
						t.ivFlowRateBox.set("required",false);
						t.ivSiteAppearanceOtherBox.set("required",false);
					}
				}
			},
			
			_setAdministrationAttr: function(administration) {
				var t = this;
				if (t.administration) {
					t.administration.set("watcher","");
				}
					
				var a = administration;
				t.administration = a;
				t._set("administration",a);
				t.addedThisSession = false;
				
				if (typeof a === "object") {
					t.administration.set("watcher",t);
					var med = t.medStore.get(t.administration.medId);
					var qt = t.quantityTimingStore.get(t.administration.qtId);
					var pat = t.patientStore.get(t.administration.patId);

					t.grid.setStore(new Memory({data: t.quantityTimingStore.query({medId:med.id})}));
					t.medication = med;
					lang.hitch(t,medicationDetail.prototype.updatePreviousAdminGrid)();

					domStyle.set(t.qtGridNode,"display","none");
					dom.byId('schedulingLinkIndicator').innerHTML = '+';
					domStyle.set(t.adminGridNode,"display","none");
					dom.byId('administrationsLinkIndicator').innerHTML = '+';
					
					var currentTime = new Date();

					var prnReasonStore = new Memory();
					var found = false;
					arrayUtils.forEach(med.medReasons,function(reason) {
						prnReasonStore.put({
							id:reason,
							name:reason
						});
						if (a.prnReason === reason) {
							found = true;
						}
					});
					if (!found && a.prnReason !== "") {
						prnReasonStore.put({
							id:a.prnReason,
							name:a.prnReason
						});
					}
					t.prnReasonBox.set("store",prnReasonStore);
					
					domStyle.set(t.errorReasonDiv,"display","none");
					t.errorReasonBox.set("required",false);
					t.quantityGivenBox.set("value","",false);
					t.unitGivenBox.set("value","",false);
					t.showIv(false,false,true);
					if (a.action === "add") {
						if (!a.scheduledEvent || a.scheduledEvent.status === 1 || a.scheduledEvent.status === 4) {
							a.date = locale.format(currentTime, {datePattern: "yyyy-MM-dd", selector: "date"});
							a.time = TimeSpinner.prototype.format(currentTime);
						} else {
							a.date = locale.format(a.scheduledEvent.start, {datePattern: "yyyy-MM-dd", selector: "date"});
							a.time = TimeSpinner.prototype.format(a.scheduledEvent.start);
						}
						if (a.scheduledEvent) {
							a.selectedScheduledEvent = a.scheduledEvent;
						}
						
						t.addedThisSession = true;
						t.set("disabled",false);
						t.showIv((med.isIv !== "" ? med.isIv : med.defaultIv));
						t.ivCheckBox.set("value",(med.isIv !== "" ? med.isIv : med.defaultIv),false);
						t.ivCheckBox.set("disabled",(med.isIv !== ""));
						//t.notGivenReasonBox.set("disabled",true);
						
						if (qt) {
							var pieces = qt.cDosing.toString().split("^");
							if (pieces.length === 1 || (pieces.length === 2 && pieces[1] === "")) {
								a.dose = pieces[0];
							}
						}
						if (a.dose === "") {
							a.dose = med.dose;
						}
						if (a.scheduledEvent) {
							a.admInst = a.scheduledEvent.admInst || "";
							a.comment = a.scheduledEvent.comment || "";
						}
						if (a.admInst === "") {
							a.admInst = med.admInst;
						}
						if (med.isPRN(t.administration.qtId) && prnReasonStore.data.length === 1) {
							a.prnReason = prnReasonStore.data[0].id;
						}
						
						if (qt && qt.dQuantity) {
							var time = currentTime;
							if (a.scheduledEvent && a.scheduledEvent.status === 4) {
								time = a.scheduledEvent.start;
							}
							
							var end = date.add(time,"second",t.unitStore.get(qt.dUnit).seconds*qt.dQuantity);
							a.sD = locale.format(time, {datePattern: "yyyy-MM-dd", selector: "date"});
							a.sT = TimeSpinner.prototype.format(time);
							a.eD = locale.format(end, {datePattern: "yyyy-MM-dd", selector: "date"});
							a.eT = TimeSpinner.prototype.format(end);
						}
						
						t.startTimeBox.set("value",null,false);
						t.endTimeBox.set("value",null,false);
						t.getAdministration("",true); // Execute field logic
					} else if (a.action === "view") {
						t.showIv(a.isIv,true);
						t.ivCheckBox.set("value",a.isIv,false);
						t.set("disabled",true);
					} else if (a.action === "error") {
						domStyle.set(t.errorReasonDiv,"display","");
						t.showIv(a.isIv,true);
						t.ivCheckBox.set("value",a.isIv,false);
						t.errorReasonBox.set("required",true);
						t.err = 1;
						t.set("disabled",true);
						t.getAdministration("",true); // Execute field logic
					} else if (a.action === "edit") {
						t.set("disabled",false);
						t.showIv(a.isIv,true,true);
						t.ivCheckBox.set("value",a.isIv,false);
						t.getAdministration(t.administration.status,true); // Execute field logic
						if (a.status === 3) {
							t.ivCheckBox.set("checked",false);
							t.ivCheckBox.set("disabled",true);
							t.administration.isIv = 0;
						}
					}
					
					// Carry forward Med Given Logic
					if (a.action === "add" || t.addedThisSession) {
						if (!med.allowGenSub) {
							a.medGiven = med.name;
							a.medGivenID = med.fdbId;
							t.findButton.set("disabled",true);
							t.medGivenBox.set("disabled",true);
						} else {
							if (t.settings.CarryFwdMedGiven) {
								var curTime = new Date();
								var firstAdminTime = null;
								var results = t.cacheManager.eventCache.query({medId:med.id,status:2,err:0},{sort:[{attribute:"start",descending:true}]});
								for (var i = 0; i < results.length; i++) {
									var firstAdminTime = results[i].start;
									if (results[i].medGiven && results[i].start <= curTime) {
										a.medGiven = results[i].medGiven;
										a.medGivenID = results[i].medGivenID;
										break;
									}
								}
								
								if (a.medGiven === "" && med.lMGTime && (firstAdminTime === null || (locale.parse(med.lMGTime,{selector:"date",datePattern:"yyyy-MM-dd HHmm"}) < firstAdminTime))) {
									// Only use the last med given from the server if we know that the administration hasn't been errored on the UI side
									a.medGiven = med.lMG;
									a.medGivenID = med.lMGID;
								}
							}
							if (a.medGiven === "") {
								a.medGiven = med.name;
								a.medGivenID = med.fdbId;
							}
						}
					} else if (a.action === "edit" && med.allowGenSub) {
						t.findButton.set("disabled",false);
						t.medGivenBox.set("disabled",false);
					} else {
						t.findButton.set("disabled",true);
						t.medGivenBox.set("disabled",true);
					}
					
					// Prep fields
					if (a.date === "") {a.date = null};
					if (a.sD === "") {a.sD = null};
					if (a.eD === "") {a.eD = null};
					if (a.expDate === "") {a.expDate = null};
					
					t.updateTags();
					t.schTimeNode.innerHTML = "";
					if (t.addedThisSession) {
						var lastAdministrationTime = a.med.getMostMostRecentAdminTimeDisplay(a.qtId);
						t.lastAdministrationNode.innerHTML = (lastAdministrationTime !== "" ? "Last Administration: "+lastAdministrationTime : "");
					}
					t.updateScheduledTime();
					t.routeNode.innerHTML = med.route;
					t.doseNode.innerHTML = med.dose;
					domStyle.set(t.medDoseContainer,"display",(med.dose !== "" ? "" : "none"));
					domStyle.set(t.durationDiv,"display","none");
					if (qt && qt.dQuantity) {
						var unit = t.unitStore.get(qt.dUnit).name;
						unit = (qt.dQuantity > 1 ? unit : unit.substring(0,unit.length-1));
						t.durationNode.innerHTML = qt.dQuantity+" "+unit;
						domStyle.set(t.durationDiv,"display","");
					}
					t.quantityConditionNode.innerHTML = (qt ? qt.conditionalDosing : "");
					t.holdDateTimeNode.innerHTML = (med.hD ? locale.format(locale.parse(med.hD, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.hT : "");
					t.discontinueDateTimeNode.innerHTML = (med.dD ? locale.format(locale.parse(med.dD, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.dT : "");
					
					domStyle.set(t.hospiceDiv,"display","none");
					if (pat.hosp) {
						domStyle.set(t.hospiceDiv,"display","");
						t.unitNode.innerHTML = pat.unit;
						t.roomNode.innerHTML = pat.room;
						t.bedNode.innerHTML = pat.bed;
					}

					t.strengthNode.innerHTML = med.strength;
					domStyle.set(t.divStrength,"display",(med.strength === "" ? "none" : ""));
					//t.volumeNode.innerHTML = med.volume;
					//domStyle.set(t.divVolume,"display",(med.volume === "" ? "none" : ""));
					t.medFormNode.innerHTML = med.form;
					domStyle.set(t.divMedForm,"display",(med.form === "" ? "none" : ""));
					
					t.administrationInstructionsBox.set("value",a.admInst,false);
					t.adminDateBox.set("value",a.date,false);
					t.adminTimeBox.set("value",locale.parse(a.time.toString(),{selector:"time", timePattern:"HHmm"}),false);
					t.errorReasonBox.set("value",a.errReason,false);
					t.adminByBox.set("value",a.adminBy,false);
					t.witnessesMultiSelect.set("value",a.witnesses,false);
					t.reactionBox.set("value",a.reaction,false);
					t.prnReasonBox.set("value",a.prnReason,false);
					t.notGivenReasonBox.set("value",a.notGivenReason,false);
					t.startDateBox.set("value",a.sD,false);
					t.startTimeBox.set("value",locale.parse(a.sT.toString(),{selector:"time", timePattern:"HHmm"}),false);
					t.endDateBox.set("value",a.eD,false);
					t.totalTimeBox.set("value",a.tT,false);
					t.endTimeBox.set("value","",(a.action === "add" || a.action === "edit" ? true : false));
					t.endTimeBox.set("value",locale.parse(a.eT.toString(),{selector:"time", timePattern:"HHmm"}),(a.action === "add" || a.action === "edit" ? true : false)); // Cause onChange to fire here if adding/editing
					t.commentBox.set("value",a.comment,false);
					
					t.medGivenBox.set("value",a.medGiven,false);
					if (t.addedThisSession) {
						t.quantityGivenBox.set("value","",false);
						a.doseUnit = (qt && med.version == 2 ? qt.unit : "");
						t.unitGivenBox.set("value",a.doseUnit,false);
					} else {
						t.quantityGivenBox.set("value",a.doseQuantity,false);
						t.unitGivenBox.set("value",a.doseUnit,false);
					}
					t.expirationDateBox.set("value",a.expDate,false);
					t.lotNumberBox.set("value",a.lotNum,false);
					
					//t.calculateIVRate("totalTime");
					
					setTimeout(function() {
						t.modified = false;
					},0);
				}
			},
			
			updateScheduledTime: function() {
				var t = this;
				var a = t.administration;
				t.schTimeNode.innerHTML = "";
				if (t.addedThisSession && a.scheduledEvent) {
					if (a.scheduledEvent) {
						var time = t._getStartTime();
						if (!time) {
							time = t._getAdminTime();
						}
						
						var scheduledTime = a.scheduledEvent.getScheduledTimeDisplayForAdmin(time);
						t.schTimeNode.innerHTML = (scheduledTime !== "" ? "Scheduled Time: "+scheduledTime : "");
					}
				}
			},
			
			checkForWarnings: function() {
				var t = this;
				var a = t.administration;
				if (t.addedThisSession) {
					a.outsideLimits = false;
					
					var lastGivenMessage = "";
					var lastGiven = a.med.getMostRecentAdministration("",true);
					if (lastGiven) {
						var minutes = date.difference(lastGiven.end,new Date(),"minute");
						var interval = "";
						if (minutes < 60) {
							interval = minutes+" minute"+(minutes > 1 ? "s" : "");					
						} else if (minutes < 1440) {
							interval = Math.floor(minutes / 60);
							interval += " hour"+(interval > 1 ? "s" : "");
						} else {
							interval = Math.floor(minutes / 1440);
							interval += " day"+(interval > 1 ? "s" : "");
						}
						lastGivenMessage = " This medication order was last given "+interval+" ago at "+formatFullDate(lastGiven.end)+".";
					} else {
						lastGivenMessage = " This medication order has not been given within the last 3 days.";
					}
					
					// Check that we have data on the client side covering the date range
					var startTime = a.getStartTime();
					var endTime = date.add(a.getEndTime(),"day",1);
					if (startTime && endTime && a.patId) {
						t.cacheManager.getData([a.patId],startTime,endTime).then(function() {
							var warning = false;
							var message = "";
							
							if (a.selectedScheduledEvent) {
								// Check whether the scheduled event is within the early/late offsets
								var times = a.selectedScheduledEvent.getEarlyLateAdminTimes();
							
								var relativeTime = "early";
								if (a.selectedScheduledEvent.start < startTime) {
									relativeTime = "late";
								}
							
								if (times.early <= startTime && times.late > startTime) { 
									if (a.selectedScheduledEvent.windowStart > startTime || a.selectedScheduledEvent.missed <= startTime) {
										message += "Charting scheduled administration at "+formatFullDate(a.selectedScheduledEvent.start)+" "+relativeTime+"."+(a.forceSelection ? "" : " <a href='#' onclick='cancelScheduledAdmin()'>Cancel</a>") + lastGivenMessage;
									} else if (!a.forceSelection) {
										a.selectedScheduledEvent = null;
									}
								} else {
									if (a.forceSelection) {
										warning = true;
										message += "Charting scheduled administration at "+formatFullDate(a.selectedScheduledEvent.start)+" "+relativeTime+"."+(a.forceSelection ? "" : " <a href='#' onclick='cancelScheduledAdmin()'>Cancel</a>") + lastGivenMessage;
										a.outsideLimits = true;
									} else {
										a.selectedScheduledEvent = null;
									}
								}
							}
							
							if (!a.selectedScheduledEvent) {
								var schEvent = a.getOverlappingSchEvent();
								
								if (!schEvent) {
									// PRN or not schedulable?
									if (!a.qt || !a.qt.schedulable()) {
										
										if (a.qt && a.qt.priority === 3 && lastGiven) {
											// This is a PRN QT set. Any warnings?
											
											//How many times in last 24 hours?
											var results = t.cacheManager.eventCache.query({medId:a.medId,status:2});
											var count = 0;
											for (var i = 0; i < results.length; i++) {
												var admin = results[i];
												if (!admin.err && (admin.getStartTime() < new Date()) && (admin.getStartTime() > date.add(new Date(),"day",-1))) {
													count++;
												}
											}
											
											var qt = a.qt;
											var quantity = "";
											var unit = "H";
											var timesPerDay = "";
											if (qt.rQuantity) {
												quantity = qt.rQuantity;
												unit = qt.rUnit;
												timesPerDay = t.unitStore.get("D").seconds / quantity * t.unitStore.get(unit).seconds;
											} else if (qt.rPattern) {
												var pattern = qt.rPattern;
												if (pattern === 1) {
													timesPerDay = 2;
												} else if (pattern === 2) {
													timesPerDay = 3;
												} else if (pattern === 3) {
													timesPerDay = 4;
												} else if (pattern === 4) {
													timesPerDay = qt.expTimes.toString().split(",").length;
												} else if (pattern === 5 || pattern === 7 || pattern === 8) {
													timesPerDay = 1;
												} else if (pattern === 6) {
													timesPerDay = t.settings.DefaultTimeQSHIFT.toString().split(",").length;
												} else if (pattern === 10) {
													timesPerDay = 0.01;
												}
												quantity = 24/timesPerDay; // Quantity in hours
											} else if (qt.meal) {
												if (qt.meal === "A") {
													timesPerDay = 3;
												} else {
													timesPerDay = 1;
												}
												quantity = 24/timesPerDay; // Quantity in hours
											} else if (qt.expTimes) {
												timesPerDay = qt.expTimes.toString().split(",").length;
												quantity = 24/timesPerDay; // Quantity in hours
											}
											
											if (count > 0) {
												message = "This medication order has already been given "+count+" time"+(count > 1 ? "s" : "")+" in the past 24 hours.";
											}
											if (quantity) {
												var seconds = t.unitStore.get(unit).seconds * quantity;
												if (date.difference(lastGiven.start,startTime,"second") < seconds || count >= timesPerDay) {
													warning = true;
												}
											}
										}
										
										// Get info about outstanding user scheduled events here
										var results = [];
										if (a.qt) {
											results = t.cacheManager.eventCache.query({qtId:a.qtId});
										} else {
											results = t.cacheManager.eventCache.query({medId:a.medId});
										}
										count = 0;
										for (var i = 0; i < results.length; i++) {
											var schEvent = results[i];
											if (!schEvent.err && (schEvent.status === 1 || schEvent.status === 4) && (schEvent.windowStart < new Date()) && (schEvent.end > date.add(new Date(),"day",-3))) {
												count++;
											}
										}
										
										if (count > 0) {
											var prepended = false;
											if (message !== "") {
												message += lastGivenMessage+"<br><br>";
												prepended = true;
											}
											message += count + " outstanding administration event"+(count > 1 ? "s" : "")+" present for this medication within the past 3 days.";
											if (!prepended) {
												message += lastGivenMessage;
											}
										} else if (a.qt && a.qt.priority === 3) {
											message += lastGivenMessage;
										}
									} else {
										// Normal, scheduled event
										message = "Administration of this medication is outside of the administration schedule."+lastGivenMessage;
										warning = true;
									}
									
									// Check for early/late scheduled event qualifiers
									var schEvents = t.cacheManager.eventCache.query({medId:a.medId});
									var schCount = 0;
									for (var i = 0; i < schEvents.length; i++) {
										var schEvent = schEvents[i];
										if (!schEvent.err && (schEvent.status === 1 || schEvent.status === 4) && (schEvent.windowStart > startTime || schEvent.missed <= startTime)) {
											var times = schEvent.getEarlyLateAdminTimes();
											if (times.early <= startTime && times.late > startTime) {
												if (schCount === 0 && message !== "") {
													message += "<br><br>";
												} else if (schCount > 0) {
													message += "<br>";
												}
												
												var relativeTime = "early";
												if (schEvent.start < startTime) {
													relativeTime = "late";
												}
												message += "Administration scheduled for "+formatFullDate(schEvent.start)+". <a href='#' onclick='selectScheduledAdmin(\""+schEvent.id+"\")'>Administer "+relativeTime+"</a>";
												schCount++;
											}
										}
									}
								} else {
									a.scheduledEvent = schEvent;
									a.selectedScheduledEvent = null;
									t.updateScheduledTime();
									message = message + lastGivenMessage;
								}
							};
							
							if (startTime > new Date()) {
								warning = true;
								if (message !== "") {
									message += "<br><br>";
								}
								message += "You have entered a future administration time.";
							}
							
							topic.publish("overlay/message",(warning ? "warning" : "information"),message);
						});
					}
				}
			},
			
			updateTags: function() {
				var t = this;
				var med = t.medStore.get(t.administration.medId);
				t.medNode.innerHTML = med.name + t.administration.statusTag + med.getPRNTag(t.administration.qtId) + med.getInfoIcon();
			},
			
			getActions: function(functions,inAdminSession) {
				var t = this;
				var actions = [];
				var action = t.administration.action;
				var status = t.administration.status;
				
				if (action === "add") {
					if (!inAdminSession) {actions.push({actionId:"cancel",type:"button",label:"Cancel",onClick:function() {functions.actionCancelUpdateAdmin()}})};
					actions.push({actionId:"given",type:"button",label:"Given",onClick:function() {functions.actionUpdateQueue(2)}});
					actions.push({actionId:"notGiven",type:"button",label:"Not Given",onClick:function() {functions.actionUpdateQueue(3)}});
					actions.push({actionId:"inProgress",type:"button",label:"In Progress",onClick:function() {functions.actionUpdateQueue(5)}});
					// Add modify med actions here too as applicable
				} else if (action === "edit") {
					if (!inAdminSession) {actions.push({actionId:"cancel",type:"button",label:"Cancel",onClick:function() {functions.actionCancelUpdateAdmin()}})};
					if (status !== "") {
						if (inAdminSession) {
							actions.push({actionId:"continue",type:"button",label:"Continue",onClick:function() {functions.actionUpdateQueue(status)}})
						} else {
							actions.push({actionId:"save",type:"button",label:"Save",onClick:function() {functions.actionUpdateQueue(status)}})
						}
					};
					if (status !== 2) {actions.push({actionId:"given",type:"button",label:"Given",onClick:function() {functions.actionUpdateQueue(2)}})};
					if (status !== 3) {actions.push({actionId:"notGiven",type:"button",label:"Not Given",onClick:function() {functions.actionUpdateQueue(3)}})};
					if (status !== 5) {actions.push({actionId:"inProgress",type:"button",label:"In Progress",onClick:function() {functions.actionUpdateQueue(5)}})};
					// Add modify med actions here too as applicable
				} else if (action === "view") {
					actions.push({actionId:"edit",type:"button",label:"Edit",onClick:function() {functions.actionInitiateEdit()}});
					if (status !== 5) {
						actions.push({actionId:"error",type:"button",label:"Error",onClick:function() {functions.actionInitiateError()}});
					}
				} else if (action === "error") {
					actions.push({actionId:"cancel",type:"button",label:"Cancel",onClick:function() {functions.actionCancelUpdateAdmin()}});
					actions.push({actionId:"save",type:"button",label:"Save",onClick:function() {functions.actionUpdateQueue(status)}});
					// Add modify med actions here too as applicable
				}
				actions.push({actionId:"history",type:"button",label:"Med History",onClick:function() {functions.actionShowHistory("med")}});
				// Show Event Detail Report action too
				if (status !== 1 && status !== 4 && status !== "" && !t.addedThisSession) {
					actions.push({actionId:"eventHistory",type:"button",label:"Event History",onClick:function() {functions.actionShowHistory("event")}});
				}
				
				return actions;
			},
			
			requireTiming: function(require) {
				var t = this;
				t.startDateBox.set("required",require);
				t.startTimeBox.set("required",require);
				t.endDateBox.set("required",require);
				t.endTimeBox.set("required",require);
				t.totalTimeBox.set("required",require);
			},
			
			parseTime: function(time) {
				var ret = locale.parse(time.toString(),{selector:"time", timePattern:"HHmm"});
				ret.setMinutes(ret.getMinutes() - ret.getTimezoneOffset());
				return ret;
			},
			
			_modifyPost: function(timeChange) {
				var t = this;
				var a = t.administration;
				
				var disableAdminTime = false;
				var disableTimes = false;
				var requireTimes = false;
				if (t.startDateBox.get("value") !== null || 
					t.startTimeBox.get("value") !== null || 
					t.endDateBox.get("value") !== null || 
					t.endTimeBox.get("value") !== null || 
					t.totalTimeBox.get("value") !== "" || 
					a.isIv ||
					(a.qt && a.qt.dQuantity && a.status !== 3)) 
				{
					var endDate = t.endDateBox.get("value");
					var endTime = t.endTimeBox.get("value");
					if (endDate) {
						t.adminDateBox.set("value",endDate,false);
						a.date = locale.format(endDate, {datePattern: "yyyy-MM-dd", selector: "date"});
					} 
					if (endTime) {
						t.adminTimeBox.set("value",endTime,false);
						a.time = TimeSpinner.prototype.format(endTime);
					}
					disableAdminTime = true;
					requireTimes = true;
				}
				
				t.adminDateBox.set("disabled",disableAdminTime);
				t.adminTimeBox.set("disabled",disableAdminTime);
				
				t.adminDateBox.set("required",!requireTimes);
				t.adminTimeBox.set("required",!requireTimes);
				t.startDateBox.set("required",requireTimes);
				t.startTimeBox.set("required",requireTimes);
				t.endDateBox.set("required",requireTimes);
				t.endTimeBox.set("required",requireTimes);
				t.totalTimeBox.set("required",requireTimes);
				
				
				t.endDateBox.validate();
				if (timeChange) {
					t.checkForWarnings();
					t.updateScheduledTime();
				}
				
				t.modified = true;
			},
			
			_modifyDate: function(value) {
				var t = this;
				if (value) {
					t.administration.date = locale.format(value, {datePattern: "yyyy-MM-dd", selector: "date"});
				} else {
					t.administration.date = "";
				}
				t._modifyPost(true);
			},
			
			_modifyTime: function(value) {
				var t = this;
				if (value) {
					t.administration.time = TimeSpinner.prototype.format(value);
				} else {
					t.administration.time = "";
				}
				t._modifyPost(true);
			},
			
			_modifyTotalTime: function(value) {
				var t = this;
				if (t.totalTimeBox.isValid()) {
					t.administration.set("tT",value);
					t.calculateIVRate("totalTime");
				} else {
					t.administration.set("tT","");
				}
				t._modifyPost(true);
			},
			
			_modifyComment: function(value) {
				var t = this;
				t.administration.comment = value;
				t._modifyPost();
			},
			
			_modifyAdminBy: function(value) {
				var t = this;
				t.administration.adminBy = value;
				t._modifyPost();
			},
			
			_modifyWitnesses: function(value) {
				var t = this;
				t.administration.witnesses = t.witnessesMultiSelect.get("value");
				t._modifyPost();
			},
			
			_modifyReaction: function(value) {
				var t = this;
				t.administration.reaction = value;
				t._modifyPost();
			},
			
			_modifyPRNReason: function(value) {
				var t = this;
				t.administration.prnReason = value;
				t._modifyPost();
			},
			
			_modifyNotGivenReason: function(value) {
				var t = this;
				t.administration.notGivenReason = value;
				t._modifyPost();
			},
			
			_modifyIVType: function(value) {
				var t = this;
				t.administration.ivType = value;
				t._modifyPost();
			},
			
			/*_modifyIVVolume: function(value) {
				var t = this;
				t.administration.ivVolume = value;
				t._modifyPost();
			},*/
			
			_modifyIVSize: function(value) {
				var t = this;
				t.administration.ivSize = value;
				t._modifyPost();
			},
			
			_modifyIVDropFactor: function(value) {
				var t = this;
				if (value !== "") {
					t.ivFlowRateBox.set("disabled",true);
					t.ivFlowRateBox.set("value","");
				} else {
					t.ivFlowRateBox.set("disabled",false);
				}
				t.administration.ivdf = value;
				t._modifyPost();
				t.calculateIVRate("dropFactor");
			},
			
			_modifyIVDripRate: function(value) {
				var t = this;
				if (value !== "" || t.ivDropFactorBox.get("value") !== "") {
					t.ivFlowRateBox.set("disabled",true);
				} else {
					t.ivFlowRateBox.set("disabled",false);
				}
				t.calculateIVRate("dripRate");
				t.administration.ivdr = value;
				t._modifyPost();
			},
			
			_modifyIVFlowRate: function(value) {
				var t = this;
				if (value !== "") {
					t.ivDripRateBox.set("disabled",true);
				} else {
					t.ivDripRateBox.set("disabled",false);
				}
				t.calculateIVRate("flowRate");
				t.administration.ivfr = value;
				t._modifyPost();
			},
			
			_modifyStartDate: function(value) {
				var t = this;
				if (value) {
					t.administration.set("sD",locale.format(value, {datePattern: "yyyy-MM-dd", selector: "date"}));
					t.calculateIVRate("totalTime");
				} else {
					t.administration.set("sD","");
				}
				t._modifyPost(true);
			},
			
			_modifyStartTime: function(value) {
				var t = this;
				if (value) {
					t.administration.set("sT",TimeSpinner.prototype.format(value));
					t.calculateIVRate("totalTime");
				} else {
					t.administration.set("sT","");
				}
				t._modifyPost(true);
			},
			
			_modifyEndDate: function(value) {
				var t = this;
				if (value) {
					t.administration.set("eD",locale.format(value, {datePattern: "yyyy-MM-dd", selector: "date"}));
					t.calculateIVRate("totalTime");
				} else {
					t.administration.set("eD","");
				}
				t._modifyPost(true);
			},
			
			_modifyEndTime: function(value) {
				var t = this;
				if (value) {
					t.administration.set("eT",TimeSpinner.prototype.format(value));
					t.calculateIVRate("totalTime");
				} else {
					t.administration.set("eT","");
				}
				t._modifyPost(true);
			},
			
			_getTotalTime: function() {
				var t = this;
				var time = null;
				if (t.totalTimeBox.isValid() && t.totalTimeBox.get("value") !== "") {
					var pieces = t.totalTimeBox.get("value").toString().split(":");
					var time = new Date(1970,0,1,pieces[0],pieces[1],0);
					//time = locale.parse(t.totalTimeBox.get("value"),{selector:"time",timePattern:"HH:mm"});
					time = t._offsetTimezone(time);
				}
				return time;
			},
			
			_getDurationAsDate: function() {
				var t = this;
				var time = null;
				var a = t.administration;
				if (a.qt && a.qt.dQuantity) {
					var time = new Date(1970,0,1,0,0,t.unitStore.get(a.qt.dUnit).seconds*a.qt.dQuantity);
					time = t._offsetTimezone(time);
				}
				return time;
			},
			
			_getAdminTime: function(includeInvalid) {
				var t = this;
				var time = null;
				if ((includeInvalid || t.adminDateBox.isValid()) && t.adminDateBox.get("value") !== null && t.adminTimeBox.get("value") !== null) {
					try {
						var adminString = locale.format(t.adminDateBox.get("value"),{selector:"date",datePattern:"MM/dd/yyyy"});
						adminString += " "+locale.format(t.adminTimeBox.get("value"),{selector:"time",timePattern:"HH:mm"});
						var time = locale.parse(adminString,{selector:"date",datePattern:"MM/dd/yyyy HH:mm"});
					} catch (exception) {}
				}
				return time;
			},
			
			_getStartTime: function(includeInvalid) {
				var t = this;
				var time = null;
				if ((includeInvalid || t.startDateBox.isValid()) && t.startDateBox.get("value") !== null && t.startTimeBox.get("value") !== null) {
					try {
						var startString = locale.format(t.startDateBox.get("value"),{selector:"date",datePattern:"MM/dd/yyyy"});
						startString += " "+locale.format(t.startTimeBox.get("value"),{selector:"time",timePattern:"HH:mm"});
						var time = locale.parse(startString,{selector:"date",datePattern:"MM/dd/yyyy HH:mm"});
					} catch (exception) {}
				}
				return time;
			},
			
			_getEndTime: function(includeInvalid) {
				var t = this;
				var time = null;
				if ((includeInvalid || t.endDateBox.isValid()) && t.endDateBox.get("value") !== null && t.endTimeBox.get("value") !== null) {
					try {
						var endString = locale.format(t.endDateBox.get("value"),{selector:"date",datePattern:"MM/dd/yyyy"});
						endString += " "+locale.format(t.endTimeBox.get("value"),{selector:"time",timePattern:"HH:mm"});
						var time = locale.parse(endString,{selector:"date",datePattern:"MM/dd/yyyy HH:mm"});
					} catch (exception) {}
				}
				return time;
			},
			
			_offsetTimezone: function(date) {
				date.setMinutes(date.getMinutes() - date.getTimezoneOffset());
				return date;
			},
			
			_modifyErrorReason: function(value) {
				var t = this;
				t.administration.errReason = value;
				t._modifyPost();
			},
			
			_modifyAdministrationInstructions: function(value) {
				var t = this;
				t.administration.admInst = value;
				t._modifyPost();
			},
			
			_modifyMedGiven: function(value) {
				var t = this;
				t.administration.set("medGiven",value);
				t._modifyPost();
			},
			
			_modifyQuantityGiven: function(value) {
				var t = this;
				var a = t.administration;
				t.calculateIVRate("amountGiven");
				t.administration.doseQuantity = value;
				t.administration.dose = (isNaN(value) ? "" : value)+" "+t.administration.doseUnit;
				t._modifyPost();
			},

			_modifyUnitGiven: function(value) {
				var t = this;
				t.administration.doseUnit = value;
				t.administration.dose = (isNaN(t.administration.doseQuantity) ? "" : t.administration.doseQuantity)+" "+t.administration.doseUnit;
				t._modifyPost();
			},
			
			calculateIVRate: function(field) {
				var t = this;
				var a = t.administration;
				if (a.isIv) {
					t.ivFlowRateBox.set("disabled",false);
					t.ivFlowRateBox.set("required",true);
					var amountGiven = (t.quantityGivenBox.isValid() ? parseFloat(t.quantityGivenBox.get("value")) : "");
					var totalTime = (t.totalTimeBox.isValid() ? t.totalTimeBox.get("value") : "");
					var hours = "";
					var minutes = "";
					if (totalTime) {
						var pieces = totalTime.toString().split(":");
						minutes = (parseInt(pieces[0]) * 60) + parseInt(pieces[1]);
						hours = minutes / 60;
					}
					var dropFactor = (t.ivDropFactorBox.isValid() ? parseFloat(t.ivDropFactorBox.get("value")) : "");
					var flowRate = (t.ivFlowRateBox.isValid() ? parseFloat(t.ivFlowRateBox.get("value")) : "");
					var dripRate = (t.ivDripRateBox.isValid() ? parseFloat(t.ivDripRateBox.get("value")) : "");
					
					if (field === "totalTime" || field === "dropFactor") {
						if (totalTime) {
							if (amountGiven !== "") {
								if (dropFactor) {
									var dripRate = Math.round((amountGiven * dropFactor * 100) / (minutes)) / 100;
									t.ivDripRateBox.set("value",dripRate,false);
									a.ivdr = dripRate;
									clearFlowRate();
								} else {
									var flowRate = Math.round((amountGiven / hours) * 100) / 100;
									t.ivFlowRateBox.set("value",flowRate,false);
									a.ivfr = flowRate;
									clearDripRate();
								}
							} else if (flowRate !== "") {
								amountGiven = Math.round((flowRate * hours) * 100) / 100;
								t.quantityGivenBox.set("value",amountGiven,false);
								a.dose = amountGiven;
								clearDripRate();
							} else if (dripRate !== "" && dropFactor) {
								amountGiven = Math.round((minutes * dripRate * 100) / (dropFactor)) / 100;
								t.quantityGivenBox.set("value",amountGiven,false);
								a.dose = amountGiven;
								clearFlowRate();
							} 
						}
					} else if (field === "dripRate" || field === "flowRate") {
						if (totalTime) {
							if (dropFactor && dripRate !== "") {
								amountGiven = Math.round((minutes * dripRate * 100) / (dropFactor)) / 100;
								t.quantityGivenBox.set("value",amountGiven,false);
								a.dose = amountGiven;
								clearFlowRate();
							} else if (flowRate !== "") {
								amountGiven = Math.round((flowRate * hours) * 100) / 100;
								t.quantityGivenBox.set("value",amountGiven,false);
								a.dose = amountGiven;
								clearDripRate();
							}
						}
					} else if (field === "amountGiven") {
						if (totalTime && amountGiven !== "") {
							if (dropFactor) {
								var dripRate = Math.round((amountGiven * dropFactor * 100) / (minutes)) / 100;
								t.ivDripRateBox.set("value",dripRate,false);
								a.ivdr = dripRate;
								clearFlowRate();
							} else {
								var flowRate = Math.round((amountGiven / hours) * 100) / 100;
								t.ivFlowRateBox.set("value",flowRate,false);
								a.ivfr = flowRate;
								clearDripRate();
							}
						}
					}
					
					function clearFlowRate() {
						t.ivFlowRateBox.set("value","",false);
						t.ivFlowRateBox.set("disabled",true);
						a.ivfr = "";
						t.ivFlowRateBox.set("required",false);
						t.ivDripRateBox.set("required",true);
					}
					function clearDripRate() {
						t.ivDripRateBox.set("value","",false);
						t.ivDripRateBox.set("disabled",true);
						a.ivdr = "";
						t.ivDropFactorBox.set("value","",false);
						a.ivdf = "";
						t.ivFlowRateBox.set("required",true);
						t.ivDripRateBox.set("required",false);
					}
				}
			},
			
			_modifyExpirationDate: function(value) {
				var t = this;
				if (value) {
					t.administration.expDate = locale.format(value, {datePattern: "yyyy-MM-dd", selector: "date"});
				} else {
					t.administration.expDate = "";
				}
				t._modifyPost();
			},
			
			_modifyLotNumber: function(value) {
				var t = this;
				t.administration.lotNum = value;
				t._modifyPost();
			},
			
			_modifyIVSiteGiven: function(value) {
				var t = this;
				t.administration.siteGiven = value;
				t._modifyPost();
			},
			
			_modifyIVSiteAppearance: function(value) {
				// value is an event
				var t = this;
				var value = t.ivSiteAppearanceMultiSelect.get("value");

				var oldValue = t.administration.ivSiteAppearance.toString().split(",");
				if (arrayUtils.indexOf(value,5) !== -1 && value.length !== 1) {
					var index = arrayUtils.indexOf(oldValue,5);
					if (index === -1) {
						value = [5];
						t.ivSiteAppearanceMultiSelect.set("value",value);
					} else {
						index = arrayUtils.indexOf(value,5);
						value.splice(index,1);
						t.ivSiteAppearanceMultiSelect.set("value",value);
					}	
				}
				
				var includeOther = (arrayUtils.indexOf(value,6) !== -1);
				if (!includeOther) {
					t.ivSiteAppearanceOtherBox.set("value","");
				}
				t.ivSiteAppearanceOtherBox.set("required",includeOther);
				t.ivSiteAppearanceOtherBox.set("disabled",!includeOther);

				t.administration.ivSiteAppearance = value.join();
				t._modifyPost();
			},
			
			_modifyIVSiteAppearanceOther: function(value) {
				var t = this;
				t.administration.ivSiteAppearanceOther = value;
				t._modifyPost();
			},
			
			_modifyIVCheckBox: function(value) {
				var t = this;
				t.administration.isIv = (value ? 1 : 0);
				t.showIv(value);
				t._modifyPost();
			},
			
			searchForMed: function() {
				var t = this;
				for(var i = t.medSearchSelect.options.length - 1; i >= 0; i--){
					t.medSearchSelect.remove(i);
				}
				
				var query = t.medGivenBox.get("value");
				var resultObj = t.medSearch(query);
				if (resultObj.error !== "") {
					t.medSearchNoResults.innerHTML = resultObj.error;
					domStyle.set(t.medResultsNode,"display","none");
					domStyle.set(t.medNoResultsNode,"display","");
				} else {
					var resultsArray = resultObj.medString.split("**");
					for (var i=0; i<resultsArray.length; i++) {
						var option = document.createElement("option");
						var values = resultsArray[i].split("~");
						option.value = values[1];
						option.innerHTML = values[0];
						t.medSearchSelect.appendChild(option);
					}
					domStyle.set(t.medResultsNode,"display","");
					domStyle.set(t.medNoResultsNode,"display","none");
				}
			},
			
			selectMed: function() {
				var t = this;
				t.administration.medGivenID = t.medSearchSelect.value;
				t.medGivenBox.set("value",t.medSearchSelect[t.medSearchSelect.selectedIndex].text);
				popup.close(t.medSearchDialog);
				t.quantityGivenBox.focus();
			},
			
			selectScheduledAdmin: function(schEventId) {
				var t = this;
				var schEvent = t.cacheManager.eventCache.get(schEventId);
				if (schEvent) {
					t.administration.selectedScheduledEvent = schEvent;
					t.administration.scheduledEvent = schEvent;
					t.checkForWarnings();
					t.updateScheduledTime();
				}
			},
			
			cancelScheduledAdmin: function() {
				var t = this;
				t.administration.selectedScheduledEvent = null;
				t.checkForWarnings();
			},
			
			scrollToTop: function() {
				this.scrollableDiv.scrollTop = 0;
			},
			
			close: function() {
				var t = this;
				t.adminTimeBox.closeDropDown();
				t.startTimeBox.closeDropDown();
				t.endTimeBox.closeDropDown();
			},
			
			resize: function() {
				var t = this;
				t.inherited(arguments);
				t.grid.resize();
				t.adminGrid.resize();
				var containerBox = domGeom.getContentBox(t.containerNode);
				var titleBox = domGeom.getContentBox(t.titleNode);
				var height = containerBox.h - titleBox.h - 12;
				if (height < 0) {
					height = 0;
				}
				domStyle.set(t.scrollableDiv,"height",height + "px");
			}
        });
	}
);