define(["dojo/Stateful","dojo/_base/declare","dojo/_base/lang","dojo/date","dojo/date/locale","thornberry/form/TimeSpinner"],function(Stateful,declare,lang,date,locale,TimeSpinner) {
	var Event = declare(Stateful, {
		status: "",
		medId: "",
		
		// Object properties
		med: null,
		qt: null,
		
		// Calculated properties
		medDetailLink: "",
		summary: "",
		statusTag: "",
		
		watcher: null,
		watchHandle: null,
		
		constructor: function(/*Object*/ data){
			var t = this;
			lang.mixin(this, data);
		},
		
		_watcherSetter: function(value) {
			var t = this;
			if (t.watchHandle) {
				t.watchHandle.unwatch();
				//t.watchHandle = null;
			}
			
			t.watcher = value;
			if (value) {
				t.watchHandle = t.watch(function(name,oldValue,newValue) {
					if (t.watcher) {
						t.watcher.notify(name,newValue);
					}
				});
			}
		},
		
		_medIdSetter: function(value) {
			var t = this;
			t.medId = value;
			t.set("med",Event.medStore.get(t.medId));
		},
		
		_medSetter: function(value) {
			var t = this;
			t.med = value;
			if (value) {
				t._changeAttrValue("medId",t.med.id);
				t.set("medDetailLink","<div><a href='#' onclick=\"medNameClickHandler(event,'"+t.med.id+"')\">"+t.getMedNameDisplay()+"</a></div>");
			} else {
				t._changeAttrValue("medId","");
				t.set("medDetailLink","");
			}
			t.updateCalendarSummary();
		},
		
		_medGivenSetter: function(value) {
			var t = this;
			t.medGiven = value;
			if (t.med) {
				t.set("medDetailLink","<div><a href='#' onclick=\"medNameClickHandler(event,'"+t.med.id+"')\">"+t.getMedNameDisplay()+"</a></div>");
			}
		},
		
		_qtIdSetter: function(value) {
			var t = this;
			t.qtId = value;
			t.qt = Event.quantityTimingStore.get(t.qtId);
		},
		
		_statusSetter: function(value) {
			var t = this;
			t.status = value;
			
			var statusTag = "";
			var status =  Event.eventStatusStore.get(t.status);
			if (status) {
				statusTag = "status-"+t.status+"-tag"
				if (t.status === 1 && t.windowStart > new Date()) {
					statusTag = "status-tag-late";
				}
				var titleAttr = "";
				if (t.status === 2) {
					titleAttr = "title='Amount Given: "+t.dose+"'";
				}
				if (t.status === 3) {
					titleAttr = "title='Not Given Reason: "+t.notGivenReason+"'";
				}
				statusTag = "<span class='tag "+statusTag+"' "+titleAttr+">"+status.name+"</span>";
			}
			t.set("statusTag",statusTag);
			t.updateCalendarSummary();
		},
		
		_sDSetter: function(value) {
			var t = this;
			t.sD = value;
			if (value) {
				t._updateTimes("start");
			}
		},
		
		_sTSetter: function(value) {
			var t = this;
			t.sT = value;
			if (value) {
				t._updateTimes("start");
			}
		},
		
		_eDSetter: function(value) {
			var t = this;
			t.eD = value;
			if (value) {
				t._updateTimes("end");
			}
		},
		
		_eTSetter: function(value) {
			var t = this;
			t.eT = value;
			if (value) {
				t._updateTimes("end");
			}
		},
		
		_tTSetter: function(value) {
			var t = this;
			t.tT = value;
			t._updateTimes("time");
		},
		
		getMedNameDisplay: function() {
			var t = this;
			return (t.medGiven ? t.medGiven : t.med.name);
		},
		
		getStartTime: function() {
			var t = this;
			var startTime;
			if (t.sD) {
				var startTime = locale.parse(t.sD+" "+t.sT,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			} else {
				startTime = locale.parse(t.date+" "+t.time,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			}
			return startTime;
		},
		
		getEndTime: function() {
			var t = this;
			var endTime;
			if (t.eD) {
				var endTime = locale.parse(t.eD+" "+t.eT,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			} else {
				endTime = locale.parse(t.date+" "+t.time,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			}
			return endTime;
		},
		
		sDGetDisplay: function() {
			return this.sD ? locale.parse(this.sD,{selector:"date",datePattern:"yyyy-MM-dd"}) : null;
		},
		
		sTGetDisplay: function() {
			return this.sT ? locale.parse(this.sT,{selector:"time",timePattern:"HHmm"}) : null;
		},
		
		eDGetDisplay: function() {
			return this.eD ? locale.parse(this.eD,{selector:"date",datePattern:"yyyy-MM-dd"}) : null;
		},
		
		eTGetDisplay: function() {
			return this.eT ? locale.parse(this.eT,{selector:"time",timePattern:"HHmm"}) : null;
		},
		
		updateCalendarSummary: function() {
			var t = this;
			if (t.med) {
				var futureClass = ""
				if (t.status === 1 && t.windowStart > new Date()) {
					futureClass = " future"
				}
				var titleAttr = "";
				if (t.status === 2) {
					titleAttr = "title='Amount Given: "+t.dose+"'";
				} else if (t.status === 3) {
					titleAttr = "title='Not Given Reason: "+t.notGivenReason+"'";
				}
				if (t.qt === null) {
					t.set("qtId",t.qtId);
				}
				var dose = (t.qt ? t.qt.formatConditionalDosing(true) : t.med.dose);
				if (titleAttr === "") {
					titleAttr = "title='"+t.getMedNameDisplay()+" Dose: "+dose+"'";
				}
				t.set("summary","<div class='event-header event-header-status-"+t.status+futureClass+"' "+titleAttr+">"+t.med.pat.name+"</div><div class='eventBody'><div style='color:rgb(51, 51, 0)'>"+t.med.name+"</div><div style='color:black'>Dose: "+dose+"</div></div>");
			} else {
				t.set("summary","");
			}
		},
		
		_updateTimes: function(changeFrom) {
			// date - yyyy-MM-dd
			// time - HHmm
			// time 2 - HH:mm
			var t = this;
			
			var start = t.parseDateAndTime(t.sD,t.sT);
			var end = t.parseDateAndTime(t.eD,t.eT);
			var activityTime = t.parseMinutes(t.tT);
			
			// enough data to calculate all dates?
			var dateChange = false;
			var startDate = "";
			var startTime = "";
			var endDate = "";
			var endTime = "";
			var activityTimeFormatted = "";
			if ((start && end) || (start && activityTime !== "") || (end && activityTime !== "")) {
				dateChange = true;
				
				if (changeFrom === "start") {
					if (activityTime !== "") {
						end = date.add(start,"minute",activityTime);
					} else {
						activityTime = date.difference(start,end,"minute");
					}
				} else if (changeFrom === "end") {
					if (start) {
						activityTime = date.difference(start,end,"minute");
					} else {
						start = date.add(end,"minute",-activityTime);
					}
				} else if (changeFrom === "time") {
					if (activityTime === "") {
						activityTime = 0;
					}
					if (start) {
						end = date.add(start,"minute",activityTime);
					} else {
						start = date.add(end,"minute",-activityTime);
					}
				}
				
				try {
					startDate = locale.format(start,{datePattern:"yyyy-MM-dd", selector:"date"});
					startTime = locale.format(start,{selector:"time",timePattern:"HHmm"});
					endDate = locale.format(end,{datePattern:"yyyy-MM-dd", selector:"date"});
					endTime = locale.format(end,{selector:"time",timePattern:"HHmm"});
					activityTimeFormatted = t.formatMinutes(activityTime);
				} catch (e) {
					dateChange = false;
				}
			}
			
			if (dateChange) {
				t._changeAttrValue("sD",startDate);
				t._changeAttrValue("sT",startTime);
				t._changeAttrValue("eD",endDate);
				t._changeAttrValue("eT",endTime);
				if (activityTimeFormatted !== t.tT) {
					t._changeAttrValue("tT",activityTimeFormatted);
				}
			}
		},
		
		getOffsetDisplay: function(targetDate,forAdmin) {
			var t = this;
			var offsetDisplay = "";
			if (t.status === 1 || t.status === 4) {
				var settings = Event.settings;
				if (t.scheduledBy !== undefined && t.scheduledBy !== "") {
					// Must be user scheduled event
					offsetDisplay = "-"+settings.OffsetUserScheduledPos+", +"+settings.OffsetUserScheduledNeg;
				} else if (t.qt && t.qt.rPattern) {
					switch(parseInt(t.qt.rPattern)) {
						case 1:
							offsetDisplay = "-"+settings.OffsetBIDPos+", +"+settings.OffsetBIDNeg;
							break;
						case 2:
							offsetDisplay = "-"+settings.OffsetTIDPos+", +"+settings.OffsetTIDNeg;
							break;
						case 3:
							offsetDisplay = "-"+settings.OffsetQIDPos+", +"+settings.OffsetQIDNeg;
							break;
						case 4:
							offsetDisplay = "-"+settings.OffsetXIDPos+", +"+settings.OffsetXIDNeg;
							break;
						case 5:
							offsetDisplay = "-"+settings.OffsetQAMPos+", +"+settings.OffsetQAMNeg;
							break;
						case 7:
							offsetDisplay = "-"+settings.OffsetQHSPos+", +"+settings.OffsetQHSNeg;
							break;
						case 8:
							offsetDisplay = "-"+settings.OffsetQPMPos+", +"+settings.OffsetQPMNeg;
							break;
						default:
							offsetDisplay = "-"+settings.OffsetDefaultPos+", +"+settings.OffsetDefaultNeg;
							break;
					};
				} else {
					offsetDisplay = "-"+settings.OffsetDefaultPos+", +"+settings.OffsetDefaultNeg;
				}
				
				if (offsetDisplay !== "") {
					if (typeof targetDate === "undefined") {
						targetDate = new Date();
					}
					if (forAdmin) {
						if (t.windowStart > targetDate || t.missed <= targetDate) {
							// Missed
							offsetDisplay = "<span class='alert' title='Time falls outside the scheduled time window'>"+offsetDisplay+"</span>";
						} else {
							// Scheduled
							offsetDisplay = "<span class='tag tag-generic' title='Time falls within the scheduled time window'>"+offsetDisplay+"</span>";
						}
					} else {
						if (t.missed <= targetDate) {
							// Missed
							offsetDisplay = "<span class='alert' title='Missed'>"+offsetDisplay+"</span>";
						} else if (t.start <= targetDate) {
							// Overdue
							offsetDisplay = "<span class='tag upcoming-tag' title='Overdue'>"+offsetDisplay+"</span>";
						} else if (t.windowStart <= targetDate) {
							// Upcoming
							offsetDisplay = "<span class='tag upcoming-tag' title='Upcoming'>"+offsetDisplay+"</span>";
						} else {
							// Scheduled
							offsetDisplay = "<span class='tag tag-generic' title='Scheduled'>"+offsetDisplay+"</span>";
						}
					}
				}
			}
			
			return offsetDisplay;
		},
		
		getScheduledTimeDisplay: function(targetDate) {
			return formatFullDate(this.start) + this.getOffsetDisplay(targetDate);
		},
		
		getScheduledTimeDisplayForAdmin: function(targetDate) {
			return formatFullDate(this.start) + this.getOffsetDisplay(targetDate,true);
		},
		
		getAdministrationTimeDisplay: function() {
			var t = this;
			var display = "";
			if (t.status === 2 || t.status === 3) {
				display = formatFullDate(this.end) + this.statusTag;
			}
			return display;
		},
		
		// Find overlapping scheduled event
		getOverlappingSchEvent: function(checkDuration,excludeId) {
			var t = this;
			var overlappingEvent = null;
			var results = Event.eventStore.query({medId:t.medId},{sort:[{attribute:"start",descending:false}]});
			var startTime = t.getStartTime();
			var endTime = t.getEndTime();
			
			var length = results.length;
			for (var i = 0; i < length; i++) {
				var schEvent = results[i];
				if (schEvent.id !== t.id && !schEvent.err && (schEvent.status === 1 || schEvent.status === 4) && (!excludeId || (excludeId !== schEvent.id)))  {
					if (checkDuration) {
						var schEnd = schEvent.missed;
						if (schEvent.end > schEvent.missed) {
							schEnd = schEvent.end;
						}
						
						if ((schEvent.windowStart <= startTime && schEnd > startTime) || (schEvent.windowStart < endTime && schEnd >= endTime)) {
							overlappingEvent = schEvent;
							break;
						}
					} else if (schEvent.windowStart <= startTime && schEvent.missed > startTime) {
						overlappingEvent = schEvent;
						break;
					}
				}
			}
			
			if (overlappingEvent && typeof overlappingEvent.sD === "undefined") {
				overlappingEvent.sD = locale.format(overlappingEvent.displayStart, {datePattern: "yyyy-MM-dd", selector: "date"});
				overlappingEvent.sT = TimeSpinner.prototype.format(overlappingEvent.displayStart);
				overlappingEvent.eD = locale.format(overlappingEvent.displayEnd, {datePattern: "yyyy-MM-dd", selector: "date"});
				overlappingEvent.eT = TimeSpinner.prototype.format(overlappingEvent.displayEnd);
			}
			
			return overlappingEvent;
		},
		
		// Find overlapping scheduled event
		getOverlappingCompletedEvent: function(checkDuration) {
			var t = this;
			var overlappingEvent = null;
			var results = Event.eventStore.query({medId:t.medId},{sort:[{attribute:"upcoming",descending:false}]});
			var startTime = t.getStartTime();
			
			var length = results.length;
			for (var i = 0; i < length; i++) {
				var compEvent = results[i];
				if (compEvent.id !== t.id && !compEvent.err && (compEvent.status === 2 || compEvent.status === 3))  {
					if (compEvent.upcoming <= startTime && compEvent.missed > startTime) {
						overlappingEvent = compEvent;
						break;
					}
				}
			}
			
			return overlappingEvent;
		},
		
		formatMinutes: function(minutes) {
			var output = "";
			if (minutes !== "") {
				var minuteDisplay = (minutes % 60).toString();
				if (minuteDisplay.length === 1) {
					minuteDisplay = "0" + minuteDisplay;
				}
				output = Math.floor(minutes/60).toString() + ":" + minuteDisplay;
			}
			return output;
		},
		
		parseMinutes: function(time) {
			var value = "";
			if (typeof time !== "undefined" && time !== "") {
				var timeArray = time.toString().split(":");
				value = (timeArray.length === 2 ? parseInt(timeArray[0])*60+parseInt(timeArray[1]) : parseInt(timeArray[0]));
			}
			return value;
		},
		
		parseDateAndTime: function(date,time) {
			var val = "";
			if (date && time) {
				val = locale.parse(date+" "+time,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			}
			return val;
		},
		
		getEarlyLateAdminTimes: function() {
			var t = this;
			var times = {
				early: 0,
				late: 0
			};
			var earlyMinutes = 0;
			var lateMinutes = 0;
			
			if (t.status === 1 || t.status === 4) {
				var settings = Event.settings;
				if (t.scheduledBy !== undefined && t.scheduledBy !== "") {
					// Must be user scheduled event
					earlyMinutes = settings.OffsetUserScheduledEarly;
					lateMinutes = settings.OffsetUserScheduledLate;
				} else if (t.qt && t.qt.rPattern) {
					switch(parseInt(t.qt.rPattern)) {
						case 1:
							earlyMinutes = settings.OffsetBIDEarly;
							lateMinutes = settings.OffsetBIDLate;
							break;
						case 2:
							earlyMinutes = settings.OffsetTIDEarly;
							lateMinutes = settings.OffsetTIDLate;
							break;
						case 3:
							earlyMinutes = settings.OffsetQIDEarly;
							lateMinutes = settings.OffsetQIDLate;
							break;
						case 4:
							earlyMinutes = settings.OffsetXIDEarly;
							lateMinutes = settings.OffsetXIDLate;
							break;
						case 5:
							earlyMinutes = settings.OffsetQAMEarly;
							lateMinutes = settings.OffsetQAMLate;
							break;
						case 7:
							earlyMinutes = settings.OffsetQHSEarly;
							lateMinutes = settings.OffsetQHSLate;
							break;
						case 8:
							earlyMinutes = settings.OffsetQPMEarly;
							lateMinutes = settings.OffsetQPMLate;
							break;
						default:
							earlyMinutes = settings.OffsetDefaultEarly;
							lateMinutes = settings.OffsetDefaultLate;
							break;
					};
				} else {
					earlyMinutes = settings.OffsetDefaultEarly;
					lateMinutes = settings.OffsetDefaultLate;
				}

				times.early = date.add(t.start,"minute",-earlyMinutes);
				times.late = date.add(t.start,"minute",lateMinutes);
			}
			
			return times;
		}
	});
	
	Event.quantityTimingStore = {};
	Event.medStore = {};
	Event.eventStatusStore = {};
	Event.settings = {};
	Event.eventStore = {};
	
	Event.getEventForAdmin = function(admin) {
		admin.start = locale.parse(admin.date+" "+admin.time,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
		admin.end = new Date(admin.start);
		
		if (admin.sD && admin.sT && admin.eD && admin.eT) {
			admin.displayStart = locale.parse(admin.sD+" "+admin.sT,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			admin.displayEnd = locale.parse(admin.eD+" "+admin.eT,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			if (admin.displayStart.getTime() === admin.displayEnd.getTime()) {
				admin.displayEnd.setMinutes(admin.displayEnd.getMinutes() + 15);
			}
		} else {
			admin.displayStart = admin.start;
			admin.displayEnd = new Date(admin.end);
			admin.displayEnd.setMinutes(admin.displayEnd.getMinutes() + 15);
		}
		
		if ((admin.status === 2 || admin.status === 3) && admin.upcomingDate && admin.missedDate) {
			admin.upcoming = locale.parse(admin.upcomingDate+" "+admin.upcomingTime,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
			admin.missed = locale.parse(admin.missedDate+" "+admin.missedTime,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
		}
		
		if (admin.status === 1 || admin.status === 4) {
			admin.windowStart = admin.displayStart;
			admin.missed = admin.displayEnd;
		}
		
		return new Event(admin);
	};
	
	Event.getEventForQueue = function(medId,qtId,eventData) {
		var admin = Event.getEmptyAdministration();
		admin.id = medId;
		admin.medId = medId;
		admin.qtId = qtId || "";
		admin.action = "add";
		admin.status = "";
		admin.type = 0; // type: 0 - incomplete, 1 - complete
		
		var event = new Event(admin);
		
		// Check where these are used and replace with event.med.pat references if applicable
		event.patId = event.med.pat.id;
		event.patName = event.med.pat.name; // for sorting
		if (eventData) {
			event.scheduledEvent = eventData;
			//event.selectedScheduledEvent = eventData;
		}
		
		
		// Add getNextOccurrence to med and qt classes
		var next = event.med.getNextOccurrence(qtId);
		var sortTime = Infinity;
		if (next) {
			sortTime = next.start;
		}
		event.sortTime = sortTime; // for sorting
		
		return event;
	};
	
	Event.getEmptyAdministration = function() {
		return {
			id: "",
			qtId: "",
			patId: "",
			medId: "",
			status: "",
			date: locale.format(new Date(), {datePattern: "yyyy-MM-dd", selector: "date"}),
			time: TimeSpinner.prototype.format(new Date()),
			adminBy: userId,
			witnesses: "",
			comment: "",
			dose: "",
			reaction: "",
			notGivenReason: "",
			err: 0,
			errReason: "",
			expDate: null,
			lotNum: "",
			medGiven: "",
			medGivenID: "",
			ivdr: "",
			ivfr: "",
			ivdf: "",
			prnReason: "",
			sD: null,
			sT: "",
			eD: null,
			eT: "",
			tT: "",
			siteGiven: "",
			isIv: 0,
			ivType: "",
			ivSize: "",
			ivVolume: "",
			ivSiteAppearance: "",
			ivSiteAppearanceOther: "",
			admInst: ""
		};
	}
	
	Event.copyForSave = function(event) {
		var ret = {
			id: event.id,
			qtId: event.qtId,
			patId: event.patId,
			medId: event.medId,
			status: event.status,
			date: event.date,
			time: event.time,
			adminBy: event.adminBy,
			witnesses: event.witnesses,
			comment: event.comment,
			dose: event.dose.toString(),
			reaction: event.reaction,
			notGivenReason: event.notGivenReason,
			err: event.err,
			errReason: event.errReason,
			expDate: event.expDate,
			lotNum: event.lotNum,
			medGiven: event.medGiven,
			medGivenID: event.medGivenID,
			ivdr: event.ivdr.toString(),
			ivfr: event.ivfr.toString(),
			ivdf: event.ivdf.toString(),
			prnReason: event.prnReason,
			sD: event.sD,
			sT: event.sT,
			eD: event.eD,
			eT: event.eT,
			tT: event.tT,
			siteGiven: event.siteGiven,
			isIv: event.isIv,
			ivType: event.ivType,
			ivSize: event.ivSize,
			ivVolume: event.ivVolume,
			ivSiteAppearance: event.ivSiteAppearance,
			ivSiteAppearanceOther: event.ivSiteAppearanceOther,
			admInst: event.admInst,
			upcomingDate: event.upcomingDate,
			upcomingTime: event.upcomingTime,
			missedDate: event.missedDate,
			missedTime: event.missedTime,
			schEventId: event.schEventId,
			outsideLimits: !!event.outsideLimits
		}
		if (event.selectedScheduledEvent) {
			ret.scheduledStartDate = locale.format(event.selectedScheduledEvent.start,{datePattern: "yyyy-MM-dd", selector: "date"});
			ret.scheduledStartTime = locale.format(event.selectedScheduledEvent.start,{selector:"time",timePattern:"HHmm"});
		}
		return ret;
	};
	
	return Event;
});