define(["dojo/_base/declare","dojo/_base/lang","dojo/date","dojo/_base/array","thornberry/med/quantityTimingSet","dojo/date/locale"],function(declare,lang,date,arrayUtils,QuantityTimingSet,locale) {
	var Medication = declare(null, {
		medDetailLink: "",
		qtArray: null,
		
		constructor: function(/*Object*/ data){
			var t = this;
			lang.mixin(t, data);
			
			if (Medication.patientStore) {
				t.pat = Medication.patientStore.get(t.patId);
				t.medDetailLink = "<div><a href='#' onclick=\"medNameClickHandler(event,'"+t.id+"')\">"+t.name+"</a></div>";
			}

			if (t.qtArray === null) {
				t.qtArray = [];
			}
			for (var i = 0; i < t.qt.length; i++) {
				var qtItem = t.qt[i];
				qtItem.medId = t.id;
				qtItem.patId = t.patId;
				qtItem.med = t;
				t.qtArray.push(new QuantityTimingSet(qtItem));
			}
		},
		
		getNextOccurrence: function(qtId) {
			var t = this;
			var results = [];
			if (qtId) {
				results = Medication.eventStore.query({qtId:qtId},{sort: [{attribute: "start", ascending: true}]});
			} else {
				results = Medication.eventStore.query({medId:t.id},{sort: [{attribute: "start", ascending: true}]});
			}
			var item = null;
			for (var i = 0; i < results.length; i++) {
				if (results[i].status === 1) {
					item = results[i];
					break;
				}
			}
			return item || "";
		},
		
		getMostRecentMissed: function(ignoreIfGivenSince) {
			// Check for past 24 hours
			var t = this;
			ignoreIfGivenSince = ignoreIfGivenSince || false;
			var overdueItem = "";
			var results = [];
			results = Medication.eventStore.query({medId:t.id},{sort: [{attribute: "end", descending: true}]});
			var start = date.add(new Date(),"day",-1);
			for (var i = 0; i < results.length; i++) {
				var item = results[i];
				if (ignoreIfGivenSince && (item.status !== 1 && item.status !== 4)) {
					break;
				}
				if (date.compare(item.end,start) === 1 && (item.status === 4)) {
					overdueItem = item;
					break;
				}
			}
			return overdueItem;
		},
		
		getMostRecentAdministration: function(qtId,givenOnly) {
			var t = this;
			var results = [];
			if (qtId) {
				results = Medication.eventStore.query({qtId:qtId},{sort: [{attribute: "end", descending: true}]});
			} else {
				results = Medication.eventStore.query({medId:t.id},{sort: [{attribute: "end", descending: true}]});
			}
			var item = null;
			for (var i = 0; i < results.length; i++) {
				if ((!results[i].err) && (results[i].status === 2 || ((!givenOnly && results[i].status === 3) && (results[i].end < new Date())))) {
					item = results[i];
					break;
				}
			}
			return item || "";
		},
		
		getQTIDForAdminNow: function() {
			var t = this;
			var qtId = "";
			var next = t.getNextOccurrence();
			if (next) {
				qtId = next.qtId;
			}
			if (!qtId) {
				for (var i = 0; i < t.qt.length; i++) {
					var qtId = t.qt[i].id;
					var currentDate = new Date();
					var qt = Medication.quantityTimingStore.get(qtId);
					if (qt) {
						if (qt.isActiveOnDate(currentDate) && (qt.administeredBy === "" || qt.administeredBy === "Agency" )) {
							qtId = qt.id;
							break;
						}
					}
				}
			}
			return qtId;
		},
		
		getPRNTag: function(qtId) {
			var t = this;
			var display = "";
			if (t.isPRN(qtId)) {
				display = (t.standing ? "<span class='tag prn-tag' title='Standing Medication'>PRN (S)</span>" : "<span class='tag prn-tag' title='PRN Medication'>PRN</span>");
			}
			return display;
		},
		
		getInfoIcon: function() {
			var t = this;
			var display = "";
			if (t.fdbId !== "") {
				display = '<div id="infoIcon" title="Info" class="icon info-icon" style="margin: 4.5px;" onclick="actionShowMedInfo('+t.fdbId+')"></div>';
			}
			return display;
		},
		
		isPRN: function(qtId) {
			var t = this;
			var qt = Medication.quantityTimingStore.get(qtId);
			var prn = false;
			if (qt && qt.priority === 3) {
				prn = true;
			} else if (!qt && t.prn) {
				prn = true;
			}
			return prn;
		},

		isPaused: function() {
			var t = this;
			var paused = false;
			if (t.pauseScheduleDate !== "") {
				var pauseTime = locale.parse(t.pauseScheduleDate+" "+t.pauseScheduleTime,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
				var resumeTime = null;
				if (t.resumeScheduleDate !== "") {
					resumeTime = locale.parse(t.resumeScheduleDate+" "+t.resumeScheduleTime,{selector:"date",datePattern:"yyyy-MM-dd HHmm"});
				}
				paused = ((new Date()).getTime() >= pauseTime.getTime() && (resumeTime === null || resumeTime.getTime() > (new Date()).getTime()));
			}
			return paused;
		},
		
		getNextAdminTimeDisplay: function(qtId) {
			var t = this;
			var display = "";
			var nextOccurrence = t.getNextOccurrence(qtId);
			if (nextOccurrence) {
				display = nextOccurrence.getScheduledTimeDisplay();
			}
			return display;
		},
		
		getMostMostRecentAdminTimeDisplay: function(qtId) {
			var t = this;
			var display = "";
			var lastAdministration = t.getMostRecentAdministration(qtId);
			if (lastAdministration) {
				display = lastAdministration.getAdministrationTimeDisplay();
			}
			return display;
		},

		getNDocNumber: function() {
			var t = this;
			var pieces = t.id.toString().split("||");
			return pieces[0];
		}
	});
	
	Medication.eventStore = {};
	Medication.quantityTimingStore = {};
	Medication.patientStore = null;
	
	return Medication;
});