// thornberry/med/medicationView.js
define([
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./templates/medicationDetail.html",
	"dojo/date/locale",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/dom-geometry",
	"dojo/on",
	"dojo/_base/lang",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/IndirectSelect",
	"gridx/modules/select/Row",
	"gridx/modules/RowHeader",
	"dojo/store/Memory",
	"thornberry/med/actions"
], function(declare,_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin,template,locale,domStyle,domClass,domGeom,on,lang,Grid,Cache,IndirectSelect,SelectRow,RowHeader,Memory,medActions){
        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
			templateString: template,
			baseClass: "medicationDetail",
			
			eventCache: {},
			quantityTimingStore: {},
			medication: {},
			nextOccurrenceQTId: "",
			medLockData: "",
			getMedLockData: function() {},
			medStore: {},
			actionShowMessage: function() {},
			navigateBack: function() {},
			
			open: false,
			
			postMixInProperties: function() {
				var t = this;
				t.inherited(arguments);
			},
			
			buildRendering: function() {
				var t = this;
				t.inherited(arguments);

				t.buildGrids();
			},

			buildGrids: function() {
				var t = this;
				var store = new Memory({
					data: []
				});
				var layout = [
					{ id : "begin", field : "id", name : "Begin", formatter : function(itemData) {return t._formatTime(itemData.startD,itemData.startT)} },
					{ id : "end", field : "id", name : "End", formatter : function(itemData) {return t._formatTime(itemData.endD,itemData.endT) } },
					{ id : "conditionalDosing", field : "conditionalDosing", name : "Dose" },
					{ id : "repeat", field : "id", name : "Schedule", formatter : function(itemData) {return itemData.formattedRepeat } },
					{ id : "totalOccurrences", field : "tOccur", name : "Total Occurrences", width : "150px" },
					{ id : "administeredBy", field : "administeredBy", name : "To Be. Adm. By", width : "115px" },
					{ id : "priority", field : "priorityTag", name : "Priority", width : "80px" },
					{ id : "nextOccurrence", field : "id", name : "Next Occurrence", formatter : function(itemData) {return itemData.med.getNextAdminTimeDisplay(itemData.id);} }
				]
				t.grid = new Grid({
					cacheClass: Cache,
					store: store,
					query: {dummy:0},
					structure: layout,
					autoHeight: true,
					selectRowTriggerOnCell: true,
					style: "width:100%;height:100%",
					modules: []
				});
				
				t.qtGridNode.appendChild(t.grid.domNode);
				t.grid.startup();
				
				var adminLayout = [
					{ id : "date", field : "date", name : "Administration Time", formatter : function(itemData) {return formatFullDate(itemData.start)} },
					{ id : "status", field : "statusTag", name : "Status" },
					{ id : "administeredBy", field : "adminBy", name : "Administered By" },
					{ id : "medGiven", field : "medGiven", name : "Medication Given" },
					{ id : "doseGiven", field : "dose", name : "Amount Given", formatter: function(itemData) {return (itemData.dose !== "" ? itemData.dose : (itemData.notGivenReason !== "" ? "Not Given: "+itemData.notGivenReason : ""))} }
				]
				t.adminGrid = new Grid({
					cacheClass: Cache,
					store: store,
					structure: adminLayout,
					autoHeight: true,
					selectRowTriggerOnCell: true,
					style: "width:100%;height:100%",
					modules: []
				});
				
				t.adminGridNode.appendChild(t.adminGrid.domNode);
				t.adminGrid.startup();
			},
			
			refreshDisplay: function() {
				var t = this;
				//t.set("medication",t.medication);
				if (t.open && t.medication && t.medication.id) {
					t.set("medication",t.medStore.get(t.medication.id));
					/*domStyle.set(t.alertRowNode,"display","none");
					var alertText = t.getAlert();
					if (alertText !== "") {
						t.alertNode.innerHTML = alertText;
						domStyle.set(t.alertRowNode,"display","");
					}
					var nextAdministrationTime = t.medication.getNextAdminTimeDisplay();
					t.nextOccurrenceNode.innerHTML = (nextAdministrationTime !== "" ? "Next Administration: "+nextAdministrationTime : "");
					t.adminInstructionsNode.innerHTML = t.medication.admInst;
					t.updatePreviousAdminGrid();*/
				}
			},
			
			postCreate: function() {
				var t = this;
				t.inherited(arguments)
				
				var seconds = (new Date()).getSeconds();
				setTimeout(function() {
					var func = lang.hitch(t,"refreshDisplay");
					func();
					setInterval(func,60000);
				},(61-seconds)*1000);
				
				setInterval(function() {
					if (t.medication && t.medication.id) {
						t.set("medLockData",t.getMedLockData(t.medication.id));
					}
				},10000);
			},
			
			_setMedicationAttr: function(med) {
				var t = this;
				t.medication = med;
				t._set("medication",med);
				
				if (med) {
					t.nextOccurrenceQTId = "";
					
					if (med.getPRNTag) {
						t.medNode.innerHTML = med.name + med.getPRNTag() + med.getInfoIcon();
					}
					
					domStyle.set(t.alertRowNode,"display","none");
					var alertText = t.getAlert();
					if (alertText !== "") {
						t.alertNode.innerHTML = alertText;
						domStyle.set(t.alertRowNode,"display","");
					}
					
					domStyle.set(t.medDoseContainer,"display",(med.dose !== "" ? "" : "none"));

					t.routeNode.innerHTML = med.route;
					t.doseNode.innerHTML = med.dose;
					t.ordPhyNode.innerHTML = med.ordPhy;
					t.clsNode.innerHTML = med.cls;
					t.pharmNode.innerHTML = med.pharm;
					t.startDateTimeNode.innerHTML = (med.sD ? locale.format(locale.parse(med.sD, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.sT : "");
					t.resumeDateTimeNode.innerHTML = (med.rD ? locale.format(locale.parse(med.rD, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.rT : "");
					t.holdDateTimeNode.innerHTML = (med.hD ? locale.format(locale.parse(med.hD, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.hT : "");
					t.discontinueDateTimeNode.innerHTML = (med.dD ? locale.format(locale.parse(med.dD, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.dT : "");
					t.adminInstructionsNode.innerHTML = paragraphFormatter(med.admInst);

					t.pauseScheduleDateTimeNode.innerHTML = (med.pauseScheduleDate ? locale.format(locale.parse(med.pauseScheduleDate, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.pauseScheduleTime : "");
					t.resumeScheduleDateTimeNode.innerHTML = (med.resumeScheduleDate ? locale.format(locale.parse(med.resumeScheduleDate, {datePattern: "yyyy-MM-dd", selector: "date"}), {datePattern: "MM/dd/yyyy", selector: "date"})+" "+med.resumeScheduleTime : "");
					
					t.strengthNode.innerHTML = med.strength;
					domStyle.set(t.divStrength,"display",(med.strength === "" ? "none" : ""));
					//t.volumeNode.innerHTML = med.volume;
					//domStyle.set(t.divVolume,"display",(med.volume === "" ? "none" : ""));
					t.medFormNode.innerHTML = med.form;
					domStyle.set(t.divMedForm,"display",(med.form === "" ? "none" : ""));

					var qtStore = new Memory({
						data: t.quantityTimingStore.query({medId:med.id})
					});
					t.grid.setStore(qtStore);
					
					t.updatePreviousAdminGrid();
					if (t.medication.id) {
						var nextAdministrationTime = t.medication.getNextAdminTimeDisplay();
						var pauseIconHTML = "";
						if (t.medication.isPaused()) {
							pauseIconHTML = '<div title="Scheduling Paused" class="pause-icon" style="margin: 4.5px;"></div>';
						}
						t.nextOccurrenceNode.innerHTML = (nextAdministrationTime !== "" ? "Next Administration: "+pauseIconHTML+nextAdministrationTime : pauseIconHTML);
						t.set("medLockData",t.getMedLockData(t.medication.id));
					}
				} else {
					t.reset();
					t.actionShowMessage("This medication has been deleted.");
					t.navigateBack();
				}
			},
			
			reset: function() {
				var t = this;
				t.nextOccurrenceQTId = "";
				t.medNode.innerHTML = "";
				domStyle.set(t.alertRowNode,"display","none");
				t.routeNode.innerHTML = "";
				t.doseNode.innerHTML = "";
				t.ordPhyNode.innerHTML = "";
				t.clsNode.innerHTML = "";
				t.pharmNode.innerHTML = "";
				t.startDateTimeNode.innerHTML = "";
				t.resumeDateTimeNode.innerHTML = "";
				t.holdDateTimeNode.innerHTML = "";
				t.discontinueDateTimeNode.innerHTML = "";
				t.adminInstructionsNode.innerHTML = "";
				t.grid.setStore(new Memory());
				t.adminGrid.setStore(new Memory());
				t.nextOccurrenceNode.innerHTML = "";
			},
			
			_setMedLockDataAttr: function(value) {
				var t = this;
				var oldValue = t.medLockData;
				t._set("medLockData",value);
				
				if (oldValue !== value) {
					var alertText = t.getAlert();
					if (alertText !== "") {
						t.alertNode.innerHTML = alertText;
						domStyle.set(t.alertRowNode,"display","");
					} else {
						domStyle.set(t.alertRowNode,"display","none");
					}
				}
			},
			
			getAlert: function() {
				var t = this;
				var display = "";
				var inProgress = t.eventCache.query({medId:t.medication.id,status:5});
				if (inProgress.length > 0) {
					display = "Administration is "+inProgress[0].statusTag;
				} else if (t.medication.getMostRecentMissed)  {
					var missedEvent = t.medication.getMostRecentMissed();
					if (missedEvent) {
						display =  "Missed administration at "+formatFullDate(missedEvent.start)+"<span class='alert' title='Missed'>Missed</span>";
					}
				}
				if (t.medLockData !== "") {
					if (display !== "") {
						display += " | ";
					}
					var pieces = t.medLockData.toString().split("~");
					var intent = "administered";
					if (pieces.length === 2 && pieces[1] === "schedule") {
						intent = "rescheduled";
					}
					display += "Medication currently being " + intent + " by " + pieces[0];
				}
				return display;
			},
			
			updatePreviousAdminGrid: function() {
				var t = this;
				var med = t.medication;
				// Get top 5 previous administrations
				var results = t.eventCache.query({medId:med.id},{sort: [{attribute: "start", descending: true}]});
				var adminStore = new Memory();
				var count = 0;
				for (var i = 0; i < results.length && count < 5; i++) {
					if (!results[i].err && (results[i].status === 2 || results[i].status === 3 || results[i].status === 5)) {
						adminStore.put(results[i]);
						count++;
					}
				}
				t.adminGrid.setStore(adminStore);
			},
			
			getActions: function(functions,viewedFromQueue,parentView) {
				var t = this;
				var actions = [];
				if (!viewedFromQueue && t.quantityTimingStore.query({medId:t.medication.id,active:true}).length > 0 && parentView !== "scheduleView") {
					actions.push({actionId:"adminNow",type:"button",label:"Admin Now",onClick:function() {functions.actionInitiateAdminNow()}});
					// Add modify med actions here too as applicable
				}
				
				if (!viewedFromQueue) {
					if (!t.medication.error) {
						actions.push({actionId:"schedule",type:"button",label:"Schedule",onClick:function() {functions.actionInitiateSchedule("schedule")}});
					}
					if (t.medication.getNextOccurrence()) {
						actions.push({actionId:"rescheduleNext",type:"button",label:"Reschedule Next",onClick:function() {functions.actionInitiateSchedule("rescheduleNext")}});
					}
					if (medActions.pauseScheduleIsValid(t.medication) === "") {
						actions.push({actionId:"pauseSchedule",type:"button",label:"Pause MAR Schedule",onClick:function() {medActions.pauseScheduleAction([t.medication])}});
					}
					if (medActions.resumeScheduleIsValid(t.medication) === "") {
						actions.push({actionId:"resumeSchedule",type:"button",label:"Resume MAR Schedule",onClick:function() {medActions.resumeScheduleAction([t.medication])}});
					}
				}

				//domStyle.set(t.infoIcon,"display",(t.medication.fdbId !== "" ? "" : "none"));
				
				actions.push({actionId:"history",type:"button",label:"Med History",onClick:function() {functions.actionShowHistory("med")}});
				// Show Event Detail Report action too
				return actions;
			},
			
			_formatTime: function(date,time) {
				var displayValue = "";
				if (date && date !== "") {
					var date = new Date(date);
					date.setMinutes(date.getMinutes() + date.getTimezoneOffset());
					displayValue += locale.format(date,{selector:"date",datePattern:"MM/dd/yyyy"});
					if (time !== "") {
						displayValue += " "+time;
					}
				}
				return displayValue;
			},
			
			scrollToTop: function() {
				this.scrollableDiv.scrollTop = 0;
			},
			
			resize: function() {
				var t = this;
				t.inherited(arguments);
				t.grid.resize();
				t.adminGrid.resize();
				
				var containerBox = domGeom.getContentBox(t.containerNode);
				var titleBox = domGeom.getContentBox(t.titleNode);
				var height = containerBox.h - titleBox.h - 16;
				if (height < 0) {
					height = 0;
				}
				domStyle.set(t.scrollableDiv,"height",height + "px");
			}
        });
	}
);