define([
	"thornberry/patientActivity/Activity",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/on",
	"dojo/date",
	"dojo/date/locale",
	"dojo/_base/array",
	"dojo/aspect",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/store/Memory",
	"dojo/store/Observable",
	"dojo/query",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/form/RadioButton",
	"dijit/form/Select",
	"dijit/form/TextBox",
	"dijit/form/Button",
	"dijit/form/FilteringSelect",
	"dijit/form/Textarea",
	"dijit/form/CheckBox",
	"dijit/form/DateTextBox",
	"dijit/popup",
	"thornberry/serverRequest",
	"dojo/topic",
	"dijit/TooltipDialog",
	"dijit/form/DropDownButton",
	"dojo/NodeList-dom",
	"dojo/NodeList-traverse"
],function(Activity, declare, lang, on, date, locale, arrayUtils, aspect, domConstruct, domStyle, domClass, Memory, 
		   Observable, query, _WidgetBase, _TemplatedMixin, _WidgetsInTemplate, RadioButton, Select, TextBox, 
		   Button, FilteringSelect, Textarea, CheckBox, DateTextBox, popup, serverRequest, topic) {
	var Meeting = declare(Activity, {
		
		typeId: 3,
		
		
		// Data model from database
		status: "",
		locationId: "", // Patient Activity Location table
		attendees: null, // array of objects
		otherAttendee1: "",
		otherAttendee1Attended: "",
		otherAttendee2: "",
		otherAttendee2Attended: "",
		dataRetrievalRange: "",
		dataRetrievalStart: "",
		dataRetrievalEnd: "",
		
		inCareMeeting: "",
		careMeetingTypeId: "",
		
		paycodeId: "",
		costCenterId: "",
		
		// Object references
		patientCareMeeting: null,
		
		// Display values
		locationDisplay: "",
		attendeeDisplay: "",
		attendeeDisplayTruncated: "",
		patientName: "",
		patientAccountNum: "",
		statusDisplay: "",
		statusSort: "",
		fromDisplay: "",
		toDisplay: "",
		
		// State
		nextAttendeeId: 1,
		
		constructor: function(data){
			var t = this;
			
			t.attendees = [];
			t.attendeeStore = new Memory();
		},
		
		postscript: function(kwargs) {
			var t = this;
			this.inherited(arguments);
			
			t.nextAttendeeId = t.attendees.length + 1;
			t.updateAttendeeDisplay();
			
			if (t.inCareMeeting) {
				t.checkReady();
			}
		},
		
		buildCustomRendering: function(editor) {
			Meeting.buildCustomRendering(editor);
		},
		
		getSaveData: function() {
			var t = this;
			var saveData = t.inherited(arguments);
			saveData.locationId = t.locationId;
			saveData.paycodeId = t.paycodeId;
			saveData.costCenterId = t.costCenterId;
			
			saveData.otherAttendee1 = t.otherAttendee1;
			saveData.otherAttendee1Attended = t.otherAttendee1Attended;
			saveData.otherAttendee2 = t.otherAttendee2;
			saveData.otherAttendee2Attended = t.otherAttendee2Attended;
			saveData.attendees = t.getAttendees();
			
			saveData.inCareMeeting = t.inCareMeeting;
			saveData.status = t.status;
			saveData.meetingData = t.meetingData;
			saveData.dataRetrievalStart = t.dataRetrievalStart;
			saveData.dataRetrievalEnd = t.dataRetrievalEnd;
			saveData.dataRetrievalRange = t.dataRetrievalRange;
			
			if (t.inCareMeeting && t.totalTime === "") {
				// default total time in case the user cleared it when not required (Pending/Ready status)
				t.set("totalTime",10);
				saveData.totalTime = t.totalTime;
			}
			
			saveData.deleteFlag = t.deleteFlag;
			
			return saveData;
		},
		
		isAttendee: function(userId) {
			var t = this;
			return arrayUtils.some(t.attendees,function(attendee) {
				return attendee.userId == userId;
			});
		},
		
		_reasonIdSetter: function(value) {
			var t = this;
			t.inherited(arguments);
			
			var reason = reasonStore.get(value);
			if (reason && t.inCareMeeting) {
				if (t.totalTime === "") {
					if (reason.minutes !== "") {
						t.set("totalTime",reason.minutes);
					} else {
						t.set("totalTime",10);
					}
				}
			}
		},
		
		_locationIdSetter: function(value) {
			var t = this;
			t.locationId = value;
			
			var loc = locationStore.get(value);
			t.locationDisplay = (loc ? escapeText(loc.name) : "");
		},
		
		_otherAttendee1Setter: function(value) {
			var t = this;
			t.otherAttendee1 = value;
			
			t.updateAttendeeDisplay();
		},
		
		_otherAttendee2Setter: function(value) {
			var t = this;
			t.otherAttendee2 = value;
			
			t.updateAttendeeDisplay();
		},
		
		_statusSetter: function(value) {
			var t = this;
			t.status = value;
			t.statusDisplay = "<span class='meeting-tag meeting-tag-"+value.toLowerCase()+"'>"+value+"</span>";
			switch(value) {
				case "Pending":
					t.statusSort = 1;
					break;
				case "Ready":
					t.statusSort = 2;
					break;
				case "Started":
					t.statusSort = 3;
					break;
				case "Completed":
					t.statusSort = 4;
					break;
				default: 
					t.statusSort = 5;
			}
		},
		
		_careMeetingTypeIdSetter: function(value) {
			var t = this;
			t.careMeetingTypeId = value;
			
			if ((typeof meetingTypeStore !== "undefined") && t.inCareMeeting && !t._inLoad) {
				var careMeetingType = meetingTypeStore.get(value);
				if (careMeetingType) {
					var reasonIdArray = careMeetingType.reasonIdArray;
					var clearCurrentReason = true;
					if (arrayUtils.some(reasonIdArray,function(reasonId) {
						return reasonId == t.reasonId;
					})) {
						var clearCurrentReason = false;
					}
					
					if (clearCurrentReason) {
						if (reasonIdArray.length === 1) {
							t.set("reasonId",reasonIdArray[0]);
						} else {
							t.set("reasonId","");
							
						}
						t.checkReady();
					}
				}
			}
		},

		checkReady: function() {
			var t = this;
			if (t.inCareMeeting && (t.status === "" || t.status === "Pending" || t.status === "Ready")) {
				var ready = true;
				if (t.reasonId == "") {
					ready = false;
				}
				if (requireSummary == 1 && t.summary === "") {
					ready = false;
				}
				// Skip payroll fields for now
				t.set("status",(ready ? "Ready" : "Pending"));
			}
		},
		
		getAttendeeId: function() {
			return this.nextAttendeeId++;
		},
		
		getAttendees: function() {
			var t = this;
			var attendeeArray = [];
			arrayUtils.forEach(t.attendees,function(attendee) {
				if (attendee.discId !== "") {
					attendeeArray.push(attendee);
				}
			});
			return attendeeArray;
		},
		
		updateAttendeeDisplay: function() {
			var t = this;
			var display = "";
			var displayTruncated = "";
			var count = 0;
			arrayUtils.forEach(t.attendees,function(attendee) {
				var discipline = disciplineStore.get(attendee.discId);
				var user = userStore.get(attendee.userId);
				if (discipline) {
					display += (display === "" ? "" : ", ") + escapeText(discipline.name) + " - " + (user ? escapeText(user.name) : "");
					if (count < Meeting.MAX_ATTENDEES_DISPLAY) {
						displayTruncated += (displayTruncated === "" ? "" : "<br />") + "<span class='bold'>" + escapeText(discipline.name) + "</span> - " + (user ? escapeText(user.name) : "");
					} else if (count === Meeting.MAX_ATTENDEES_DISPLAY) {
						displayTruncated += "<br />...";
					}
					count++;
				}
			});
			
			if (t.otherAttendee1 !== "") {
				display += (display === "" ? "" : ", ") + escapeText(t.otherAttendee1);
				if (count < Meeting.MAX_ATTENDEES_DISPLAY) {
					displayTruncated += (displayTruncated === "" ? "" : "<br />") + escapeText(t.otherAttendee1);
				} else if (count === Meeting.MAX_ATTENDEES_DISPLAY) {
					displayTruncated += "<br />...";
				}
				count++;
			}
			if (t.otherAttendee2 !== "") {
				display += (display === "" ? "" : ", ") + escapeText(t.otherAttendee2);
				if (count < Meeting.MAX_ATTENDEES_DISPLAY) {
					displayTruncated += (displayTruncated === "" ? "" : "<br />") + escapeText(t.otherAttendee2);
				} else if (count === Meeting.MAX_ATTENDEES_DISPLAY) {
					displayTruncated += "<br />...";
				}
				count++;
			}
			
			t.attendeeDisplay = display;
			t.attendeeDisplayTruncated = displayTruncated;
		},

		getCalendars: function() {
			var t = this;
			var calendarArray = t.inherited(arguments);
			var attendees = t.getAttendees();
			var userIdIndex = {};
			arrayUtils.forEach(attendees,function(attendee) {
				if (typeof userIdIndex[attendee.userId] === "undefined") {
					calendarArray.push(attendee.userId+"||User");
					userIdIndex[attendee.userId] = true;
				}
			});
			return calendarArray;
		}
	});
	
	Meeting.buildCustomRendering = function(editor) {
		editor._decoratePatientData = function() {
			query("section>h3:first-child",editor.patientDataNode).forEach(function(node) {
				//if (query("h3~*",node.parentNode).length > 0) {
					var expandNode = domConstruct.create("a",{innerHTML:"-"},node,"before");
					domStyle.set(expandNode,{display:"inline-block",width:"15px"});
					domStyle.set(node,"display","inline-block");
					var toggle = function() {
						var display = "";
						if (this.innerHTML === "+") {
							this.innerHTML = "-";
						} else {
							display = "none";
							this.innerHTML = "+";
						}
						query("h3~*",node.parentNode).forEach(function(node) {
							domStyle.set(node,"display",display);
						});
					};
					editor.own(on(expandNode,"click",toggle));
					editor.own(on(node,"click",lang.hitch(expandNode, toggle)));
					lang.hitch(expandNode,toggle)();
				//}
			});
			
			var classVal = "";
			var nodeCount = 0;
			query(".meetingDataHtml>",editor.patientDataNode).forEach(function(node) {
				if (node.tagName === "H2") {
					nodeCount++;
				}
				if (node.tagName === "section") {
					domClass.add(node,"section"+nodeCount);
				}
			});
			
			nodeCount = 0;
			query("h2",editor.patientDataNode).forEach(function(node) {
				domConstruct.create("br",{},node,"before");
				var expandNode = domConstruct.create("a",{innerHTML:"-"},node,"before");
				domStyle.set(expandNode,{display:"inline-block",width:"15px"});
				domStyle.set(node,"display","inline-block");
				
				var nodeCounter = nodeCount + 1;
				nodeCount++;
				
				var toggle = function() {
					var display = "";
					if (this.innerHTML === "+") {
						this.innerHTML = "-";
					} else {
						display = "none";
						this.innerHTML = "+";
					}
					query(".section"+nodeCounter,editor.patientDataNode).forEach(function(node) {
						domStyle.set(node,"display",display);
					});
				};
				editor.own(on(expandNode,"click",toggle));
				editor.own(on(node,"click",lang.hitch(expandNode, toggle)));
			});
		};
		
		var disabled = (editor.actionType === "View" || editor.actionType === "Prepare" ? true : false);
		var activity = editor.activityNew;
		activity._inLoad = true;
		
		aspect.after(editor,"okToSave",function(ok) {
			if (ok) {
				if (arrayUtils.every(activity.attendees,function(attendee) {return attendee.userId === "";}) && activity.otherAttendee1 === "" && activity.otherAttendee2 === "") {
					actionShowMessage("Please enter at least one attendee");
					ok = false;
				}
			}
			return ok;
		});
		
		if (editor.actionType === "Add") {
			var count = 0;
			arrayUtils.forEach(referralArray,function(referral) {
				if (referral.userIds.length > 0) {
					for (var i = 0; i < referral.userIds.length && count < Meeting.MAX_ATTENDEES; i++) {
						var userId = referral.userIds[i];
						activity.attendees.push({
							id: activity.getAttendeeId(),
							discId: referral.discId,
							userId: userId
						});
						count++;
					}
				} else if (count < Meeting.MAX_ATTENDEES) {
					activity.attendees.push({
						id: activity.getAttendeeId(),
						discId: referral.discId
					});
					count++;
				}
			});
			activity.updateAttendeeDisplay();
		}
		
		editor.attendeeStore = new Observable(new Memory({
			data: activity.attendees
		}));
		
		var Custom = declare([_WidgetBase,_TemplatedMixin,_WidgetsInTemplate], {
			templateString:
			'<div>'
			+ (payrollOn ? 
				'<div class="content-flex-container">\
					<div class="flex-field-container">\
						<div class="field-name field-name-medium">\
							<label>Paycode</label>\
						</div>\
						<div class="field-value field-value-medium-large">\
							<input data-dojo-attach-point="paycodeNode">\
						</div>\
					</div>\
					<div class="flex-field-container">\
						<div class="field-name field-name-medium">\
							<label>Cost Center</label>\
						</div>\
						<div class="field-value field-value-medium-large">\
							<input data-dojo-attach-point="costCenterNode">\
						</div>\
					</div>\
				</div>'
			: '') +
				'<div class="content-area-title">\
					<div class="content-divider"></div>\
				</div>\
				<div class="content-area-title">\
					<a style="display:inline-block;width:15px" data-dojo-attach-point="patientDataExpandNode">+</a>\
					<div class="content-area-title-text" style="display:inline-block">Patient Data</div>\
				</div>\
				<div style="display:none" data-dojo-attach-point="patientDataDisplayNode">\
					<div class="content-flex-container" data-dojo-attach-point="retrieveDataSectionNode">\
						<div class="flex-field-container">\
							<div class="field-value field-value-medium-large">\
								<input data-dojo-attach-point="retrieveOptionsNode">\
							</div>\
						</div>\
						<div class="flex-field-container">\
							<div class="field-value field-value-medium-large">\
								<input data-dojo-attach-point="retrieveDataNode">\
							</div>\
						</div>\
					</div>\
					<div class="content-flex-container" data-dojo-attach-point="retrieveStartEndNode" style="display:none">\
						<div class="flex-field-container">\
							<div class="field-name field-name-smaller">\
								<label>Start</label>\
							</div>\
							<div class="field-value field-value-small">\
								<input data-dojo-attach-point="retrieveStartNode">\
							</div>\
						</div>\
						<div class="flex-field-container">\
							<div class="field-name field-name-smaller">\
								<label>End</label>\
							</div>\
							<div class="field-value field-value-small">\
								<input data-dojo-attach-point="retrieveEndNode">\
							</div>\
						</div>\
					</div>\
					<div class="content-flex-container" data-dojo-attach-point="patientDataNode">\
					</div>\
				</div>\
				<div class="content-area-title">\
					<div class="content-divider"></div>\
				</div>'
			+ (activity.inCareMeeting ? 
				'<div class="content-flex-container" data-dojo-attach-point="lastSummaryDisplayNode">\
					<div class="flex-field-container">\
						<div class="field-name field-name-large">\
							<label>Last Meeting Note</label>\
						</div>\
						<div class="field-value field-value-grand">\
							<input data-dojo-attach-point="lastSummaryNode">\
						</div>\
					</div>\
				</div>'
			: '') + 
				'<div class="content-flex-container" data-dojo-attach-point="locationDisplayNode">\
					<div class="flex-field-container">\
						<div class="field-name field-name-medium">\
							<label>Location</label>\
						</div>\
						<div class="field-value field-value-medium-large">\
							<input data-dojo-attach-point="locationNode">\
						</div>\
					</div>\
				</div>\
				<div class="content-flex-container" data-dojo-attach-point="addAttendeeDiv">\
					<div class="flex-field-container">\
						<div class="field-value field-value-medium-large">\
							<input data-dojo-attach-point="addAttendeeNode">\
						</div>\
					</div>\
				</div>\
				<div class="content-flex-container">\
					<div class="flex-field-container">\
						<div class="field-name field-name-large">\
							<label>Other Attendee</label>\
						</div>\
						<div class="field-value field-value-large">\
							<input data-dojo-attach-point="otherAttendee1Node">\
						</div>\
					</div>\
					<div class="flex-field-container">\
						<div class="field-name field-name-medium">\
							<label>In Attendance</label>\
						</div>\
						<div class="field-value field-value-smallest">\
							<input data-dojo-attach-point="otherAttending1Node">\
						</div>\
					</div>\
				</div>\
				<div class="content-flex-container">\
					<div class="flex-field-container">\
						<div class="field-name field-name-large">\
							<label>Other Attendee</label>\
						</div>\
						<div class="field-value field-value-large">\
							<input data-dojo-attach-point="otherAttendee2Node">\
						</div>\
					</div>\
					<div class="flex-field-container">\
						<div class="field-name field-name-medium">\
							<label>In Attendance</label>\
						</div>\
						<div class="field-value field-value-smallest">\
							<input data-dojo-attach-point="otherAttending2Node">\
						</div>\
					</div>\
				</div>'
				+ (activity.inCareMeeting ? "" : 
					'<div class="content-area-title">\
						<div class="content-divider"></div>\
					</div>') +
			'</div>'
		});
		var custom = new Custom({attachScope: editor});
		domConstruct.place(custom.domNode,editor.customNode);
		custom.startup();
		editor.custom = custom;
		
		editor.require(editor.startTimeField,false);
		if (!activity.inCareMeeting || activity.status === "Started" || activity.status === "Completed") {
			editor.require(editor.startTimeField);
			editor.require(editor.endTimeField);
			editor.require(editor.totalTimeField);
			if (activity.inCareMeeting) {
				editor.require(editor.summaryField);
			}
		}
		
		editor.locationIdField = new Select({
			disabled: disabled,
			style: "width:150px"
		},editor.locationNode);
		editor.locationIdField.startup();
		editor.require(editor.locationIdField);
		editor.own(on(editor.locationIdField,"change",function(value){editor._modifyValue("locationId");}));
		editor.populateSelect(editor.locationIdField,locationStore,activity.locationId,true);
		
		arrayUtils.forEach(activity.attendees,function(attendeeData) {
			var attendee = Meeting.addAttendee(editor,disabled,attendeeData);
			domConstruct.place(attendee.domNode,editor.addAttendeeDiv,"before");
		});
		
		editor.addAttendeeButton = new Button({
			label: "Add Attendee",
		},editor.addAttendeeNode);
		editor.addAttendeeButton.startup();
		editor.own(on(editor.addAttendeeButton,"click",function() {
			var attendee = Meeting.addAttendee(editor,disabled);
			domConstruct.place(attendee.domNode,editor.addAttendeeDiv,"before");
		}));
		if (disabled || editor.attendeeStore.data.length >= Meeting.MAX_ATTENDEES) {
			domStyle.set(editor.addAttendeeButton.domNode,"display","none");
		}
		
		editor.attendeeStore.query({}).observe(function() {
			editor.activityNew.updateAttendeeDisplay();
			if (editor.attendeeStore.data.length < Meeting.MAX_ATTENDEES) {
				domStyle.set(editor.addAttendeeButton.domNode,"display","");
			} else {
				domStyle.set(editor.addAttendeeButton.domNode,"display","none");
			}
			editor.defer("checkConflicts");
		});
		
		editor.otherAttendee1Field = new TextBox({
			disabled: disabled,
			maxLength: 30,
			value: activity.otherAttendee1
		},editor.otherAttendee1Node);
		editor.otherAttendee1Field.startup();
		editor.own(on(editor.otherAttendee1Field,"change",function(value) {
			if (value !== "") {
				editor.otherAttendee1AttendedField.set("disabled",false);
			} else {
				editor.otherAttendee1AttendedField.set("disabled",true);
				editor.otherAttendee1AttendedField.set("checked",false);
			}
			editor._modifyValue("otherAttendee1");
		}));
		
		editor.otherAttendee1AttendedField = new CheckBox({
			disabled: disabled || activity.otherAttendee1 === "",
			checked: activity.otherAttendee1Attended
		},editor.otherAttending1Node);
		editor.otherAttendee1AttendedField.startup();
		editor.own(on(editor.otherAttendee1AttendedField,"change",function(){
			activity.set("otherAttendee1Attended",editor.otherAttendee1AttendedField.get("checked"));
			editor._postModify();
		}));
		
		editor.otherAttendee2Field = new TextBox({
			disabled: disabled,
			maxLength: 30,
			value: activity.otherAttendee2
		},editor.otherAttendee2Node);
		editor.otherAttendee2Field.startup();
		editor.own(on(editor.otherAttendee2Field,"change",function(value) {
			if (value !== "") {
				editor.otherAttendee2AttendedField.set("disabled",false);
			} else {
				editor.otherAttendee2AttendedField.set("disabled",true);
				editor.otherAttendee2AttendedField.set("checked",false);
			}
			editor._modifyValue("otherAttendee2");
		}));
		
		editor.otherAttendee2AttendedField = new CheckBox({
			disabled: disabled || activity.otherAttendee2 === "",
			checked: activity.otherAttendee2Attended
		},editor.otherAttending2Node);
		editor.otherAttendee2AttendedField.startup();
		editor.own(on(editor.otherAttendee2AttendedField,"change",function(){
			activity.set("otherAttendee2Attended",editor.otherAttendee2AttendedField.get("checked"));
			editor._postModify();
		}));
		
		if (activity.inCareMeeting) {
			domStyle.set(editor.locationDisplayNode,"display","none");
			domConstruct.place(editor.summaryDisplayNode,editor.locationDisplayNode,"before");
			
			editor.okToSave = function() {
				var t = this;
				var ok = false;
				
				if (!t.formNode.validate()) {
					actionShowMessage("Please correct any invalid fields.");
				} else if (query(".pinkBackground",t.containerNode).length > 0) {
					actionShowMessage("Please fill in all required fields.");
				} else if ((t.actionType === "Edit" || t.actionType === "Prepare") && !t.modified && t.status !== "Pending") {
					actionShowMessage("You have not made any changes.");
				} else if (arrayUtils.every(activity.attendees,function(attendee) {return attendee.userId === "";}) && activity.otherAttendee1 === "" && activity.otherAttendee2 === "") {
					actionShowMessage("Please enter at least one attendee");
				} else {
					ok = true;
				}
				return ok;
			};
			
			if (activity.lastMeetingSummary !== "" && PAGE === "PatientCareMeetings") {
				editor.lastSummaryField = new Textarea({
					maxLength: 10000,
					disabled: true,
					value: activity.lastMeetingSummary
				},editor.lastSummaryNode);
				editor.lastSummaryField.startup();
			} else {
				domStyle.set(editor.lastSummaryDisplayNode,"display","none");
			}
			
			if (PAGE === "PatientActivity") {
				var node = domConstruct.create("div",{innerHTML: "Patient Care Meeting"},editor.titleNode,"after");
				domStyle.set(node,{float:"right"});
				domClass.add(node,"content-area-title-text");
			}
			
			// Filter reasons by activity type and care meeting type
			var careMeetingTypeReasonIds = null;
			if ((typeof meetingTypeStore !== "undefined")) {
				careMeetingType = meetingTypeStore.get(activity.careMeetingTypeId);
				if (careMeetingType) {
					careMeetingTypeReasonIds = careMeetingType.reasonIdArray;
				}
			}
			var restrictReasonArray = [];
			arrayUtils.forEach(reasonStore.data,function(reason) {
				var include = true;
				if (arrayUtils.indexOf(reason.activityTypeList.toString().split(","),activity.typeId) === -1) {
					include = false;
				}
				if (careMeetingTypeReasonIds && arrayUtils.indexOf(careMeetingTypeReasonIds,reason.id) === -1) {
					include = false;
				}
				if (include) {
					restrictReasonArray.push(reason.id);
				}
			});
			editor.populateSelect(editor.reasonIdField,reasonStore,activity.reasonId,(editor.actionType !== "Add"),restrictReasonArray);
		}
			
		editor.own(on(editor.patientDataExpandNode,"click",function() {
			if (editor.patientDataExpandNode.innerHTML === "+") {
				editor.patientDataExpandNode.innerHTML = "-";
				domStyle.set(editor.patientDataDisplayNode,"display","");
			} else {
				editor.patientDataExpandNode.innerHTML = "+";
				domStyle.set(editor.patientDataDisplayNode,"display","none");
			}
		}));
		
		if (activity.meetingData) {
			setTimeout(function() {
				editor.patientDataNode.innerHTML = activity.meetingData;
				editor._decoratePatientData();
			},0);
		} else {
			editor.patientDataNode.innerHTML = "Patient data has not been retrieved.";
			if (editor.actionType !== "Edit" && editor.actionType !== "Add") {
				domStyle.set(editor.retrieveDataSectionNode,"display","none");
			}
		}
		
		var disabledCareMeeting = (editor.actionType !== "Edit" && editor.actionType !== "Add") || (!!activity.inCareMeeting && activity.status === "Completed");
			
		editor.retrieveOptionsSelect = new Select({
			options: [
				{label:"All since last meeting",value:1},
				{label:"From cert period start",value:2},
				{label:"User specified date range",value:3}
			],
			value: activity.dataRetrievalRange,
			disabled: disabledCareMeeting
		},editor.retrieveOptionsNode);
		editor.own(on(editor.retrieveOptionsSelect,"change",function(value) {
			domStyle.set(editor.retrieveStartEndNode,"display",(value === 3 ? "" : "none"));
			editor.require(editor.retrieveStartField,(value === 3 ? true : false));
			editor.require(editor.retrieveEndField,(value === 3 ? true : false));
			domStyle.set(editor.retrieveDataButton.domNode,"display","");
			
			editor.retrieveStartField.set("value",date.add(new Date(),"day",-14));
			editor.retrieveEndField.set("value",new Date());
			
			editor.updateFieldPinking();
		}));
		editor.retrieveOptionsSelect.startup();
		domStyle.set(editor.retrieveStartEndNode,"display",(activity.dataRetrievalRange == 3 ? "" : "none"));
		
		editor.retrieveStartField = new DateTextBox({
			style: "width:110px",
			disabled: disabledCareMeeting,
			value: (activity.dataRetrievalStart ? locale.parse(activity.dataRetrievalStart,{selector:"date",datePattern:"yyyy-MM-dd"}) : null),
			constraints: {min:date.add(new Date(),"day",-3700),max:new Date()}
		},editor.retrieveStartNode);
		editor.retrieveStartField.startup();
		editor.own(on(editor.retrieveStartField,"change",function(value) {
			domStyle.set(editor.retrieveDataButton.domNode,"display","");
			editor.updateFieldPinking();
			if (editor.retrieveStartField.isValid()) {
				editor.retrieveEndField.constraints.min = value;
			}
		}));
		editor.own(on(editor.retrieveStartField,"blur",function(){checkForT(editor.retrieveStartField.id);}));
		editor.require(editor.retrieveStartField,(activity.dataRetrievalRange == 3 ? true : false));
		
		editor.retrieveEndField = new DateTextBox({
			style: "width:110px",
			disabled: disabledCareMeeting,
			value: (activity.dataRetrievalEnd ? locale.parse(activity.dataRetrievalEnd,{selector:"date",datePattern:"yyyy-MM-dd"}) : null),
			constraints: {max:new Date()}
		},editor.retrieveEndNode);
		editor.retrieveEndField.startup();
		editor.own(on(editor.retrieveEndField,"change",function(value) {
			domStyle.set(editor.retrieveDataButton.domNode,"display","");
			editor.updateFieldPinking();
		}));
		editor.own(on(editor.retrieveEndField,"blur",function(){checkForT(editor.retrieveEndField.id);}));
		editor.require(editor.retrieveEndField,(activity.dataRetrievalRange == 3 ? true : false));
			
		if ((editor.actionType === "Edit" || editor.actionType === "Add") && (!activity.inCareMeeting || activity.status !== "Completed")) {	
			editor.retrieveDataButton = new Button({
				label: (activity.meetingData ? "Update Data" : "Retrieve Data"),
				style: "display:"+(activity.meetingData ? "none" : ""),
				onClick: function() {
					var dataRetrievalRange = editor.retrieveOptionsSelect.get("value");
					var dataRetrievalStart = editor.retrieveStartField.get("value");
					var dataRetrievalEnd = editor.retrieveEndField.get("value");
					
					if (dataRetrievalRange == 3 && (!dataRetrievalStart || !dataRetrievalEnd || !editor.retrieveStartField.isValid() || !editor.retrieveEndField.isValid())) {
						actionShowMessage("Please enter valid start and end dates");
					} else {
						dataRetrievalStart = (dataRetrievalStart ? locale.format(dataRetrievalStart,{selector:"date",datePattern:"yyyy-MM-dd"}) : "");
						dataRetrievalEnd = (dataRetrievalEnd ? locale.format(dataRetrievalEnd,{selector:"date",datePattern:"yyyy-MM-dd"}) : "");
						
						editor.patientDataNode.innerHTML = "Loading...";
						
						topic.publish("preventSave",true);
						serverRequest("Patient.Activity.Utility","RetrievePatientData",{
							ndocNum: activity.ndocNum,
							dataRetrievalRange: dataRetrievalRange,
							activityDate: activity.dateString,
							dataRetrievalStart: dataRetrievalStart,
							dataRetrievalEnd: dataRetrievalEnd
						}).then(function(response) {
							topic.publish("preventSave",false);
							editor.patientDataNode.innerHTML = response.dataString;
							activity.set("meetingData",response.dataString);
							editor._postModify();
							editor._decoratePatientData();
							
							activity.set("dataRetrievalRange",dataRetrievalRange);
							activity.set("dataRetrievalStart",dataRetrievalStart);
							activity.set("dataRetrievalEnd",dataRetrievalEnd);
							editor._postModify();
							
							domStyle.set(editor.retrieveDataButton.domNode,"display","none");
						},function(error) {
							actionShowMessage(error);
							topic.publish("preventSave",false);
						});
					}
				}
			},editor.retrieveDataNode);
		} else {
			domStyle.set(editor.retrieveDataNode,"display","none");
		}
		
		editor.titleNode.innerHTML = editor.actionType + " " + activityTypeStore.get(activity.typeId).name + " " + activity.statusDisplay;
		
		editor.own(aspect.after(activity,"_statusSetter",function() {
			editor.titleNode.innerHTML = editor.actionType + " " + activityTypeStore.get(activity.typeId).name + " " + activity.statusDisplay;
		}));
		
		if (payrollOn) {
			editor.paycodeIdField = new Select({
				disabled: disabled,
				style: "width:150px"
			},editor.paycodeNode);
			editor.paycodeIdField.startup();
			editor.require(editor.paycodeIdField);
			editor.own(on(editor.paycodeIdField,"change",function(){editor._modifyValue("paycodeId");}));
			
			var updatePaycodeField = function() {
				var paycodeIds = [];
				var reason = reasonStore.get(activity.get("reasonId"));
				if (reason && reason.paycodeIds !== "") {
					paycodeIds = reason.paycodeIds.toString().split(",");
					if (paycodeIds.length > 1) {
						paycodeIds.push(999);
					}
				}
				editor.populateSelect(editor.paycodeIdField,paycodeStore,activity.paycodeId,(editor._inLoad && editor.actionType !== "Add"),paycodeIds);
				editor.paycodeIdField.set("disabled",(disabled || editor.paycodeIdField.options.length === 1 ? true : false));
			};
			editor.own(aspect.after(editor.reasonIdField,"onChange",function() {
				activity.set("paycodeId","");
				updatePaycodeField();
				editor.updateFieldPinking();
			}));
			updatePaycodeField();
			
			editor.costCenterIdField = new Select({
				disabled: disabled,
				style: "width:150px"
			},editor.costCenterNode);
			editor.costCenterIdField.startup();
			editor.populateSelect(editor.costCenterIdField,costCenterStore,activity.costCenterId,(editor._inLoad && editor.actionType !== "Add"),costCenterIdArray);
			editor.require(editor.costCenterIdField);
			editor.own(on(editor.costCenterIdField,"change",function(){editor._modifyValue("costCenterId");}));
			if (editor.costCenterIdField.options.length === 1) {
				editor.costCenterIdField.set("disabled",true);
			}
		}

		editor.defer(function() {
			activity._inLoad = false;
		});
	};
	
	Meeting.Attendee = declare([_WidgetBase,_TemplatedMixin,_WidgetsInTemplate], {
		templateString:
		'<div>\
			<div class="content-flex-container">\
				<a data-dojo-attach-point="expandNode" style="display:inline-block;width:15px">+</a>\
				<div class="flex-field-container">\
					<div class="field-name field-name-medium">\
						<label>Discipline</label>\
					</div>\
					<div class="field-value field-value-medium">\
						<input data-dojo-attach-point="disciplineNode">\
					</div>\
				</div>\
				<div class="flex-field-container">\
					<div class="field-name field-name-medium">\
						<label>Attendee</label>\
					</div>\
					<div class="field-value field-value-large">\
						<input data-dojo-attach-point="attendeeNode">\
					</div>\
				</div>\
				<div class="flex-field-container">\
					<div class="field-name field-name-medium">\
						<label>In Attendance</label>\
					</div>\
					<div class="field-value field-value-smallest">\
						<input data-dojo-attach-point="attendingNode">\
					</div>\
				</div>\
				<a data-dojo-attach-point="removeNode">Remove</a>\
			</div>\
			<div class="content-flex-container" data-dojo-attach-point="prepNotesDiv" style="padding-left:123px;display:none">\
				<div class="flex-field-container">\
					<div class="field-name field-name-medium">\
						<label>Prep Notes</label>\
					</div>\
					<br />\
					<div class="field-value field-value-grand">\
						<input data-dojo-attach-point="prepNotesNode">\
					</div>\
				</div>\
			</div>\
		</div>',
		
		attendee: null,
		noEdits: false,
		editor: null,
		
		buildRendering: function() {
			var t = this;
			t.inherited(arguments);
			
			t.discIdField = new FilteringSelect({
				disabled: t.noEdits,
				store: disciplineStore,
				value: t.attendee.discId,
				style: "width: 125px",
				required: false
			},t.disciplineNode);
			t.discIdField.startup();
			t.own(on(t.discIdField,"change",function(value) {
				var disabled = (value ? false : true);
				t.userIdField.set("disabled",disabled);
				
				if (value) {
					var store = new Memory({
						data: userStore.query({discId:value,active:1}),
					});
					t.userIdField.set("store",store);
					t.userIdField.set("value",(store.data.length === 1 ? store.data[0].id : ""));
				} else {
					t.userIdField.set("value","");
				}
				t._modifyValue("discId");
				t.editor.updateFieldPinking();
				t.editor.activityNew.updateAttendeeDisplay();
				t.editor.defer("checkConflicts");
			}));
			
			var store = new Memory();
			if (t.attendee.discId) {
				store.setData(userStore.query({discId:t.attendee.discId,active:1}));
			}
			if (!store.get(t.attendee.userId)) {
				var user = userStore.get(t.attendee.userId);
				if (user) {
					store.put(user);
				}
			}
		
			t.userIdField = new FilteringSelect({
				store: store,
				value: t.attendee.userId,
				style: "width: 200px",
				required: false,
				disabled: (t.attendee.discId && !t.noEdits ? false : true)
			},t.attendeeNode);
			t.userIdField.startup();
			t.own(on(t.userIdField,"change",function(value) {
				if (value && arrayUtils.some(t.editor.activityNew.attendees,function(attendee) {
					return attendee.userId == value;
				})) {
					actionShowMessage("Warning: duplicate user selected");
				}
				
				var disabled = (value ? false : true);
				t.attendedField.set("disabled",disabled);
				t.prepNotesField.set("disabled",disabled);
				t.attendedField.set("checked",false);
				t.prepNotesField.set("value","");
				
				if (value !== t.attendee.userId && value === userId) {
					t.expandPrepNotes();
				}
				
				t._modifyValue("userId");
				t.editor.activityNew.updateAttendeeDisplay();
				t.editor.updateFieldPinking();
				t.editor.defer("checkConflicts");
			}));
			
			t.attendedField = new CheckBox({
				checked: (t.attendee.attended == 1),
				disabled: (t.attendee.userId && !t.noEdits ? false : true)
			},t.attendingNode);
			t.attendedField.startup();
			t.own(on(t.attendedField,"change",function() {
				t.attendee.attended = t.attendedField.get("checked");
				t.editor.set("modified",true);
			}));
			
			t.prepNotesField = new Textarea({
				maxLength: 1500,
				value: t.attendee.prepNotes,
				disabled: (t.attendee.userId && !t.noEdits ? false : true)
			},t.prepNotesNode);
			t.prepNotesField.startup();
			t.own(on(t.prepNotesField,"change",function() {t._modifyValue("prepNotes");}));
			
			if (t.noEdits) {
				domStyle.set(t.removeNode,"display","none");
			}
		},
		
		postCreate: function() {
			var t = this;
			t.inherited(arguments);
			
			t.own(on(t.removeNode,"click",function() {
				t.editor.attendeeStore.remove(t.attendee.id);
				
				// Set all later attendees as changed so they properly get shifted up and saved
				t.editor.attendeeStore.query(
					function(object){ return object.id > t.attendee.id; }
				).forEach(
					function(attendee){
						attendee.changed = true;
					}
				);
				
				t.editor.set("modified",true);
				t.destroyRecursive();
			}));
			
			t.own(on(t.expandNode,"click",lang.hitch(t,"expandPrepNotes")));
			if (userId === t.attendee.userId) {
				t.expandPrepNotes();
				if (t.editor.actionType === "Prepare") {
					t.prepNotesField.set("disabled",false);
				}
			}
		},
		
		expandPrepNotes: function() {
			var t = this;
			if (t.expandNode.innerHTML === "+") {
				t.expandNode.innerHTML = "-";
				domStyle.set(t.prepNotesDiv,"display","");
				t.prepNotesField.resize();
			} else {
				t.expandNode.innerHTML = "+";
				domStyle.set(t.prepNotesDiv,"display","none");
			}
		},
		
		_modifyValue: function(fieldName) {
			var t = this;
			var widget = t[fieldName+"Field"];
			if ((typeof widget.isValid === "undefined") || widget.isValid()) {
				t.attendee[fieldName] = widget.get("value");
			}
			if (fieldName === "prepNotes") {
				t.attendee.changed = true;
			}
			t.editor.set("modified",true);
		},
	});
	
	Meeting.MAX_ATTENDEES = 20;
	Meeting.MAX_ATTENDEES_DISPLAY = 4;
	
	Meeting.addAttendee = function(editor,disabled,attendee) {
		if (typeof attendee === "undefined") {
			attendee = {
				id: editor.activityNew.getAttendeeId(),
				discId: "",
				userId: "",
				attended: "",
				prepNotes: "",
				changed: false
			};
			editor.attendeeStore.put(attendee);
		}
		
		var attendee = new Meeting.Attendee({
			editor: editor,
			attendee: attendee,
			noEdits: disabled,
			attendeeStore: editor.attendeeStore
		});
		attendee.startup();
		return attendee;
	};
	
	return Meeting;
});