define([
    "dojo/_base/declare",
    "thornberry/report/formatter",
    "thornberry/layout/View",
    "thornberry/report/GridView",
    "dijit/registry",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/dom",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/store/Memory",
    "dojo/on",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "gridx/modules/Bar",
    "gridx/modules/ColumnResizer",
    "gridx/modules/VirtualVScroller",
    "dojo/aspect",
    "dijit/form/Button",
    "thornberry/serverRequest",
    "dojo/query",
    "dojo/NodeList-dom"
],function(declare,formatter,View,GridView,registry,arrayUtils,lang,dom,domConstruct,domStyle,domClass,Memory,on,Grid,Cache,SingleSort,Bar,ColumnResizer,VirtualVScroller,aspect,Button,serverRequest,query,NodeList) {
    
    // Define global function used by the detail formatter
    showReportDetails = function(row,column) {};

    var CrossTabGridView = declare([View], {
        dataStore: null,

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.dataStore = new Memory({
                idProperty:"_row"
            });
        },

        buildContent: function() {
            var t = this;
            t.inherited(arguments);

            t.defer(function() {
                t.showList.push(t.controller.summarizationIconNode);
                t.showList.push(t.controller.printIconNode);
                if (t.controller.report.filterColumns !== "") {
                    t.showList.push(t.controller.filterIconNode);
                }
            });

            t.refreshButton = new Button({
                label: "Refresh",
                onClick: function() {
                    t.updateGridData(true);
                }
            }).placeAt(t.controller.actionContainer);
            t.controller.registerToggledNode(t.refreshButton.domNode);
            t.controller.updateDisableList.push(t.refreshButton);

            var layout = [
                { id : "_row", field : "_row", name : "" }
            ];
			
			var modelExtensions = [];
			var modules = [
				ColumnResizer,
                VirtualVScroller	
			];
			if (t.controller.report.interactiveCrossTabSort) {
				modules.push(SingleSort);
				modelExtensions.push("gridx/core/model/extensions/Sort");
			}

            t.grid = new Grid({
                cacheClass: Cache,
                store: t.dataStore,
                structure: layout,
                modules: modules,
                modelExtensions: modelExtensions
            });

            t.grid.connect(t.grid.body, "onAfterRow", function(row) {
                var item = row.grid.model.byId(row.id).item;
                if (item._row === "[ALL]") {
                    domClass.add(row.node(),"bold");
                }
            });
            
            t.domNode.appendChild(t.grid.domNode);
            t.grid.startup();

            t.detailGridView = new GridView({
                controller: t.controller,
                title: t.controller.report.displayName,

                buildContent: function() {
                    // Summary:
                    //      This method is responsible for setting up the content pane's content 
                    var t = this;
                    t.inherited("buildContent",arguments);
        
                    t.backButton = new Button({
                        label: "Back",
                        onClick: function() {
                            t.controller.popView();
                        }
                    }).placeAt(t.controller.actionContainer);
                    t.controller.registerToggledNode(t.backButton.domNode);
                },

                updateGridData: function(force) {
                    // Summary:
                    //      This method should be called whenever the user makes changes that affect what data displays in the grid. It gathers
                    //      all the context information and passes it to the report class's implementation of Report.Definition.Abstract:GenerateData()
                    //      and then passes the results to the GridView
                    var t = this;
                    //if (t.controller.modified.options) {
                        t.setColumns(t.controller.columnPane.get("value"));
                    //}

                    if (t.controller.modified.any) {
                        t.resetPagination();
                    }
    
                    var context = t.controller.getContext("Interactive",t.controller.report);
                    context.pageStart = t.get("dataPageStart");
                    context.pageSize = t.get("dataPageSize");
                    context.options.rowValue = t.rowValue;
                    context.options.columnValue = t.columnValue;
                    context.sort = t.getSort();
    
                    domStyle.set("loadingOverlay","display","block");
                    
                    serverRequest(t.controller.report.className,"GenerateData",context,t.controller.report.async).then(function(results) {
                        t.setResults(results);
                        domStyle.set("loadingOverlay","display","none");
                        t.controller.clearModified();
                    },function(error) {
                        domStyle.set("loadingOverlay","display","none");
                        t.showMessage(error);
                    });
                },

                refresh: function() {
                    var t = this;
                    domStyle.set(t.backButton.domNode,"display","");
                }
            });

            t.defer(function() {
                t.detailGridView.showList = new NodeList();
                t.detailGridView.showList.push(t.controller.optionIconNode);
                t.detailGridView.showList.push(t.controller.printIconNode);

                t.showList.push(t.controller.presetControlContainer);
            });
            t.controller.addView(t.detailGridView);
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            t.own(on(t,"show",function() {
                if (!t.controller.closing) {
                    t.updateGridData();
                }
            }));

            t.own(aspect.after(t.controller,"setModified",function() {
                if (t.controller.viewStack.selectedChildWidget.id === t.id) {
                    t.updateGridData();
                }
            }));

            showReportDetails = function(row,column) {
                var title = (row !== "[ALL]" ? row : "")+(row !== "[ALL]" && column !== "[ALL]" ? " and " : "")+(column !== "[ALL]" ? column : "");
                t.detailGridView.title = "Details "+(row !== "[ALL]" || column !== "[ALL]" ? "for " : "")+title;

                t.detailGridView.rowValue = row;
                t.detailGridView.columnValue = column;
                t.detailGridView.resetPagination();
                t.controller.pushView(t.detailGridView);
            };
        },

        updateGridData: function(force) {
            // Summary:
            //      This method should be called whenever the user makes changes that affect what data displays in the grid. It gathers
            //      all the context information and passes it to the report class's implementation of Report.Definition.Abstract:GenerateData()
            //      and then passes the results to the GridView
            var t = this;
            if (t.controller.modified.any || force) {
                var context = t.controller.getContext("Interactive",t.controller.report);

                domStyle.set("loadingOverlay","display","block");
                
                serverRequest(t.controller.report.className,"GenerateData",context,t.controller.report.async).then(function(results) {
                    t.setResults(results);
                    domStyle.set("loadingOverlay","display","none");
                    t.controller.clearModified();
                },function(error) {
                    domStyle.set("loadingOverlay","display","none");
                    t.showMessage(error);
                });
            }
        },

        setResults: function(results) {
            var t = this;
            t.inherited(arguments);

            var columns = [
                { id : "_row", field : "_row", name : "" }
            ];

            var columnDefinition = t.controller.report.getColumn("id");
			var aggregate = t.controller.summarizationEditor.aggregatesField.get("value").toString();
			if (!aggregate.match(/PERCENT/g)) {
				var aggregateColumn = aggregate.split("||")[1];
				if (aggregateColumn) {
					columnDefinition = t.controller.report.getColumn(aggregateColumn);
				}
			}
           
            columnDefinition = lang.clone(columnDefinition);
            columnDefinition.originalFormatter = columnDefinition.formatter;
            columnDefinition.formatter = "crossTabCell";

            var count = 0;
            arrayUtils.forEach(results.columns,function(columnName) {
                count++;
                var column = { id : count, field : columnName, name : "<div style=''>"+escapeText(columnName)+"</div>" };
                column.formatter = formatter.getForColumn(column,columnDefinition);
                columns.push(column);
            });

            t.grid.setColumns(columns);

            t.dataStore.setData(results.data);
            try {
                t.grid.sort.clear();
            } catch (e) {}
            t.grid.vScroller.scrollToRow(0,1);
            t.grid.model.clearCache();
            t.grid.body.refresh();
        },

        refresh: function() {
            var t = this;
            domStyle.set(t.refreshButton.domNode,"display","");
        }
	});
	
	return CrossTabGridView;
});