define([
    "dojo/_base/declare",
    "thornberry/report/Controller",
    "thornberry/report/SummarizableGridView",
    "thornberry/object/ObjectEditor",
    "thornberry/object/template",
    "thornberry/form/DragNDropSelect",
    "dijit/layout/StackContainer",
    "dijit/layout/BorderContainer",
    "dijit/layout/ContentPane",
    "dijit/registry",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/store/Memory",
    "dojo/on",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "gridx/modules/Bar",
    "gridx/modules/ColumnResizer",
    "dojo/aspect",
    "dojox/form/CheckedMultiSelect",
    "dojo/Stateful"
],function(declare,Controller,SummarizableGridView,ObjectEditor,template,DragNDropSelect,StackContainer,BorderContainer,ContentPane,registry,arrayUtils,lang,domConstruct,domStyle,Memory,on,Grid,Cache,SingleSort,Bar,ColumnResizer,aspect,CheckedMultiSelect,Stateful) {
	var SummarizableReportController = declare([Controller], {
        summarizeByModified: false,

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.summaryStore = new Memory({
                idProperty:"Name"
            });

            t.modified.summarization = true;
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t._createSummaryView();
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            if (t.report.hasInteractivePresentation()) {
                t.own(aspect.after(t,"_setResultsAttr",function(results) {
                    t.summarizationEditor.set("modified",false);
                },true));
            }
        },

        updateFromPreset: function(preset) {
            var t = this;
            t.inherited(arguments);

            if (preset !== null) {
                t.summarizationEditor.summarizeByField.set("value",preset.summarizeBy);
                t.summarizationEditor.aggregatesField.set("value",preset.aggregations);
            }
            t.defer(function() {
                t.summarizationEditor.set("modified",false);
            });
        },

        createGrid: function() {
            var t = this;
            t.set("gridView",new SummarizableGridView({
                controller: t,
                title: t.report.displayName
            }));
        },

        _createSummaryView: function() {
            var t = this;

            t.summarizationEditor = new ObjectEditor({
                templateString: template.autoGenerateSummarizationTemplate(t.report),
                report: t.report,
                controller: t,
                object: new Stateful({}),

                buildRendering: function() {
                    var t = this;
                    t.inherited("buildRendering",arguments);

                    if (t.report) {
                        t.summarizeByField = new CheckedMultiSelect({
                            multiple: false
                        },t.summarizeByNode);
                        arrayUtils.forEach(t.report.summarizableColumns.toString().split(","),function(column) {
                            var column = t.report.getColumn(column);
                            if (column) {
                                var option = domConstruct.create("option",{label:escapeText(column.displayName),value:column.name});
                                t.summarizeByField.addOption(option);
                            }
                        });
            
                        var option = domConstruct.create("option",{label:escapeText("None"),value:"None"});
                        t.summarizeByField.addOption(option);
            
                        t.summarizeByField.name = "summarizeBy";
                        t.own(on(t.summarizeByField,"change",function(value) {t._modifyValue(this.name);}));

                        var aggregateStore = new Memory({
                            data: [
                                {id:"_COUNT",field:"_COUNT",name:"Count",column:"",aggregateFunction:"COUNT"},
                                {id:"_PERCENTCOUNT",field:"_PERCENTCOUNT",name:"% Count",column:"",aggregateFunction:"PERCENT COUNT"}
                            ]
                        });
                        arrayUtils.forEach(t.report.columnDefinitions,function(definition) {
                            if ((definition.isDisplayed || definition.isFilterable) && definition.aggregateFunctions !== "") {
                                arrayUtils.forEach(definition.aggregateFunctions.toString().split(","),function(aggFunc) {
                                    aggregateStore.put({
                                        id: aggFunc+"||"+definition.name,
                                        field: aggFunc+definition.name,
                                        column: definition.name,
                                        aggregateFunction: aggFunc,
                                        name: t.controller.getAggregateName(aggFunc)+" "+definition.displayName
                                    });
                                });
                            }
                        });

                        t.aggregatesField = new DragNDropSelect({},t.aggregatesNode);
                        t.aggregatesField.name = "aggregates";
                        t.own(on(t.aggregatesField,"change",function(value) {t._modifyValue(this.name);}));
                        t.aggregatesField.startup();

                        t.aggregatesField.set("store",aggregateStore);
                    }
                },

                postCreate: function() {
                    var t = this;
                    t.inherited("postCreate",arguments);

                    t.registerChangeLogic("summarizeBy",function() {
                        t.require("aggregates",(t.summarizeByField.get("value") !== "None"));
                        t.disable("aggregates",(t.summarizeByField.get("value") === "None"));
                    });
                },

                _aggregatesOnValidate: function() {
                    var t = this;
                    var valid = true;
                    if (t.aggregatesField.get("value").length > 10) {
                        valid = false;
                        t.controller.showMessage("Please select no more than 10 aggregates.");
                    }
                    return valid;
                }
            });

            t.summarizationContainer.appendChild(t.summarizationEditor.domNode);
            t.summarizationEditor._postModify();

            t.summarizationView.isModified = function() {
                return t.summarizationEditor.get("modified");
            };
        },

        getAggregateName: function(aggregateFunction) {
            var name = "";
            switch(aggregateFunction) {
                case "AVG":
                    name = "Mean";
                    break;
                case "MIN":
                    name = "Min.";
                    break;
                case "MAX":
                    name = "Max.";
                    break;
                case "COUNT DISTINCT":
                    name = "Distinct";
                    break;
                case "SUM":
                    name = "Sum";
                    break;
                case "STDDEV_POP":
                    name = "Pop. STDEV";
                    break;
                case "Q1":
                    name = "Lower (Q1)";
                    break;
                case "Q2":
                    name = "Median (Q2)";
                    break;
                case "Q3":
                    name = "Upper (Q3)";
                    break;
                case "MODE":
                    name = "Mode";
                    break;
                case "COUNT":
                    name = "Count";
                    break;
                case "PERCENT COUNT":
                    name = "% Count";
                    break;
                case "PERCENT SUM":
                    name = "% Sum";
                    break;
                case "PERCENT TOTAL COUNT":
                    name = "% of Total Count";
                    break;
                case "SUMROUND":
                    name = "Sum";
                    break;
                default:
                    name = aggregateFunction;
            }
            return name;
        },

        getContext: function(presentationType,report) {
            var t = this;
            var context = t.inherited(arguments);

            // Only re-gather summary data if the filters change
            var summarizeBy = t.summarizationEditor.summarizeByField.get("value");
            if ((summarizeBy !== "None") && (t.modified.filters || t.modified.search || t.modified.summarization || presentationType === "PDF")) {
                context.summarizeBy = summarizeBy;
                context.aggregations = arrayUtils.map(t.summarizationEditor.aggregatesField.get("value"),function(id) {
                    return t.summarizationEditor.aggregatesField.store.get(id);
                });
            }
            return context;
        },

        onApplySummarization: function() {
            var t = this;

            if (t.summarizationEditor.okToSave()) {
                t.set("search","");
                if (t.summarizationEditor.get("modified")) {
                    var currentPreset = t.presetManager.currentPreset;
                    currentPreset.summarizeBy = t.summarizationEditor.summarizeByField.get("value");
                    currentPreset.aggregations = t.summarizationEditor.aggregatesField.get("value");
                    t.setCurrentPresetModified();

                    t.setModified("summarization");
                    t.summarizationEditor.set("modified",false);
                }

                if (t.getView() === t.summarizationView.id) {
                    t.popView();
                }
            }
        },

        onCancelSummarization: function() {
            var t = this;
            t.onCancel(t.summarizationView);
            t.summarizationEditor.set("modified",false);
            t.popView();
        }
	});
	
	return SummarizableReportController;
});