define([
    "dojo/_base/declare",
    "thornberry/report/GridView",
    "thornberry/report/formatter",
    "dijit/layout/StackContainer",
    "dijit/layout/BorderContainer",
    "dijit/layout/ContentPane",
    "dijit/registry",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/dom",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/store/Memory",
    "dojo/on",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "gridx/modules/Bar",
    "gridx/modules/ColumnResizer",
    "dojo/aspect",
    "dojox/form/CheckedMultiSelect"
],function(declare,GridView,formatter,StackContainer,BorderContainer,ContentPane,registry,arrayUtils,lang,dom,domConstruct,domStyle,domClass,Memory,on,Grid,Cache,SingleSort,Bar,ColumnResizer,aspect,CheckedMultiSelect) {
	var SummarizableGridView = declare([GridView], {
        summaryStore: null,

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.summaryStore = new Memory({
                idProperty:"Name"
            });
        },

        buildContent: function() {
            var t = this;
            t.inherited(arguments);

            t.defer(function() {
                t.showList.push(t.controller.summarizationIconNode);
            });

            t.borderContainer = new BorderContainer({
                splitter: true,
                gutters: false
            });

            t.gridPane = new ContentPane({
                region: "center",
                style: "padding: 0px"
            });

			if (t.controller.report.summaryPosition === "bottom") {
				t.summaryGridPane = new ContentPane({
					region: "bottom",
					style: "height:200px;display:none;padding: 0px",
					splitter:true
				});
			} else {
				// Default to "right"
				t.summaryGridPane = new ContentPane({
					region: "right",
					style: "width:400px;display:none;padding: 0px",
					splitter:true
				});
			}
            
            
            t.borderContainer.addChild(t.gridPane);
            t.borderContainer.addChild(t.summaryGridPane);
            
            domConstruct.place(t.grid.domNode,t.gridPane.domNode);
            t.domNode.appendChild(t.borderContainer.domNode);

            var layout = [
                { id : "name", field : "Name", name : "Name" }
            ];

            t.summaryGrid = new Grid({
                cacheClass: Cache,
                store: t.summaryStore,
                structure: layout,
                modules: [
                    Bar,
                    SingleSort,
                    ColumnResizer
                ],
                modelExtensions: [
                    "gridx/core/model/extensions/Sort"
                ],
                barTop: [
                    {content: "<span id='groupHeader'>Summary</span>", className: "emphasize"}
                ]
            });

            t.summaryGrid.connect(t.summaryGrid.body, "onAfterRow", function(row) {
                var item = row.grid.model.byId(row.id).item;
                if (item.Name === "[ALL]") {
                    domClass.add(row.node(),"bold");
                }
            });
            
            t.summaryGridPane.domNode.appendChild(t.summaryGrid.domNode);
            t.summaryGrid.startup();
        },

        setResults: function(results) {
            var t = this;
            t.inherited(arguments);

            var showSummary = true;
            if (results.summarizedData) {
                var columns = [
                    { id : "name", field : "Name", name : "Name" }
                ];

                arrayUtils.forEach(t.controller.summarizationEditor.aggregatesField.get("value"),function(id) {
                    var aggregate = lang.clone(t.controller.summarizationEditor.aggregatesField.store.get(id));
                    if (aggregate.column !== "" && aggregate.aggregateFunction.indexOf("PERCENT") === -1 && aggregate.aggregateFunction.indexOf("COUNT") === -1) {
                        aggregate.formatter = formatter.getForColumn(aggregate,t.controller.report.getColumn(aggregate.column));
                    }
                    columns.push(aggregate);
                });

                t.summaryGrid.setColumns(columns);

                var column = t.controller.report.getColumn(t.controller.summarizationEditor.summarizeByField.get("value"));
                t.summaryGrid.column("name").setName(column.displayName);
                t.summaryGrid.header.refresh();

                t.summaryStore.setData(results.summarizedData);
                t.summaryGrid.model.clearCache();
                t.summaryGrid.body.refresh();
            } else if (t.controller.summarizationEditor.summarizeByField.get("value") === "None") {
                showSummary = false;
            }

            if (showSummary && domStyle.get(t.summaryGridPane.domNode,"display") === "none" || !showSummary && domStyle.get(t.summaryGridPane.domNode,"display") !== "none") {
                domStyle.set(t.summaryGridPane.domNode,"display",(showSummary ? "" : "none"));
                t.borderContainer.resize();
            }
        }
	});
	
	return SummarizableGridView;
});