/*	
	Author: Ryan Witherell
	Created: 2020-10-22
	Purpose: 
	History:
*/
define([
	"dojo/_base/declare",
	"thornberry/report/filter/DateRange",
	"dojo/_base/lang",
	"dojo/date",
	"dojo/_base/array"
], function (declare, DateRange, lang, date, arrayUtils) {
	return declare([DateRange], {
		value: { start: null, end: null, financialPeriod: false },

        getFilterValue: function() {
            // Summary: 
            //      This returns the value to be used by the corresponding column class's GenerateWHERE() method

			if (this.preset.get("value") == "financial") {
				return { start: null, end: null, financialPeriod: true };
			} else {
				return lang.mixin(this.inherited(arguments),{ financialPeriod: false });
			}
        },

        getDisplayValue: function() {
            // Summary: 
            //      This returns the value to be used for display in the generated report

			var filterVal = this.getFilterValue();
            var startFormatted = filterVal.start;
            var endFormatted = filterVal.end;
			var displayVal = "";
			
			var startDate = this.startDate.get("value");
			var endDate = this.endDate.get("value");

            if (startDate !== null && endDate !== null) {
				displayVal = startFormatted + " through " + endFormatted;
            } else if (startDate == null && endDate !== null) {
				displayVal = "on or before " + endFormatted;
            } else if (startDate !== null && endDate == null) {
				displayVal = "on or after " + startFormatted;
            }

            return displayVal;
        },

        serialize: function() {
            // Summary: 
			//      This returns the value to be stored in ^UserSettings

			var value = "";
			var preset = this.preset.get("value");

			if (preset !== "") {
				value = preset;
			} else {
				var startDate = this.startDate.get("value");
				var endDate = this.endDate.get("value");
				var today = new Date();
				today.setHours(0);
				today.setMinutes(0);
				today.setSeconds(0);
				today.setMilliseconds(0);
				
				var startOffset = (startDate !== null ? date.difference(today, startDate, "day") : null);
				var endOffset   = (endDate !== null ? date.difference(today, endDate, "day") : null);

				value = startOffset + "," + endOffset;
			}
			return value;
        },

        deserialize: function(value) {
            // Summary: 
			//      This sets the filter's value from the serialized value
			
			if (arrayUtils.indexOf(["today", "-3days", "-7days", "-14days", "-30days", "-1year", "+3days", "+7days", "+14days", "+30days", "wtd", "mtd", "qtd", "ytd", "lm", "lq", "lytd", "fytd", "financial"], value) !== -1) {
				this.preset.set("value", value);
				this._selectPreset();
			} else {
				this.preset.set("value", "");

				var startOffset = null;
				var endOffset = null;

				if (value !== null) {
					startOffset = value.split(",")[0];
					endOffset = value.split(",")[1];

					startOffset = (startOffset === "" ? null : +startOffset);
					endOffset   = (endOffset   === "" ? null : +endOffset);
				}
				
				this.startDate._onChangeActive = false;
				this.startDate.set("value", (startOffset !== null ? date.add(new Date(), "day", startOffset) : null));
				this.startDate._onChangeActive = true;

				this.endDate._onChangeActive = false;
				this.endDate.set("value", (endOffset !== null ? date.add(new Date(), "day", endOffset) : null));
				this.endDate._onChangeActive = true;

				this.onChange();
				this._updateConstraints();
			}
		},

		buildRendering: function() {
			this.inherited(arguments);
			this.getFiscalYear();

			this.preset.removeOption(this.preset.getOptions());
			var options = [];
			options.push({ label: "(Select)",				value: "" });
			options.push({ label: "Financial Period",		value: "financial" });
			options.push({ label: "Today",					value: "today" });
			options.push({ label: "Past 3 days",			value: "-3days" });
			options.push({ label: "Past 7 days",			value: "-7days" });
			options.push({ label: "Past 14 days",			value: "-14days" });
			options.push({ label: "Past 30 days",			value: "-30days" });
			options.push({ label: "Past year",				value: "-1year" });
			options.push({ label: "Next 3 days",			value: "+3days" });
			options.push({ label: "Next 7 days",			value: "+7days" });
			options.push({ label: "Next 14 days",			value: "+14days" });
			options.push({ label: "Next 30 days",			value: "+30days" });
			options.push({ label: "Week to date",			value: "wtd" });
			options.push({ label: "Month to date",			value: "mtd" });
			options.push({ label: "Quarter to date",		value: "qtd" });
			options.push({ label: "Year to date",			value: "ytd" });
			options.push({ label: "Last month",				value: "lm" });
			options.push({ label: "Last quarter",			value: "lq" });
			options.push({ label: "Fiscal year to date",	value: "fytd" });
			options.push({ label: "Last year to date",		value: "lytd" });

			this.preset.addOption(options);
		},

		onChange: function() {},
		focus: function() {},

		validate: function() {
			return this.startDate.validate() && this.endDate.validate();
		},

		_selectPreset: function() {
			var preset = this.preset.get("value");

			var startDate, endDate;
			var today = new Date();

			this.startDate._onChangeActive = false;
			this.endDate._onChangeActive = false;
			
			this.startDate.set("disabled",preset == "financial");
			this.endDate.set("disabled",preset == "financial");

			if (preset == "financial") {
				this.startDate.set("value", null, false);
				this.endDate.set("value", null, false);
			} else if (preset == "today") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", today, false);
			} else if (preset == "+3days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +3), false);
			} else if (preset == "+7days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +7), false);
			} else if (preset == "+14days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +14), false);
			} else if (preset == "+30days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +30), false);
			} else if (preset == "-3days") {
				this.startDate.set("value", date.add(today, "day", -3), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-7days") {
				this.startDate.set("value", date.add(today, "day", -7), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-14days") {
				this.startDate.set("value", date.add(today, "day", -14), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-30days") {
				this.startDate.set("value", date.add(today, "day", -30), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-1year") {
				this.startDate.set("value", date.add(today, "year", -1), false);
				this.endDate.set("value", today, false);
			} else if (preset == "wtd") {
				var dayOfWeek = today.getDay();
				this.startDate.set("value", date.add(today, "day", -dayOfWeek), false);
				this.endDate.set("value", today, false);
			} else if (preset == "mtd") {
				startDate = new Date(today);
				startDate.setDate(1);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", today, false);
			} else if (preset == "qtd") {
				var currentMonth = today.getMonth();
				var monthOfQuarter = currentMonth%3;
				startDate = new Date(today);
				startDate.setDate(1);
				startDate.setMonth(currentMonth - monthOfQuarter);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", today, false);
			}  else if (preset == "ytd") {
				startDate = new Date(today);
				startDate.setMonth(0);
				startDate.setDate(1);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", today, false);
			} else if (preset == "lm") {
				startDate = date.add(today, "month", -1);
				startDate.setDate(1);
				endDate = new Date(today);
				endDate.setDate(1);
				endDate = date.add(endDate, "day", -1);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", endDate, false);
			} else if (preset == "lq") {
				var currentMonth = today.getMonth();
				var monthOfQuarter = currentMonth%3;
				startDate = new Date(today);
				startDate.setDate(1);
				startDate.setMonth(currentMonth - monthOfQuarter - 3);
				endDate = new Date();
				endDate.setDate(1);
				endDate.setMonth(currentMonth - monthOfQuarter);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", date.add(endDate, "day", -1), false);
			} else if (preset == "lytd") {
				startDate =  new Date(today);
				startDate.setMonth(0);
				startDate.setDate(1);
				this.startDate.set("value", date.add(startDate, "year", -1), false);
				this.endDate.set("value", date.add(today, "year", -1), false);
			} else if (preset == "fytd") {
				this.startDate.set("value", this._fiscalYearStart, false);
				this.endDate.set("value", today, false);
			}

			this.startDate._onChangeActive = true;
			this.endDate._onChangeActive = true;

			this.onChange();
			this._updateConstraints();
		},

		_getValueAttr: function() {
			var value = this.getFilterValue();
			if (value.start === null && value.end === null && !value.financialPeriod) {
				value = null;
			}
			return value;
		},

		_setDisabledAttr: function (/*Boolean*/ value) {
			this.startDate.set("disabled", value);
			this.endDate.set("disabled", value);
			this.preset.set("disabled", value);
		}
    });
});